dnl Option used to select an compiler warnings
dnl -------------------------------------------------
dnl WARNINGS_CHECK ([ACTION-IF-ENABLED], [ACTION-IF-DISABLED])
dnl available values are:
dnl		--enable-warnings
dnl		--disable-warnings (default)
dnl
AC_DEFUN([WARNINGS_CHECK],
[
	AC_MSG_CHECKING(for warnings)
	AC_ARG_ENABLE(warnings,
		AC_HELP_STRING(
			[--enable-warnings],
			[enable compiler warnings (GCC only)]
		),
		[
			if test $enableval = 'yes'; then
				AM_CONDITIONAL(WARNINGS, true)
				CFLAGS="$CFLAGS -Wall"

				ifelse([$1],,:,[$1])

				AC_MSG_RESULT([enabled])
			else
				AM_CONDITIONAL(WARNINGS, false)

				ifelse([$2],,:,[$2])

				AC_MSG_RESULT([disabled])
			fi
		],
		[
			AM_CONDITIONAL(WARNINGS, false)

			ifelse([$2],,:,[$2])

			AC_MSG_RESULT([disabled (default), enable with --enable-warnings])
		]
	)
])

dnl Option used to enable warnings as error
dnl -------------------------------------------------
dnl WERROR_CHECK ([ACTION-IF-ENABLED], [ACTION-IF-DISABLED])
dnl
dnl available values are:
dnl		--enable-werror
dnl		--disable-werror (default)
dnl
AC_DEFUN([WERROR_CHECK],
[
	AC_MSG_CHECKING(for warnings as errors)
	AC_ARG_ENABLE(werror,
		AC_HELP_STRING(
			[--enable-werror],
			[enable warnings as error]
		),
		[
			if test $enableval = 'yes'; then
				AM_CONDITIONAL(WERROR, true)
				CFLAGS="$CFLAGS -Werror"

				ifelse([$1],,:,[$1])

				AC_MSG_RESULT([enabled])
			else
				AM_CONDITIONAL(WERROR, false)

				ifelse([$2],,:,[$2])

				AC_MSG_RESULT([disabled])
			fi
		],
		[
			AM_CONDITIONAL(WERROR, false)

			ifelse([$2],,:,[$2])

			AC_MSG_RESULT([disabled (default), enable with --enable-werror])
		]
	)
])

dnl Option used to enable debugging messages
dnl -------------------------------------------------
dnl DEBUG_CHECK ([ACTION-IF-ENABLED], [ACTION-IF-DISABLED])
dnl
dnl available values are:
dnl		--enable-debug
dnl		--disable-debug (default)
dnl
AC_DEFUN([DEBUG_CHECK],
[
	AC_MSG_CHECKING(for debugging messages)
	AC_ARG_ENABLE(debug,
		AC_HELP_STRING(
			[--enable-debug],
			[Build in debugging symbols and enable debugging messages]
		),
		[
			if test $enableval = 'yes'; then
				AC_DEFINE([DEBUG],[1],[Build in debugging symbols and enable debugging messages])
				AM_CONDITIONAL(DEBUG, true)
				CFLAGS=`echo $CFLAGS | sed -e 's#\-g[0-9]*##g'`
				CFLAGS=`echo $CFLAGS | sed -e 's#\-O[0-9s]*##g'`
				CFLAGS="$CFLAGS -O1 -g3"

				ifelse([$1],,:,[$1])

				AC_MSG_RESULT([enabled])
			else
				AM_CONDITIONAL(DEBUG, false)

				ifelse([$2],,:,[$2])

				AC_MSG_RESULT([disabled])
			fi
		],
		[
			AM_CONDITIONAL(DEBUG, false)

			ifelse([$2],,:,[$2])

			AC_MSG_RESULT([disabled (default), enable with --enable-debug])
		]
	)
])

dnl Option used to enable runtime traces
dnl -------------------------------------------------
dnl TRACE_CHECK ([ACTION-IF-ENABLED], [ACTION-IF-DISABLED])
dnl
dnl available values are:
dnl		--enable-trace
dnl		--disable-trace (default)
dnl
AC_DEFUN([TRACE_CHECK],
[
	AC_MSG_CHECKING(for runtime traces)
	AC_ARG_ENABLE(trace,
		AC_HELP_STRING(
			[--enable-trace],
			[enable runtime traces]
		),
		[
			if test $enableval = 'yes'; then
				AM_CONDITIONAL(ENABLE_TRACE, true)
				AC_DEFINE([ENABLE_TRACE],[1],[enable trace outputs in general])
				AC_DEFINE([ENABLE_LOG],[1],[enable log (errors) outputs in general])

				ifelse([$1],,:,[$1])

				AC_MSG_RESULT([enabled])
			else
				AM_CONDITIONAL(ENABLE_TRACE, false)

				ifelse([$2],,:,[$2])

				AC_MSG_RESULT([disabled])
			fi
		],
		[
			AM_CONDITIONAL(ENABLE_TRACE, false)

			ifelse([$2],,:,[$2])

			AC_MSG_RESULT([disabled (default), enable with --enable-trace])
		]
	)
])


dnl Option used to set additional (device specific) CFLAGS
dnl -------------------------------------------------
dnl
AC_DEFUN([WITH_CFLAGS_CHECK],
[
	AC_MSG_CHECKING(for additional CFLAGS)
	AC_ARG_WITH(cflags,
		AC_HELP_STRING(
			[--with-cflags=val],
			[pass additional (device specific) CFLAGS, not required for Linux 2.6]
		),
		[
			if test "$withval" = yes; then
				AC_MSG_ERROR([Please provide a value for the maximum devices]);
			fi
			CFLAGS="$CFLAGS $withval"
			AC_MSG_RESULT([$withval])
		],
		[
			AC_MSG_RESULT([not set (default), set with --with-cflags])
		]
	)
])

dnl Option used to enable counting of events
dnl -------------------------------------------------
dnl DEBUG_CHECK ([ACTION-IF-ENABLED], [ACTION-IF-DISABLED])
dnl
dnl available values are:
dnl		--enable-event-counter
dnl		--disable-event-counter (default)
dnl
AC_DEFUN([EVENT_COUNTER_CHECK],
[
	AC_MSG_CHECKING(for event counter)
	AC_ARG_ENABLE(event-counter,
		AC_HELP_STRING(
			[--enable-event-counter],
			[Enable counting of events]
		),
		[
			if test $enableval = 'yes'; then
				AC_DEFINE([EVENT_COUNTER],[1],[Enable counting of events])

				ifelse([$1],,:,[$1])

				AC_MSG_RESULT([enabled])
			else
				ifelse([$2],,:,[$2])

				AC_MSG_RESULT([disabled])
			fi
		],
		[
			ifelse([$2],,:,[$2])

			AC_MSG_RESULT([disabled (default), enable with --enable-event-counter])
		]
	)
])
dnl
dnl Option used to enable/disable interrupts
dnl -------------------------------------------------
dnl INTERRUPTS_CHECK([DEFAULT-ACTION], [ACTION-IF-ENABLED], [ACTION-IF-DISABLED])
dnl
dnl available values are:
dnl		--enable-interrupts (default)
dnl		--disable-interrupts
dnl
AC_DEFUN([INTERRUPTS_CHECK],
[
	AC_MSG_CHECKING(for interrupts)
	AC_ARG_ENABLE(interrupts,
		AC_HELP_STRING(
			[--enable-interrupts],
			[Enable/disable interrupt support]
		),
		[__enable_interrupts=$enableval],
		[__enable_interrupts=ifelse([$1],,[yes],[$1])]
	)

	if test "$__enable_interrupts" = "yes" -o "$__enable_interrupts" = "enable"; then
		ifelse([$2],,:,[$2])

		AC_MSG_RESULT([enabled])
	else
		ifelse([$3],,:,[$3])

		AC_MSG_RESULT([disabled])
	fi

	unset __enable_interrupts
])
dnl
dnl Searching for the driver path
dnl -------------------------------------------------
dnl DRV_PATH_CHECK([VAR-NAME], [DRV-NAMES], [BASE-DIRS])
dnl
dnl VAR-NAME 	- [out] variable name to store result
dnl DRV-NAMES	- list of driver names
dnl BASE-DIRS 	- list of base directories
dnl
AC_DEFUN([DRV_PATH_CHECK],
[
	__pwd=$(pwd)

	dnl determine absolute path to the srcdir
	if [[ -z "$__abs_srcdir" ]]; then
		cd "$srcdir"
		__abs_srcdir=$(pwd)
		cd "$__pwd"
	fi

	if test "${cached_found_base_drv_path+set}" != set; then
		__want_base_drv_path="no"
		__found_base_drv_path="no"

		AC_ARG_WITH([drv-incl],
			AC_HELP_STRING(
				[--with-drv-incl@<:@=DIR@:>@],
				[Path to the base directory for driver search includes.]
				[@<:@default=<abs_srcdir>/..@:>@.]
				),
			 [
				__with_base_drv_path=$withval
				__want_base_drv_path="yes"
			 ],
			[__with_base_drv_path="$__abs_srcdir/.."]
		)

		if test -d "$__with_base_drv_path"; then
			__found_base_drv_path="yes"
		else
			if test "x$__want_base_drv_path" == "xyes"; then
				AC_MSG_RESULT([$__with_base_drv_path (no)(intermediate)])
				AC_MSG_ERROR([incorrect basedir passed, please specify correct value using '--with-drv-incl'])
			fi
		fi

		cached_found_base_drv_path=$__found_base_drv_path;
		cached_with_base_drv_path=$__with_base_drv_path;

		unset __found_base_drv_path __with_base_drv_path __want_base_drv_path
	fi

	for __base in $cached_with_base_drv_path $3 $__abs_srcdir; do
		cd "$__pwd"
		cd "$__base" &> /dev/null || continue
		for __name in $2; do
			cd "./$__name" &> /dev/null || continue
			__drv_path=$(pwd)
			break 3
		done
	done

	cd "$__pwd"
	unset __pwd

	if [[ "x$__drv_path" != "x" ]]; then
		$1=$__drv_path
	else
		$1=$__abs_srcdir
	fi

	unset __base __rel __name __drv_path
])
dnl
dnl Searching for correct path of header file
dnl -------------------------------------------------
dnl DRV_HEADER_PATH_CHECK([VAR-NAME], [HEARED-NAME], [SUB-DIRS],[ACTION-IF-FOUND], [ACTION-IF-NOT-FOUND])
dnl
dnl VAR-NAME 		- [in/out] variable name with base directory
dnl HEARED-NAME	- header file name
dnl SUB-DIRS		- list of subdirectory names (default: . include src)
dnl
AC_DEFUN([DRV_HEADER_PATH_CHECK],
[
	__pwd=$(pwd)

	for __rel in ifelse([$3],,. include src,[$3]); do
		cd "$__pwd"
		cd "$$1/$__rel" &> /dev/null || continue
		if test -f "$2"; then
			__hdr_path=$(pwd)
			break
		fi
	done

	cd "$__pwd"

	unset __pwd __rel

	if [[ "x$__hdr_path" != "x" ]]; then
		$1=$__hdr_path

		ifelse([$4],,:,[$4])
	else
		ifelse([$5],,:,[$5])
	fi

	unset __hdr_path
])
dnl
dnl WITH_DRV_INCL_CHECK(
dnl 	[WITH-INCL], [DRV-NAME], [DRV-DIRS], [DRV-HEADER],
dnl 	[DEFAULT-PATH], [ACTION-IF-PRESENT], [ACTION-IF-MISSED]
dnl ----------------------------------------------------------
dnl
dnl Checks for [DRV-NAME]
dnl specify --with-[WITH-INCL]-incl
dnl If not supplied it checks for default and returns error when
dnl the header file was not found.
dnl
AC_DEFUN([WITH_DRV_INCL_CHECK],
[
	AC_MSG_CHECKING(for $2 includes)
	if test "${cached_found_$1_incl+set}" != set; then
		__want_$1_incl="no"
		__found_$1_incl="no"

		DRV_PATH_CHECK([__$1_default_incl],[$3])

		AC_ARG_WITH([$1-incl],
			AC_HELP_STRING(
				[--with-$1-incl@<:@=DIR@:>@],
				[Path to $2 includes.]
				[@<:@default=<basedir>/<drv_name>/include@:>@; ]
				[<drv_name> are list of @<:@$3@:>@; ]
				[<basedir> are configurable with '--with-drv-incl']
				),
			 [
				__with_$1_incl=$withval
				__want_$1_incl="yes"
			 ],
			[__with_$1_incl=ifelse([$5],,[$__$1_default_incl],[$5])]
		)

		DRV_HEADER_PATH_CHECK([__with_$1_incl], [$4],,
			[__found_$1_incl="yes"])

		AC_MSG_RESULT([$__with_$1_incl ($__found_$1_incl)])
		cached_found_$1_incl=$__found_$1_incl;
		cached_want_$1_incl=$__want_$1_incl;
		cached_with_$1_incl=$__with_$1_incl;

		unset __found_$1_incl __with_$1_incl __want_$1_incl
	else
		AC_MSG_RESULT([$cached_with_$1_incl ($cached_found_$1_incl) (cached)])
	fi

	if test "x$cached_found_$1_incl" == "xyes"; then
		ifelse([$6],,[:],[$6])
	else
		__msg="not found, please specify correct value using '--with-$1-incl'"

		if test "x$cached_want_$1_incl" == "xyes"; then
			AC_MSG_ERROR([$__msg])
		fi

		AC_MSG_WARN([$__msg])

		unset __msg

		ifelse([$7],,[:],[$7])
	fi

])dnl


dnl DRV_TAPI_INCL_CHECK_FOR_LL_DRV
dnl ----------------------------------------------------------
dnl
dnl Checks for drv_tapi includes for LL drivers
dnl specify --with-tapi-incl
dnl
AC_DEFUN([DRV_TAPI_INCL_CHECK_FOR_LL_DRV],
[
   AM_CONDITIONAL(HL_TAPI_INCL_PATH_SET, false)
   __tapi_incl_path_set="no"
   dnl set HL TAPI include path
   AC_ARG_ENABLE(tapiincl,
       AC_HELP_STRING(
           [--enable-tapiincl=x],
           [(deprecated) use --with-tapi-incl instead]
       ),
       [
         AC_MSG_WARN([Warning: using deprecated configure switch --enable-tapiincl, use --with-tapi-incl instead])
         __tapi_incl_path=$enableval
         __tapi_incl_path_set="yes"
       ]
   )
   AC_ARG_WITH(tapi-incl,
       AC_HELP_STRING(
           [--with-tapi-incl=x],
           [set the High-Level TAPI include path]
       ),
       [
         __tapi_incl_path=$withval
         __tapi_incl_path_set="yes"
       ]
   )
   
   dnl check if path was specified
   if test "x$__tapi_incl_path_set" == "xyes"; then
      dnl check if path exists
      if test -n $__tapi_incl_path; then
         dnl create an absolute path to be valid also from within the build dir
         if test "`echo $__tapi_incl_path|cut -c1`" != "/" ; then
            __tapi_incl_path=`echo $PWD/$__tapi_incl_path`
         fi
         AC_MSG_RESULT([enable including of TAPI headers from $__tapi_incl_path])
         AC_SUBST([HL_TAPI_INCL_PATH],[$__tapi_incl_path])
         AM_CONDITIONAL(HL_TAPI_INCL_PATH_SET, true)
      else
         AC_MSG_WARN([tapi include path $__tapi_incl_path is invalid])
      fi
   else
      AC_MSG_RESULT([assuming default TAPI include path. Change with --with-tapi-incl=path])
   fi
   
   unset __tapi_incl_path_set __tapi_incl_path
])


dnl DRV_MPS_INCL_CHECK_FOR_LL_DRV
dnl ----------------------------------------------------------
dnl
dnl Checks for mps driver includes
dnl specify --with-mps-incl
dnl
AC_DEFUN([DRV_MPS_INCL_CHECK_FOR_LL_DRV],
[
   AM_CONDITIONAL(MPS_INCL_PATH_SET, false)
   __mps_incl_path_set="no"
   dnl set HL MPS include path
   AC_ARG_ENABLE(mpsincl,
       AC_HELP_STRING(
           [--enable-mpsincl=x],
           [(deprecated) use --with-mps-incl instead]
       ),
       [
         AC_MSG_WARN([Warning: using deprecated configure switch --enable-mpsincl, use --with-mps-incl instead])
         __mps_incl_path=$enableval
         __mps_incl_path_set="yes"
       ]
   )
   AC_ARG_WITH(mps-incl,
       AC_HELP_STRING(
           [--with-mps-incl=x],
           [set the MPS driver include path]
       ),
       [
         __mps_incl_path=$withval
         __mps_incl_path_set="yes"
       ]
   )
   
   if test "x$__mps_incl_path_set" == "xyes"; then

      if test -n $__mps_incl_path; then
         # create an absolute path to be valid also from within the build dir
         if test "`echo $__mps_incl_path|cut -c1`" != "/" ; then
            __mps_incl_path=`echo $PWD/$__mps_incl_path`
         fi
         AC_MSG_RESULT([enable including of MPS headers from $__mps_incl_path])
         AC_SUBST([MPS_INCL_PATH],[$__mps_incl_path])
         AM_CONDITIONAL(MPS_INCL_PATH_SET, true)
      else
         AC_MSG_WARN([MPS include path $__mps_incl_path is invalid])
      fi
   else
      AC_MSG_RESULT([assuming default MPS include path. Change with --with-mps-incl=path])
   fi
   
   unset __mps_incl_path_set __mps_incl_path
])

dnl
dnl Add include/library path with backward compatibility
dnl -------------------------------------------------
dnl TD_DECT_CHECK(
dnl [1-WIITH-CONFIGURE], [2-ENABLE-CONFIGURE], [3-SHORT-NAME]
dnl [4-LIB-INCLUDE], [5-DIR-DEFAULT], [6-DIR-TO-CHECK],
dnl [7-FILES-TO-CHECK], [8-PATH-NAME-TO-SET])
dnl
AC_DEFUN([TD_INCL_CHECK_WITH_BW_COMP],
[
   _$3_path=$5
   _$3_path_was_set="no"
   dnl check outdated 
   AC_ARG_ENABLE($2,
		AS_HELP_STRING(
			[--enable-$2=I/path/to/$3/$4],
			[deprecated option, use --with-$1 instead]
		),
		[
         AC_MSG_WARN([Warning: using deprecated configure switch --enable-$2])
         _$3_path="$enableval"
         _$3_path_was_set="yes"
		]
   )
   dnl but this is the option that should be used
   AC_ARG_WITH($1,
		AC_HELP_STRING(
			[--with-$1=I/path/to/$3/$4],
			[Set the $3 $4 path]
		),
		[
         _$3_path="$withval"
         _$3_path_was_set="yes"
		]
	)
   
   dnl create an absolute path to be valid also from within the build dir
   if test "`echo ${_$3_path}|cut -c1`" != "/" ; then
      _$3_path=`echo $PWD/${_$3_path}`
   fi
   
   dnl validate path
	AC_MSG_CHECKING(for $3 $4)
   _$3_valid_path="yes"
   dnl is dir existing
   if test -d "${_$3_path}/$6"; then
      dnl for all files check if they exist
      for _$3_file in $7; do
         dnl is file existing
         if test ! -r "${_$3_path}/$6/$_$3_file"; then
            dnl no such file
            _$3_valid_path="no"
         fi
      done
   else
      dnl no such directory
      _$3_valid_path="no"
   fi
   
   dnl check if path was found
   if test "x$_$3_valid_path" == "xyes"; then
      dnl it is all OK - set the path
      AC_SUBST([$8],[$_$3_path])
   else
      dnl print error
      if test "x$_$3_path_was_set" == "xyes"; then
         AC_MSG_ERROR([The $3 $4 directory ${_$3_path}$6 is not valid.])
      else
         AC_MSG_ERROR([Please specify the $3 $4 directory path. (--with-$1=x)])
      fi
   fi
	AC_MSG_RESULT([${$8}])
   
   unset _$3_valid_path _$3_path _$3_path_was_set _$3_file
])
