#ifndef _DRV_API_H_TAPI
#define _DRV_API_H_TAPI
/****************************************************************************
                              Copyright (c) 2009
                            Lantiq Deutschland GmbH
                     Am Campeon 3; 85579 Neubiberg, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

 ****************************************************************************/

/**
   \file drv_api.h
   This file contains the defines, the structures declarations
                 the tables declarations and the global functions declarations.

      Use compiler switch ENABLE_TRACE for trace output, for debugging purposes.
      Compiler switch for OS is needed. Use LINUX for linux and VXWORKS for
      VxWorks..
*/

/* ========================================================================== */
/*                                 Includes                                   */
/* ========================================================================== */
#ifdef HAVE_CONFIG_H
#include <drv_config.h>
#endif
#ifdef VXWORKS
#include "prjParams.h"
#endif

#include "drv_tapi_osmap.h"

#ifdef LINUX
   #ifdef __NO_INLINE__
      #error "no inline, maybe use of -O0 ???"
   #endif /* __NO_INLINE__ */
#elif !defined(VXWORKS) && !defined(WINDOWS)
   #error TAPI driver - no OS specified. Please define your operating system!
#endif /* OS */

/* ============================= */
/* Feature declaration           */
/* ============================= */
/* ========================================================================== */
/*                       Global Macro Definitions                             */
/* ========================================================================== */

/*
   list of possible features to check against TAPI_CFG_FEATURES
*/

/* support for Network Linetesting (NLT) tests */
#define TAPI_FEAT_NLT                        0x00000001
/* support for voice connections */
#define TAPI_FEAT_VOICE                      0x00000002
/* support for statistic collection support */
#define TAPI_FEAT_STATISTICS                 0x00000004
/* support for FAX connections */
#define TAPI_FEAT_FAX_T38                    0x00000008
/* support for FAX connections inside firmware */
#define TAPI_FEAT_FAX_STACK                  0x00000010
/* support for announcements inside firmware */
#define TAPI_FEAT_ANNOUNCEMENTS              0x00000020
/* support for peak detector */
#define TAPI_FEAT_PEAKD                      0x00000040
/* support for MF R2 detector */
#define TAPI_FEAT_MF_R2                      0x00000080
/* support for CID features */
#define TAPI_FEAT_CID                        0x00000100
/* support for Analog Line Continuous Measurement */
#define TAPI_FEAT_CONT_MEAS                  0x00000200
/* support for DTMF */
#define TAPI_FEAT_DTMF                       0x00000400
/* support for DECT features */
#define TAPI_FEAT_DECT                       0x00000800
/* support for GR909 tests */
#define TAPI_FEAT_GR909                      0x00001000
/* support for HDLC features */
#define TAPI_FEAT_HDLC                       0x00002000
/* support for Hook state machine */
#define TAPI_FEAT_HOOKSTATE                  0x00004000
/* support for Metering features */
#define TAPI_FEAT_METERING                   0x00008000
/* support for packet handling (AAL/RTP) */
#define TAPI_FEAT_PACKET                     0x00010000
/* support for KPI features */
#define TAPI_FEAT_KPI                        0x00020000
/* support for QOS features */
#define TAPI_FEAT_QOS                        0x00040000
/* support for polling features */
#define TAPI_FEAT_POLL                       0x00080000
/* support for FXO features */
#define TAPI_FEAT_FXO                        0x00100000
/* support for phone detection features */
#define TAPI_FEAT_PHONE_DETECTION            0x00200000

/* \todo: implement as feature
   TAPI_PACKET_OWNID
   TAPI_POWERSAVE
   TAPI_GNDKH_STATEMACHINE
   EVENT_LOGGER_DEBUG
   TAPI_ONE_DEVNODE <- ??????
*/

/* ---------------------- */

#ifdef TAPI_NLT
   #define TAPI_CFG_ADD_FEAT_NLT      TAPI_FEAT_NLT
#else
   #define TAPI_CFG_ADD_FEAT_NLT      0
#endif /* TAPI_NLT */

#ifdef TAPI_VOICE
   #define TAPI_CFG_ADD_FEAT_VOICE   (TAPI_FEAT_VOICE)
#else
   #define TAPI_CFG_ADD_FEAT_VOICE   0
#endif /* TAPI_VOICE */

#ifdef TAPI_STATISTICS
   #define TAPI_CFG_ADD_FEAT_STATISTICS   (TAPI_FEAT_STATISTICS)
#else
   #define TAPI_CFG_ADD_FEAT_STATISTICS   0
#endif /* TAPI_STATISTICS */

#ifdef TAPI_FAX_T38
   #define TAPI_CFG_ADD_FEAT_FAX_T38 (TAPI_FEAT_FAX_T38)
#else
   #define TAPI_CFG_ADD_FEAT_FAX_T38 0
#endif /* TAPI_FAX_T38 */

#ifdef TAPI_FAX_T38_FW
   #define TAPI_CFG_ADD_FEAT_FAX_STACK (TAPI_FEAT_FAX_STACK)
#else
   #define TAPI_CFG_ADD_FEAT_FAX_STACK 0
#endif /* TAPI_FAX_T38_FW */

#ifdef TAPI_ANNOUNCEMENTS
   #define TAPI_CFG_ADD_FEAT_ANNOUNCEMENTS (TAPI_FEAT_ANNOUNCEMENTS)
#else
   #define TAPI_CFG_ADD_FEAT_ANNOUNCEMENTS 0
#endif /* TAPI_ANNOUNCEMENTS */

#ifdef TAPI_PEAKD
   #define TAPI_CFG_ADD_FEAT_PEAKD (TAPI_FEAT_PEAKD)
#else
   #define TAPI_CFG_ADD_FEAT_PEAKD 0
#endif /* TAPI_PEAKD */

#ifdef TAPI_MF_R2
   #define TAPI_CFG_ADD_FEAT_MF_R2 (TAPI_FEAT_MF_R2)
#else
   #define TAPI_CFG_ADD_FEAT_MF_R2 0
#endif /* TAPI_MF_R2 */

#ifdef TAPI_CID
   #define TAPI_CFG_ADD_FEAT_CID (TAPI_FEAT_CID)
#else
   #define TAPI_CFG_ADD_FEAT_CID 0
#endif /* TAPI_CID */

#ifdef TAPI_CONT_MEASUREMENT
   #define TAPI_CFG_ADD_FEAT_CONT_MEAS (TAPI_FEAT_CONT_MEAS)
#else
   #define TAPI_CFG_ADD_FEAT_CONT_MEAS 0
#endif /* TAPI_CONT_MEASUREMENT */

#ifdef TAPI_DTMF
   #define TAPI_CFG_ADD_FEAT_DTMF (TAPI_FEAT_DTMF)
#else
   #define TAPI_CFG_ADD_FEAT_DTMF 0
#endif /* TAPI_DTMF */

#ifdef TAPI_GR909
   #define TAPI_CFG_ADD_FEAT_GR909 (TAPI_FEAT_GR909)
#else
   #define TAPI_CFG_ADD_FEAT_GR909 0
#endif /* TAPI_GR909 */

#ifdef TAPI_HDLC
   #define TAPI_CFG_ADD_FEAT_HDLC (TAPI_FEAT_HDLC)
#else
   #define TAPI_CFG_ADD_FEAT_HDLC 0
#endif /* TAPI_HDLC */

#ifdef TAPI_HOOKSTATE
   #define TAPI_CFG_ADD_FEAT_HOOKSTATE (TAPI_FEAT_HOOKSTATE)
#else
   #define TAPI_CFG_ADD_FEAT_HOOKSTATE 0
#endif /* TAPI_HOOKSTATE */

#ifdef TAPI_METERING
   #define TAPI_CFG_ADD_FEAT_METERING (TAPI_FEAT_METERING)
#else
   #define TAPI_CFG_ADD_FEAT_METERING 0
#endif /* TAPI_METERING */

#ifdef TAPI_PACKET
   #define TAPI_CFG_ADD_FEAT_PACKET (TAPI_FEAT_PACKET)
#else
   #define TAPI_CFG_ADD_FEAT_PACKET 0
#endif /* TAPI_PACKET */

#ifdef KPI_SUPPORT
   #define TAPI_CFG_ADD_FEAT_KPI (TAPI_FEAT_KPI)
#else
   #define TAPI_CFG_ADD_FEAT_KPI 0
#endif /* KPI_SUPPORT */

#ifdef QOS_SUPPORT
   #define TAPI_CFG_ADD_FEAT_QOS (TAPI_FEAT_QOS)
#else
   #define TAPI_CFG_ADD_FEAT_QOS 0
#endif /* QOS_SUPPORT */

#ifdef TAPI_POLL
   #define TAPI_CFG_ADD_FEAT_POLL (TAPI_FEAT_POLL)
#else
   #define TAPI_CFG_ADD_FEAT_POLL 0
#endif /* QOS_SUPPORT */

#ifdef DECT_SUPPORT
   #define TAPI_CFG_ADD_FEAT_DECT (TAPI_FEAT_DECT)
#else
   #define TAPI_CFG_ADD_FEAT_DECT 0
#endif /* DECT_SUPPORT */

#ifdef TAPI_FXO
   #define TAPI_CFG_ADD_FEAT_FXO (TAPI_FEAT_FXO)
#else
   #define TAPI_CFG_ADD_FEAT_FXO 0
#endif /* TAPI_FXO */

#ifdef TAPI_PHONE_DETECTION
   #define TAPI_CFG_ADD_FEAT_PHONE_DETECTION (TAPI_FEAT_PHONE_DETECTION)
#else
   #define TAPI_CFG_ADD_FEAT_PHONE_DETECTION 0
#endif /* DECT_PHONE_DETECTION */

#define TAPI_CFG_FEATURES ( 0             \
      | TAPI_CFG_ADD_FEAT_NLT             \
      | TAPI_CFG_ADD_FEAT_VOICE           \
      | TAPI_CFG_ADD_FEAT_STATISTICS      \
      | TAPI_CFG_ADD_FEAT_FAX_T38         \
      | TAPI_CFG_ADD_FEAT_FAX_STACK       \
      | TAPI_CFG_ADD_FEAT_ANNOUNCEMENTS   \
      | TAPI_CFG_ADD_FEAT_PEAKD           \
      | TAPI_CFG_ADD_FEAT_MF_R2           \
      | TAPI_CFG_ADD_FEAT_CID             \
      | TAPI_CFG_ADD_FEAT_CONT_MEAS       \
      | TAPI_CFG_ADD_FEAT_DTMF            \
      | TAPI_CFG_ADD_FEAT_GR909           \
      | TAPI_CFG_ADD_FEAT_HDLC            \
      | TAPI_CFG_ADD_FEAT_HOOKSTATE       \
      | TAPI_CFG_ADD_FEAT_METERING        \
      | TAPI_CFG_ADD_FEAT_PACKET          \
      | TAPI_CFG_ADD_FEAT_KPI             \
      | TAPI_CFG_ADD_FEAT_QOS             \
      | TAPI_CFG_ADD_FEAT_POLL            \
      | TAPI_CFG_ADD_FEAT_DECT            \
      | TAPI_CFG_ADD_FEAT_FXO             \
      | TAPI_CFG_ADD_FEAT_PHONE_DETECTION \
   )

/* ========================================================================== */
/*                       Global Macro Definitions                             */
/* ========================================================================== */

/* return values */

#ifndef OK
/** return value on success */
#define OK      0
#endif

#ifndef ERROR
/** return value on failure */
#define ERROR  (-1)
#endif

/** mark variable as unused, to suppress compilation warrnings

   \remarks: (ANSI X3.159-1989)
      void is used, in any context where the value of an expression
      is to be discarded, to indicate explicitly that a value is
      ignored by writing the cast (void).
*/
#define IFX_UNUSED(var) ((IFX_void_t)(var))

#ifndef ARRAY_SIZE
   #define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))
#endif /* ARRAY_SIZE */

#endif /* _DRV_API_H_TAPI */
