/******************************************************************************

                               Copyright (c) 2008
                            Infineon Technologies AG
                     Am Campeon 1-12; 81726 Munich, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

******************************************************************************/

/**
   \file drv_tapi_debug.c
   This module provide debug functionality.

   \remarks
      This file contains the automatically generated code, please be careful
      during editing.
*/

/* ========================================================================== */
/*                                 Includes                                   */
/* ========================================================================== */

#include "drv_tapi.h"
#include "drv_tapi_api.h"
#include "drv_tapi_debug.h"
#include "drv_tapi_errno.h"

#ifdef EVENT_LOGGER_DEBUG
   #ifdef LINUX
      #include <asm/ioctl.h>
   #endif
   #ifdef VXWORKS
      #include "ioctl.h"
   #endif
#endif /* EVENT_LOGGER_DEBUG */

#if (defined(IFXOS_USE_DEV_IO) && (IFXOS_USE_DEV_IO == 1))
   #include <ifxos_device_access.h>
   #ifdef LINUX
       #include <asm/ioctl.h> /** \todo Fix it */
   #endif
#endif
/* ========================================================================== */
/*                             Macro definitions                              */
/* ========================================================================== */

#define TAPI_EL_APPEND(fmt, ...) \
      n = snprintf(pOutput + len, (IFX_uint32_t)nOutLen - len, fmt, __VA_ARGS__), \
      len += (n < 0) ? 0 : (IFX_uint32_t)n

#define TAPI_EL_ADD_PADDING(nCnt)  \
   for (k = 0; k < nCnt; k++)        \
   {                                 \
      n = snprintf(pOutput + len, (IFX_uint32_t)nOutLen - len, " "); \
      len += (n < 0) ? 0 : (IFX_uint32_t)n; \
   }

/* ========================================================================== */
/*                             Type definitions                               */
/* ========================================================================== */

typedef struct _IoctlLkupTable
{
   IFX_uint32_t val;
   IFX_char_t name[40];
} IoctlLkupTable_t;

/* ========================================================================== */
/*                             Local variables                                */
/* ========================================================================== */

#define IOCTL_LKUP_TBL_ADD(ioctl) \
   { ioctl, #ioctl }

/**
   Lookup table TAPI ioctl command

   \remarks
      !!!IMPORTANT!!!
      Never touch that data between 'IOCTL_LOOKUP_START'
         and 'IOCTL_LOOKUP_END' and 'IOCTL_QOS_LOOKUP_START'
         and 'IOCTL_QOS_LOOKUP_END' directly.
      Use 'drv_tapi_io.xml' and 'drv_tapi_qos_io.xml'
         with 'make ioctl' instead!
*/
static IoctlLkupTable_t const tapi_ioctl_table[] = {
/* IOCTL_LOOKUP_START */
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_CH_INIT),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_DEV_START),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_DEV_STOP),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_LINE_FEED_SET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_LINE_TYPE_SET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_WLEC_PHONE_CFG_SET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_WLEC_PHONE_CFG_GET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_WLEC_PCM_CFG_SET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_WLEC_PCM_CFG_GET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_LINE_HOOK_VT_SET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_PHONE_VOLUME_SET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_PCM_VOLUME_SET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_PHONE_RELATIVE_LEVEL_SET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_PHONE_RELATIVE_LEVEL_GET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_LINE_LEVEL_SET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_LINE_HOOK_STATUS_GET),
#if (TAPI_CFG_FEATURES & TAPI_FEAT_METERING)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_METER_CFG_SET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_METERING) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_METERING)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_METER_START),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_METERING) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_METERING)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_METER_STOP),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_METERING) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_METERING)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_METER_BURST),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_METERING) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_METERING)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_METER_STATISTICS_GET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_METERING) */
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_TONE_LEVEL_SET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_TONE_TABLE_CFG_SET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_TONE_PLAY),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_TONE_LOCAL_PLAY),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_TONE_NET_PLAY),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_TONE_BUSY_PLAY),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_TONE_RINGBACK_PLAY),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_TONE_DIALTONE_PLAY),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_TONE_LOCAL_STOP),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_TONE_NET_STOP),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_TONE_STOP),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_SIG_DETECT_ENABLE),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_SIG_DETECT_DISABLE),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_DTMF_RX_CFG_SET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_DTMF_RX_CFG_GET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_TONE_CPTD_START),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_TONE_CPTD_STOP),
#if (TAPI_CFG_FEATURES & TAPI_FEAT_CID)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_CID_RX_START),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_CID) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_CID)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_CID_RX_STOP),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_CID) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_CID)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_CID_RX_STATUS_GET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_CID) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_CID)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_CID_RX_DATA_GET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_CID) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_CID)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_CID_TX_INFO_START),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_CID) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_CID)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_CID_TX_INFO_STOP),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_CID) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_CID)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_CID_TX_SEQ_START),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_CID) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_CID)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_CID_CFG_SET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_CID) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_PKT_RTP_CFG_SET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE) */
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_PKT_EV_OOB_DTMF_SET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_PKT_EV_OOB_MFTD_SET),
#if (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_PKT_RTP_PT_CFG_SET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_PKT_NET_CFG_SET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_PKT_RTCP_STATISTICS_GET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_PKT_RTCP_STATISTICS_RESET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE) */
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_PKT_EV_GENERATE),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_PKT_EV_GENERATE_ABCD_CFG),
#if (TAPI_CFG_FEATURES & TAPI_FEAT_PACKET)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_PKT_FLUSH),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_PACKET) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_JB_CFG_SET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_JB_STATISTICS_GET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_JB_STATISTICS_RESET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_MAP_PHONE_ADD),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_MAP_PHONE_REMOVE),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_MAP_DATA_ADD),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_MAP_DATA_REMOVE),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_MAP_PCM_ADD),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_MAP_PCM_REMOVE),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE) && (TAPI_CFG_FEATURES & TAPI_FEAT_DECT)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_MAP_DECT_ADD),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE) && (TAPI_CFG_FEATURES & TAPI_FEAT_DECT) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE) && (TAPI_CFG_FEATURES & TAPI_FEAT_DECT)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_MAP_DECT_REMOVE),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE) && (TAPI_CFG_FEATURES & TAPI_FEAT_DECT) */
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_MAP_MON_ADD),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_MAP_MON_REMOVE),
#if (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_ENC_CFG_SET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_DEC_CFG_SET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_ENC_START),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_ENC_STOP),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_ENC_HOLD),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_DEC_START),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_DEC_STOP),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_VOICE) */
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_COD_DEC_HP_SET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_COD_VOLUME_SET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_ENC_AGC_CFG),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_ENC_AGC_ENABLE),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_ENC_VAD_CFG_SET),
#if (TAPI_CFG_FEATURES & TAPI_FEAT_ANNOUNCEMENTS)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_COD_ANNOUNCE_CFG_SET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_ANNOUNCEMENTS) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_ANNOUNCEMENTS)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_COD_ANNOUNCE_BUFFER_FREE),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_ANNOUNCEMENTS) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_ANNOUNCEMENTS)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_COD_ANNOUNCE_START),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_ANNOUNCEMENTS) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_ANNOUNCEMENTS)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_COD_ANNOUNCE_STOP),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_ANNOUNCEMENTS) */
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_ENC_ROOM_NOISE_DETECT_START),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_ENC_ROOM_NOISE_DETECT_STOP),
#if (TAPI_CFG_FEATURES & TAPI_FEAT_KPI)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_KPI_CH_CFG_SET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_KPI) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_KPI)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_KPI_GRP_CFG_SET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_KPI) */
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_VERSION_GET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_DEBUG_REPORT_SET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_VERSION_CHECK),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_CAP_NR),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_CAP_LIST),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_CAP_CHECK),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_LASTERR),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_RING_CFG_SET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_RING_CADENCE_HR_SET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_RING_CFG_GET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_RING),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_RING_MAX_SET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_RING_CADENCE_SET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_RING_START),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_RING_STOP),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_PCM_IF_CFG_SET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_PCM_CFG_SET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_PCM_CFG_GET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_PCM_ACTIVATION_SET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_PCM_ACTIVATION_GET),
#if (TAPI_CFG_FEATURES & TAPI_FEAT_HDLC)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_PCM_HDLC_CFG_SET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_HDLC) */
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_PCM_LOOP_CFG_SET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_PCM_DEC_HP_SET),
#if (TAPI_CFG_FEATURES & TAPI_FEAT_FAX_STACK)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_T38_SESS_START),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_FAX_STACK) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_FAX_STACK)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_T38_SESS_STOP),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_FAX_STACK) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_FAX_STACK)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_T38_SESS_STATISTICS_GET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_FAX_STACK) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_FAX_STACK)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_T38_CAP_GET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_FAX_STACK) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_FAX_STACK)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_T38_CFG_SET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_FAX_STACK) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_FAX_STACK)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_T38_CFG_GET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_FAX_STACK) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_FAX_STACK)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_T38_FDP_CFG_SET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_FAX_STACK) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_FAX_STACK)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_T38_FDP_CFG_GET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_FAX_STACK) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_FAX_STACK)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_T38_TRACE_SET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_FAX_STACK) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_FAX_T38)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_T38_MOD_START),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_FAX_T38) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_FAX_T38)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_T38_DEMOD_START),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_FAX_T38) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_FAX_T38)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_T38_STOP),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_FAX_T38) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_FAX_T38)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_T38_STATUS_GET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_FAX_T38) */
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_TEST_HOOKGEN),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_TEST_LOOP),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_EVENT_GET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_EVENT_ENABLE),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_EVENT_DISABLE),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_EVENT_MULTI_ENABLE),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_EVENT_MULTI_DISABLE),
#if (TAPI_CFG_FEATURES & TAPI_FEAT_POLL)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_POLL_CONFIG),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_POLL) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_POLL)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_POLL_DEV_ADD),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_POLL) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_POLL)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_POLL_DEV_REM),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_POLL) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_POLL)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_POLL_TEST),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_POLL) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_FXO)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_FXO_DIAL_CFG_SET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_FXO) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_FXO)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_FXO_FLASH_CFG_SET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_FXO) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_FXO)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_FXO_OSI_CFG_SET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_FXO) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_FXO)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_FXO_DIAL_START),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_FXO) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_FXO)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_FXO_DIAL_STOP),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_FXO) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_FXO)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_FXO_HOOK_SET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_FXO) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_FXO)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_FXO_FLASH_SET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_FXO) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_FXO)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_FXO_BAT_GET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_FXO) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_FXO)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_FXO_HOOK_GET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_FXO) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_FXO)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_FXO_APOH_GET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_FXO) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_FXO)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_FXO_RING_GET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_FXO) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_FXO)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_FXO_POLARITY_GET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_FXO) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_FXO)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_FXO_LINE_MODE_SET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_FXO) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_DECT)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_DECT_ACTIVATION_SET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_DECT) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_DECT)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_DECT_CFG_SET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_DECT) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_DECT)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_DECT_ENC_CFG_SET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_DECT) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_DECT)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_DECT_EC_CFG_SET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_DECT) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_DECT)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_DECT_VOLUME_SET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_DECT) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_DECT)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_DECT_STATISTICS_GET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_DECT) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_DECT)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_TONE_DECT_PLAY),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_DECT) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_DECT)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_TONE_DECT_STOP),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_DECT) */
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_MWL_ACTIVATION_SET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_MWL_ACTIVATION_GET),
#if (TAPI_CFG_FEATURES & TAPI_FEAT_PEAKD)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_PEAK_DETECT_START),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_PEAKD) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_PEAKD)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_PEAK_DETECT_STOP),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_PEAKD) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_PEAKD)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_PEAK_DETECT_RESULT_GET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_PEAKD) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_MF_R2)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_TONE_MF_R2_START),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_MF_R2) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_MF_R2)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_TONE_MF_R2_STOP),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_MF_R2) */
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_CALIBRATION_START),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_CALIBRATION_STOP),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_CALIBRATION_RESULTS_GET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_CALIBRATION_CFG_SET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_CALIBRATION_CFG_GET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_COMTEL_START),
#if (TAPI_CFG_FEATURES & TAPI_FEAT_CONT_MEAS)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_CONTMEASUREMENT_REQ),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_CONT_MEAS) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_CONT_MEAS)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_CONTMEASUREMENT_GET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_CONT_MEAS) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_CONT_MEAS)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_CONTMEASUREMENT_RESET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_CONT_MEAS) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_PHONE_DETECTION)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_LINE_MEASURE_CAPACITANCE_START),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_PHONE_DETECTION) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_PHONE_DETECTION)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_LINE_MEASURE_CAPACITANCE_STOP),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_PHONE_DETECTION) */
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_LINE_BATTERY_VOLTAGE_GET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_LINE_BATTERY_VOLTAGE_SET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_NLT_TEST_START),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_NLT_RESULT_GET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_VERSION_DEV_ENTRY_GET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_VERSION_CH_ENTRY_GET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_PKT_RTP_EXT_STATISTICS_GET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_PKT_RTP_EXT_STATISTICS_CFG),
#if (TAPI_CFG_FEATURES & TAPI_FEAT_GR909)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_GR909_START),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_GR909) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_GR909)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_GR909_STOP),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_GR909) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_GR909)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_GR909_RESULT),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_GR909) */
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_COD_MOS_CFG_SET),
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_COD_MOS_RESULT_GET),
#if (TAPI_CFG_FEATURES & TAPI_FEAT_PHONE_DETECTION)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_LINE_PHONE_DETECT_CFG_SET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_PHONE_DETECTION) */
#if (TAPI_CFG_FEATURES & TAPI_FEAT_PHONE_DETECTION)
   IOCTL_LKUP_TBL_ADD (IFX_TAPI_LINE_PHONE_DETECT_CFG_GET),
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_PHONE_DETECTION) */
/* IOCTL_LOOKUP_END */

#if (TAPI_CFG_FEATURES & TAPI_FEAT_QOS)
/* IOCTL_QOS_LOOKUP_START */
   IOCTL_LKUP_TBL_ADD (FIO_QOS_START),
   IOCTL_LKUP_TBL_ADD (FIO_QOS_ON_SOCKET_START),
   IOCTL_LKUP_TBL_ADD (FIO_QOS_STOP),
   IOCTL_LKUP_TBL_ADD (FIO_QOS_CLEAN),
/* IOCTL_QOS_LOOKUP_END */
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_QOS) */
   { 0, "" }
};

/* ========================================================================== */
/*                           Function prototypes                              */
/* ========================================================================== */
#ifdef EVENT_LOGGER_DEBUG
static IFX_int32_t TAPI_EL_format(
      IFX_int32_t nDevType,
      IFX_int32_t nDevNum,
      IFX_int32_t nChNum,
      IFX_char_t* sTime,
      IFX_int32_t nLogType,
      IFX_int32_t nDataSize,
      IFX_void_t* pData,
      IFX_char_t* pOutput,
      IFX_int32_t nOutLen
   );
#endif /* EVENT_LOGGER_DEBUG */

/* ========================================================================== */
/*                         Function implementation                            */
/* ========================================================================== */

/**
   Set the trace level

   \param pDev    reference to device context
   \param nLevel  Minimal level for traced messages

   \return
      TAPI_statusOk - success
      TAPI_statusLLNotSupp - service not supported by low-level driver
      TAPI_statusLLFailed - failed low-level call
*/
IFX_int32_t TAPI_DebugReportSet (TAPI_DEV *pDev, IFX_uint32_t nLevel)
{
   IFX_UNUSED (pDev);

   SetTraceLevel (TAPI_DRV, nLevel);

   return TAPI_statusOk;
}

/**
   TAPI syslog initialization

   \param pDev  - reference to device context

   \return none
*/
/*lint -save -esym(715, pDev) */
IFX_void_t TAPI_LogInit (TAPI_DEV *pDev)
{
#ifdef EVENT_LOGGER_DEBUG
   EL_REG_Register (DRV_TAPI_NAME, IFX_TAPI_DEV_TYPE_NONE,
      pDev->nDevID /* dev num */, TAPI_EL_format /* cb func */);
#if 0
   {
      IFX_char_t buf[20];

      /* Workaround. For Danube "ifx_tapi_open" isn't called on startup. Thence
       * when "echo "*"  > /proc/driver/el/logs/all" is called in, for example,
       * custom-user-startup script, driver hasn't been registered yet and after
       * registration here log for TAPI is disabled. It should be enabled.
       * It is also impossible to register TAPI driver for event logger on startup,
       * because number of devices is unknown. */
      sprintf (buf, "%s:%d", DRV_TAPI_NAME, pDev->nDevID);
      EL_CFG_SetupLogging(EL_LOG_TYPE_ALL, buf);
   }
#endif /* 0 */
#endif /* EVENT_LOGGER_DEBUG */
}

/**
   TAPI syslog uninitialization

   \param pDev  - reference to device context

   \return none
*/
IFX_void_t TAPI_LogClose (TAPI_DEV *pDev)
{
#ifdef EVENT_LOGGER_DEBUG
   EL_REG_Unregister (DRV_TAPI_NAME, IFX_TAPI_DEV_TYPE_NONE,
      pDev->nDevID /* dev num */);
#endif /* EVENT_LOGGER_DEBUG */
}
/*lint -restore */

/*
   Retrieve named IOCTL representatuion

   \param nIoctl  TAPI IOCTL number

   \return
      Handle to the zero terminated string

   \remarks
      Do not require memory freeing.
      Do not thread-safe realization for unknown ioctls.
*/
IFX_char_t const *TAPI_ioctlNameGet(IFX_uint32_t nIoctl)
{
   static IFX_char_t pBuf[0x11];
   IoctlLkupTable_t const *pIoctl = tapi_ioctl_table;

   while (pIoctl->val != 0 && pIoctl->val != nIoctl)
      pIoctl++;

   if (pIoctl->val != 0)
      return pIoctl->name;

   /* \note: Used local static buffer to skip memory management code.
      That is not thread-safe code, but used because of it is executed in
      rarely cases, like for unsupported or low-level services.
      Feel free to rewrite that implementation if it is raise some problems.
    */
   snprintf (pBuf, sizeof (pBuf), "0x%08X", nIoctl);

   return pBuf;
}

#ifdef EVENT_LOGGER_DEBUG
/**
   Provide ioctl data formatting

   \param nLogType - type of the log
   \param nDataSize - size of the data passed to this function
   \param pData - pointer to the data block
   \param pOutput - formatted log in text format, used for the output
   \param nOutLen - available length of the buffer

   \return Callback status, look at enum EL_CB_Status_t and
           pOutput variable can contain valid log as the null terminated string
           for the output
   \remarks
      pData format:
         [data1 size][data1][data2 size][data2]...[dataN size][dataN]
*/
static IFX_int32_t TAPI_EL_IOCTL_format(
      IFX_int32_t nLogType,
      IFX_uint32_t nDataSize,
      IFX_void_t* pData,
      IFX_char_t* pOutput,
      IFX_uint32_t nOutLen
   )
{
   IFX_uint32_t nIoctl = 0, nErr = 0;
   IFX_uint32_t i = 0, len = 0;
   IFX_int32_t n = 0;
   IFX_uint16_t val_len = 0;
   IFX_boolean_t bContinued = IFX_FALSE;

   if ((pOutput == IFX_NULL) || (nOutLen == 0))
   {
      return EL_CB_StatusErr;
   }

   /* Get length of the ioctl data */
   if (nDataSize > 6U)
      val_len = ((IFX_uint16_t*)pData)[3];
   else
      val_len = 0;

   /* last 4 bytes contain return value (error/success) */
   for (i = nDataSize - 4U; i < nDataSize; i++)
      nErr = (nErr << 8) | ((IFX_uint8_t*)pData)[i];

   if (nLogType == EL_LOG_TYPE_IOCTL_RD)
   {
      if (nErr != 0)
         TAPI_EL_APPEND (" < ERROR [%08X]", nErr);
      else if (val_len == 0)
         TAPI_EL_APPEND ("%s", " >");
      else
         TAPI_EL_APPEND ("%s", " <");
   }

   memcpy (&nIoctl, ((IFX_uint8_t*)pData) + 2, sizeof (IFX_uint32_t));

   TAPI_EL_APPEND (" (%s)", TAPI_ioctlNameGet (nIoctl));

   TAPI_EL_APPEND (" %08X", nIoctl);

   if ((val_len > TAPI_EL_IOCTL_MAX_LEN) || (val_len >= (nOutLen - len)))
   {
      val_len = min ((IFX_uint32_t)TAPI_EL_IOCTL_MAX_LEN, nOutLen - (len + 10U));/*lint !e155 */
      bContinued = IFX_TRUE;
   }

   for (i = 0; i < val_len; i++)
   {
      TAPI_EL_APPEND ("%s%02X",
         ((i & 0x3) == 0x0) ? " " : "",
         ((IFX_uint8_t*)pData)[8 + i]);
   }

   if (bContinued == IFX_TRUE)
   {
      TAPI_EL_APPEND ("%s", " ...");
   }

   return EL_CB_StatusOutput;
}

/**
   Test callback driver function

   \param nDevType - type of the device
   \param nDevNum - device number
   \param nChNum - channel number
   \param sTime - time in text format
   \param nLogType - type of the log
   \param nDataSize - size of the data passed to this function
   \param pData - pointer to the data block
   \param pOutput - formatted log in text format, used for the output
   \param nOutLen - available length of the buffer

   \return Callback status, look at enum EL_CB_Status_t and
           pOutput variable can contain valid log as the null terminated string
           for the output
*/
#ifdef TAPI_VERSION3
static IFX_int32_t TAPI_EL_format(
      IFX_int32_t nDevType,
      IFX_int32_t nDevNum,
      IFX_int32_t nChNum,
      IFX_char_t* sTime,
      IFX_int32_t nLogType,
      IFX_int32_t nDataSize,
      IFX_void_t* pData,
      IFX_char_t* pOutput,
      IFX_int32_t nOutLen
   )
{
   IFX_uint32_t len = 0;
   IFX_int32_t n = 0;
   IFX_int32_t k;

   TAPI_ASSERT (sTime != IFX_NULL);
   TAPI_ASSERT (pData != IFX_NULL);
   TAPI_ASSERT (pOutput != IFX_NULL);
   TAPI_ASSERT (nOutLen > 0);

   IFX_UNUSED (nDevType);

   if ((pOutput == IFX_NULL) || (nOutLen <= 0))
   {
      return EL_CB_StatusErr;
   }

   /* Low Level drivers used in CPE do not have callback functions for log
      formating. Spaces are added to have similar log formating like in 
      low level drivers. */
   switch (nLogType)
   {
      case EL_LOG_TYPE_IOCTL_WR:
         TAPI_EL_APPEND ("%s", "[iw]");
         TAPI_EL_ADD_PADDING(2);
         break;
      case EL_LOG_TYPE_IOCTL_RD:
         TAPI_EL_APPEND ("%s", "[ir]");
         TAPI_EL_ADD_PADDING(2);
         break;
      default:
         return EL_CB_StatusFormat;
   }
   TAPI_EL_APPEND ("%s", sTime );

   if (strlen(DRV_TAPI_NAME) < 10)
       TAPI_EL_ADD_PADDING((10 - strlen(DRV_TAPI_NAME)));
   TAPI_EL_APPEND ("%s", DRV_TAPI_NAME);
   TAPI_EL_ADD_PADDING(7);
   TAPI_EL_APPEND ("%02d", nDevNum);
   TAPI_EL_ADD_PADDING(2);
   TAPI_EL_APPEND ("%02d", nChNum);

   return TAPI_EL_IOCTL_format (nLogType, (IFX_uint32_t)nDataSize, pData,
      pOutput + len, (IFX_uint32_t)nOutLen - len);
}
#else /* TAPI_VERSION3 */
static IFX_int32_t TAPI_EL_format(
      IFX_int32_t nDevType,
      IFX_int32_t nDevNum,
      IFX_int32_t nChNum,
      IFX_char_t* sTime,
      IFX_int32_t nLogType,
      IFX_int32_t nDataSize,
      IFX_void_t* pData,
      IFX_char_t* pOutput,
      IFX_int32_t nOutLen
   )
{
   IFX_uint32_t len = 0;
   IFX_int32_t n = 0;

   TAPI_ASSERT (sTime != IFX_NULL);
   TAPI_ASSERT (pData != IFX_NULL);
   TAPI_ASSERT (pOutput != IFX_NULL);
   TAPI_ASSERT (nOutLen > 0);

   IFX_UNUSED (nDevType);

   if ((pOutput == IFX_NULL) || (nOutLen <= 0))
   {
      return EL_CB_StatusErr;
   }

   TAPI_EL_APPEND ("%s %s ", sTime, DRV_TAPI_NAME);

   switch (nLogType)
   {
      case EL_LOG_TYPE_IOCTL_WR:
         TAPI_EL_APPEND ("%s", "[iw]");
         break;
      case EL_LOG_TYPE_IOCTL_RD:
         TAPI_EL_APPEND ("%s", "[ir]");
         break;
      default:
         return EL_CB_StatusFormat;
   }

   TAPI_EL_APPEND (" %d %d", nDevNum, nChNum);

   return TAPI_EL_IOCTL_format (nLogType, (IFX_uint32_t)nDataSize, pData,
      pOutput + len, (IFX_uint32_t)nOutLen - len);
}
#endif /* TAPI_VERSION3 */
#endif /* EVENT_LOGGER_DEBUG */

/* ========================================================================== */
/*                         Function pointer exports                           */
/* ========================================================================== */
