/******************************************************************************

                              Copyright (c) 2009
                            Lantiq Deutschland GmbH
                     Am Campeon 3; 85579 Neubiberg, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

******************************************************************************/

/**
   \file drv_tapi_event.c
   Contains TAPI Event Handling.
*/

/* ============================= */
/* Includes                      */
/* ============================= */

#include "drv_tapi.h"
#include "drv_tapi_errno.h"
#include "drv_tapi_event.h"
#include "drv_tapi_cid.h"

/*lint -save
  -esym(529,lock) -esym(754,reserved) -esym(754, status) */

/* ============================= */
/* Local Macros  Definitions    */
/* ============================= */

/* ============================= */
/* Global variable definition    */
/* ============================= */

/* ============================= */
/* Global functions declaration  */
/* ============================= */

/* ============================= */
/* Type declarations             */
/* ============================= */

/* These structures are used for masking events. */
typedef struct
{
   IFX_uint32_t reserved:21;
   IFX_uint32_t comtel_end:1;
   IFX_uint32_t linemode:1;
   IFX_uint32_t flash:1;
   IFX_uint32_t offhook:1;
   IFX_uint32_t onhook:1;
   IFX_uint32_t ringing_end:1;
   IFX_uint32_t ringburst_end:1;
   IFX_uint32_t ring:1;
   IFX_uint32_t contMeas:1;
   IFX_uint32_t rawoffhook:1;
   IFX_uint32_t rawonhook:1;
} IFX_TAPI_EVENT_FXS_BITS_t;

typedef union
{
   IFX_TAPI_EVENT_FXS_BITS_t bits;
   IFX_uint32_t status;
} IFX_TAPI_EVENT_FXS_DIS_t;

typedef struct
{
   IFX_uint32_t reserved:24;
   IFX_uint32_t bat_feeded:1;
   IFX_uint32_t bat_dropped:1;
   IFX_uint32_t polarity:1;
   IFX_uint32_t ring_start:1;
   IFX_uint32_t ring_stop:1;
   IFX_uint32_t osi:1;
   IFX_uint32_t apoh:1;
   IFX_uint32_t nopoh:1;
} IFX_TAPI_EVENT_FXO_BITS_t;

typedef union
{
   IFX_TAPI_EVENT_FXO_BITS_t bits;
   IFX_uint32_t status;
} IFX_TAPI_EVENT_FXO_DIS_t;

typedef struct
{
   IFX_uint32_t reserved:30;
   IFX_uint32_t gr909_rdy:1;
   IFX_uint32_t nlt_end:1;
} IFX_TAPI_EVENT_LT_BITS_t;


typedef union
{
   IFX_TAPI_EVENT_LT_BITS_t bits;
   IFX_uint32_t status;
} IFX_TAPI_EVENT_LT_DIS_t;


typedef struct
{
   IFX_uint32_t reserved:30;
   IFX_uint32_t pulseStart:1;
   IFX_uint32_t digit:1;
} IFX_TAPI_EVENT_PULSE_BITS_t;

typedef union
{
   IFX_TAPI_EVENT_PULSE_BITS_t bits;
   IFX_uint32_t status;
} IFX_TAPI_EVENT_PULSE_DIS_t;

typedef struct
{
   IFX_uint32_t reserved:28;
   IFX_uint32_t end_internal:1;
   IFX_uint32_t end_external:1;
   IFX_uint32_t digit_internal:1;
   IFX_uint32_t digit_external:1;
} IFX_TAPI_EVENT_DTMF_BITS_t;

typedef union
{
   IFX_TAPI_EVENT_DTMF_BITS_t bits;
   IFX_uint32_t status;
} IFX_TAPI_EVENT_DTMF_DIS_t;

typedef struct
{
   IFX_uint32_t reserved:22;
   IFX_uint32_t rx_err_read:1;
   IFX_uint32_t rx_cd:1;
   IFX_uint32_t rx_end:1;
   IFX_uint32_t sm_end:1;
   IFX_uint32_t tx_ringcad_err:1;
   IFX_uint32_t tx_noack_err:1;
   IFX_uint32_t tx_info_end:1;
   IFX_uint32_t tx_info_start:1;
   IFX_uint32_t tx_seq_end:1;
   IFX_uint32_t tx_seq_start:1;
} IFX_TAPI_EVENT_CID_BITS_t;

typedef union
{
   IFX_TAPI_EVENT_CID_BITS_t bits;
   IFX_uint32_t status;
} IFX_TAPI_EVENT_CID_DIS_t;

typedef struct
{
   IFX_uint32_t reserved:29;
   IFX_uint32_t end_internal:1;
   IFX_uint32_t end_external:1;
   IFX_uint32_t busy:1;
} IFX_TAPI_EVENT_TONE_GEN_BITS_t;

typedef union
{
   IFX_TAPI_EVENT_TONE_GEN_BITS_t bits;
   IFX_uint32_t status;
} IFX_TAPI_EVENT_TONE_GEN_DIS_t;

typedef struct
{
   IFX_uint32_t reserved:26;
   IFX_uint32_t cpt_end_external:1;
   IFX_uint32_t cpt_end_internal:1;
   IFX_uint32_t receive:1;
   IFX_uint32_t transmit:1;
   IFX_uint32_t cpt_external:1;
   IFX_uint32_t cpt_internal:1;
} IFX_TAPI_EVENT_TONE_DET_BITS_t;

typedef union
{
   IFX_TAPI_EVENT_TONE_DET_BITS_t bits;
   IFX_uint32_t status;
} IFX_TAPI_EVENT_TONE_DET_DIS_t;

typedef struct
{
   IFX_uint32_t reserved:30;
   IFX_uint32_t v21h_internal:1;
   IFX_uint32_t v21h_external:1;
   IFX_uint32_t cas_internal:1;
   IFX_uint32_t cas_external:1;
   IFX_uint32_t hold_internal:1;
   IFX_uint32_t hold_external:1;
   IFX_uint32_t v8bis_internal:1;
   IFX_uint32_t v8bis_external:1;
   IFX_uint32_t v32ac_internal:1;
   IFX_uint32_t v32ac_external:1;
   IFX_uint32_t v22orbell_internal:1;
   IFX_uint32_t v22orbell_external:1;
   IFX_uint32_t v22_internal:1;
   IFX_uint32_t v22_external:1;
   IFX_uint32_t bell_internal:1;
   IFX_uint32_t bell_external:1;
   IFX_uint32_t v27_internal:1;
   IFX_uint32_t v27_external:1;
   IFX_uint32_t v18a_internal:1;
   IFX_uint32_t v18a_external:1;
   IFX_uint32_t v21l_internal:1;
   IFX_uint32_t v21l_external:1;
   IFX_uint32_t cngmod_internal:1;
   IFX_uint32_t cngmod_external:1;
   IFX_uint32_t cngfax_internal:1;
   IFX_uint32_t cngfax_external:1;
   IFX_uint32_t am_internal:1;
   IFX_uint32_t am_external:1;
   IFX_uint32_t pr_internal:1;
   IFX_uint32_t pr_external:1;
   IFX_uint32_t ced_internal:1;
   IFX_uint32_t ced_external:1;
   IFX_uint32_t dis_internal:1;
   IFX_uint32_t dis_external:1;
} IFX_TAPI_EVENT_FAX_SIG_BITS_t;

typedef union
{
   IFX_TAPI_EVENT_FAX_SIG_BITS_t bits;
   IFX_uint32_t status[2];
} IFX_TAPI_EVENT_FAX_SIG_DIS_t;

typedef struct
{
   IFX_uint32_t reserved:28;
   IFX_uint32_t room_noise:1;
   IFX_uint32_t room_silence:1;
   IFX_uint32_t dec_chg:1;
   IFX_uint32_t announce:1;
   IFX_uint32_t mos:1;
} IFX_TAPI_EVENT_CODER_BITS_t;

typedef union
{
   IFX_TAPI_EVENT_CODER_BITS_t bits;
   IFX_uint32_t status;
} IFX_TAPI_EVENT_CODER_DIS_t;

typedef struct
{
   IFX_uint32_t reserved:29;
   IFX_uint32_t first:1;
   IFX_uint32_t broken:1;
   IFX_uint32_t changed:1;
} IFX_TAPI_EVENT_RTP_BITS_t;

typedef union
{
   IFX_TAPI_EVENT_RTP_BITS_t bits;
   IFX_uint32_t status;
} IFX_TAPI_EVENT_RTP_DIS_t;

typedef struct
{
   IFX_uint32_t reserved:32;
} IFX_TAPI_EVENT_AAL_BITS_t;

/*lint -save -esym(754,bits) -esym(754, status) */
/* for future use */
typedef union
{
   IFX_TAPI_EVENT_AAL_BITS_t bits;
   IFX_uint32_t status;
} IFX_TAPI_EVENT_AAL_DIS_t;
/*lint -restore */

typedef struct
{
   IFX_uint32_t reserved:30;
   IFX_uint32_t event:1;
} IFX_TAPI_EVENT_RFC2833_BITS_t;

typedef union
{
   IFX_TAPI_EVENT_RFC2833_BITS_t bits;
   IFX_uint32_t status;
} IFX_TAPI_EVENT_RFC2833_DIS_t;

typedef struct
{
   IFX_uint32_t reserved:31;
   IFX_uint32_t ingress_fifo_full:1;
} IFX_TAPI_EVENT_KPI_BITS_t;

typedef union
{
   IFX_TAPI_EVENT_KPI_BITS_t bits;
   IFX_uint32_t status;
} IFX_TAPI_EVENT_KPI_DIS_t;

typedef struct
{
   IFX_uint32_t reserved:27;
   IFX_uint32_t error_setup:1;
   IFX_uint32_t error_data:1;
   IFX_uint32_t error_write:1;
   IFX_uint32_t error_read:1;
   IFX_uint32_t error_ovld:1;
} IFX_TAPI_EVENT_T38_BITS_t;

typedef union
{
   IFX_TAPI_EVENT_T38_BITS_t bits;
   IFX_uint32_t status;
} IFX_TAPI_EVENT_T38_DIS_t;

typedef struct
{
   IFX_uint32_t reserved:32;
} IFX_TAPI_EVENT_DOWNLOAD_BITS_t;

typedef union
{
   IFX_TAPI_EVENT_DOWNLOAD_BITS_t bits;
   IFX_uint32_t status;
} IFX_TAPI_EVENT_DOWNLOAD_DIS_t;

typedef struct
{
   IFX_uint32_t mbx_congestion:1;
   IFX_uint32_t reserved:31;
} IFX_TAPI_EVENT_INFORMATION_BITS_t;

typedef union
{
   IFX_TAPI_EVENT_INFORMATION_BITS_t bits;
   IFX_uint32_t status;
} IFX_TAPI_EVENT_INFO_DIS_t;

typedef struct
{
   IFX_uint32_t reserved:29;
   IFX_uint32_t cerr:1;
   IFX_uint32_t gpiohl:1;
   IFX_uint32_t gpiolh:1;
} IFX_TAPI_EVENT_DEBUG_BITS_t;

typedef union
{
   IFX_TAPI_EVENT_DEBUG_BITS_t bits;
   IFX_uint32_t status;
} IFX_TAPI_EVENT_DEBUG_DIS_t;

typedef struct
{
   IFX_uint32_t reserved:31;
   IFX_uint32_t alive:1;
} IFX_TAPI_EVENT_LL_DRIVER_BITS_t;

typedef union
{
   IFX_TAPI_EVENT_LL_DRIVER_BITS_t bits;
   IFX_uint32_t status;
} IFX_TAPI_EVENT_LL_DRIVER_DIS_t;

typedef struct
{
   IFX_uint32_t reserved:32;
} IFX_TAPI_EVENT_FAULT_GENERAL_BITS_t;

typedef union
{
   IFX_TAPI_EVENT_FAULT_GENERAL_BITS_t bits;
   IFX_uint32_t status;
} IFX_TAPI_EVENT_FAULT_GEN_DIS_t;

typedef struct
{
   IFX_uint32_t reserved:24;
   IFX_uint32_t overtemp_end:1;
   IFX_uint32_t gk_high_end:1;
   IFX_uint32_t gk_low_end:1;
   IFX_uint32_t overtemp:1;
   IFX_uint32_t gk_high:1;
   IFX_uint32_t gk_low:1;
   IFX_uint32_t gk_neg:1;
   IFX_uint32_t gk_pos:1;
} IFX_TAPI_EVENT_FAULT_LINE_BITS_t;

typedef union
{
   IFX_TAPI_EVENT_FAULT_LINE_BITS_t bits;
   IFX_uint32_t status;
} IFX_TAPI_EVENT_FAULT_LINE_DIS_t;

/*lint -save -esym(754,spi_access) */
typedef struct
{
   IFX_uint32_t reserved:25;
   IFX_uint32_t ssi_err : 1;
   IFX_uint32_t hw_reset:1;
   IFX_uint32_t sync:1;
   /* for future use */
   IFX_uint32_t spi_access:1;
   IFX_uint32_t clock_fail:1;
   IFX_uint32_t clock_fail_end:1;
   IFX_uint32_t hw_fault:1;
} IFX_TAPI_EVENT_FAULT_HW_BITS_t;
/*lint -restore */

typedef union
{
   IFX_TAPI_EVENT_FAULT_HW_BITS_t bits;
   IFX_uint32_t status;
} IFX_TAPI_EVENT_FAULT_HW_DIS_t;

typedef struct
{
   IFX_uint32_t reserved:32;
} IFX_TAPI_EVENT_FAULT_FW_BITS_t;

typedef union
{
   IFX_TAPI_EVENT_FAULT_FW_BITS_t bits;
   IFX_uint32_t status;
} IFX_TAPI_EVENT_FAULT_FW_DIS_t;

typedef struct
{
   IFX_uint32_t reserved:32;
} IFX_TAPI_EVENT_FAULT_SW_BITS_t;

/*lint -save -esym(754,bits) -esym(754, status) */
/* for future use */
typedef union
{
   IFX_TAPI_EVENT_FAULT_SW_BITS_t bits;
   IFX_uint32_t status;
} IFX_TAPI_EVENT_FAULT_SW_DIS_t;
/*lint -restore */

typedef struct
{
   IFX_uint32_t reserved:31;
   IFX_uint32_t buf_underrun:1;
} IFX_TAPI_EVENT_LIN_BITS_t;

typedef union
{
   IFX_TAPI_EVENT_LIN_BITS_t bits;
   IFX_uint32_t status;
} IFX_TAPI_EVENT_LIN_DIS_t;

typedef struct
{
   IFX_uint32_t metering_end:1;
   IFX_uint32_t reserved:31;
}IFX_TAPI_EVENT_METERING_BITS_t;

typedef union
{
   IFX_TAPI_EVENT_METERING_BITS_t bits;
   IFX_uint32_t status;
}IFX_TAPI_EVENT_METERING_DIS_t;

typedef struct
{
   IFX_uint32_t reserved:31;
   IFX_uint32_t calibration_end:1;
} IFX_TAPI_EVENT_CALIBRATION_BITS_t;

typedef union
{
   IFX_TAPI_EVENT_CALIBRATION_BITS_t bits;
   IFX_uint32_t status;
} IFX_TAPI_EVENT_CALIBRATION_DIS_t;

typedef struct
{
   IFX_uint32_t reserved:31;
   IFX_uint32_t lcap_disable:1;
} IFX_TAPI_EVENT_LCAP_BITS_t;

typedef union
{
   IFX_TAPI_EVENT_LCAP_BITS_t bits;
   IFX_uint32_t status;
} IFX_TAPI_EVENT_LCAP_DIS_t;

typedef struct
{
   IFX_uint32_t reserved:30;
   IFX_uint32_t end:1;
   IFX_uint32_t start:1;
} IFX_TAPI_EVENT_MF_R2_BITS_t;

typedef union
{
   IFX_TAPI_EVENT_MF_R2_BITS_t bits;
   IFX_uint32_t status;
} IFX_TAPI_EVENT_MF_R2_DIS_t;

/*lint -save
   -esym(754,aal) -esym(754,download) -esym(754,fault_general)
   -esym(754,fault_fw) -esym(754,fault_sw) */
typedef struct
{
   IFX_TAPI_EVENT_FXS_DIS_t fxs;
   IFX_TAPI_EVENT_FXO_DIS_t fxo;
   IFX_TAPI_EVENT_LT_DIS_t lt;
   IFX_TAPI_EVENT_PULSE_DIS_t pulse;
   /* Event mask for all possible modules where DTMF
      detector can be enabled. */
   IFX_TAPI_EVENT_DTMF_DIS_t dtmf[IFX_TAPI_MODULE_TYPE_DECT+1];
   IFX_TAPI_EVENT_CID_DIS_t cid;
   IFX_TAPI_EVENT_TONE_GEN_DIS_t tone_gen;
   IFX_TAPI_EVENT_TONE_DET_DIS_t tone_det[IFX_TAPI_MODULE_TYPE_DECT+1];
   IFX_TAPI_EVENT_FAX_SIG_DIS_t fax_sig[IFX_TAPI_MODULE_TYPE_DECT+1];
   IFX_TAPI_EVENT_CODER_DIS_t coder;
   IFX_TAPI_EVENT_RTP_DIS_t rtp;
   /* for future use */
   IFX_TAPI_EVENT_AAL_DIS_t aal;
   IFX_TAPI_EVENT_RFC2833_DIS_t rfc2833;
   IFX_TAPI_EVENT_KPI_DIS_t kpi;
   IFX_TAPI_EVENT_T38_DIS_t t38;
   /* for future use */
   IFX_TAPI_EVENT_DOWNLOAD_DIS_t download;
   IFX_TAPI_EVENT_INFO_DIS_t information;
   IFX_TAPI_EVENT_DEBUG_DIS_t debug;
   IFX_TAPI_EVENT_LL_DRIVER_DIS_t ll_driver;
   /* for future use */
   IFX_TAPI_EVENT_FAULT_GEN_DIS_t fault_general;
   IFX_TAPI_EVENT_FAULT_LINE_DIS_t fault_line;
   IFX_TAPI_EVENT_FAULT_HW_DIS_t fault_hw;
   /* for future use */
   IFX_TAPI_EVENT_FAULT_FW_DIS_t fault_fw;
   /* for future use */
   IFX_TAPI_EVENT_FAULT_SW_DIS_t fault_sw;
   IFX_TAPI_EVENT_LIN_DIS_t lin;
   IFX_TAPI_EVENT_METERING_DIS_t metering;
   IFX_TAPI_EVENT_CALIBRATION_DIS_t calibration;
   IFX_TAPI_EVENT_LCAP_DIS_t lcap;
   IFX_TAPI_EVENT_MF_R2_DIS_t mf_r2[IFX_TAPI_MODULE_TYPE_DECT + 1];
} IFX_TAPI_EVENT_MASK_t;
/*lint -restore */

struct IFX_TAPI_EVENT_HANDLER_DATA
{
   /* Tapi event fifo -- High proity event */
   FIFO_ID              *pTapiEventFifoHi;
   /* Tapi event fifo -- Low proity event */
   FIFO_ID              *pTapiEventFifoLo;
   /* Mask for Tapi events (enable/disable single events) */
   IFX_TAPI_EVENT_MASK_t eventMask;
   /* Fifo concurent access protection mutex. */
   TAPI_OS_mutex_t       fifoAcc;
};


/* ============================= */
/* Local variable definition     */
/* ============================= */

/** Memory pool for event dispatcher (global for all channels) */
static BUFFERPOOL *pIFX_TAPI_BP_Deferred_Event = IFX_NULL;
/* Buffer pool access protection */
static TAPI_OS_mutex_t semBufferPoolAcc;


/* ============================= */
/* Local function declaration    */
/* ============================= */

static IFX_boolean_t tapi_FaxModem_Status_Check (
   TAPI_CHANNEL *pChannel,
   IFX_TAPI_EVENT_t *pEvent);

static IFX_int32_t tapi_FaxModem_EventConvert (
   IFX_TAPI_EVENT_t const *pEvent,
   IFX_TAPI_SIG_DETECTION_t *pResult);

static IFX_void_t tapi_WakeUp (TAPI_DEV *pTapiDev);
static IFX_void_t tapi_DiscardEvent (IFX_void_t *pEvent);

static IFX_int32_t tapi_EventFifoGet (
   TAPI_CHANNEL *pChannel,
   IFX_TAPI_EVENT_t *pTapiEvent);

static IFX_int32_t tapi_PhoneEventGet (
   TAPI_DEV *pTapiDev,
   IFX_TAPI_EVENT_t *pEvent);

static IFX_int32_t tapi_EventStateSet (
   IFX_TAPI_DRV_CTX_t *pDrvCtx,
   IFX_TAPI_EVENT_t *pEvent,
   IFX_uint32_t nEvtState);

/* ============================= */
/* Local function definition     */
/* ============================= */

/* ============================= */
/* Global function definition    */
/* ============================= */

/**
   Enable or disable an event on the specified channel(s).

   \param  pChannel     Pointer to TAPI_CHANNEL structure.
   \param  pEvent       The event which mask is to be set.
   \param  value        -IFX_EVENT_ENABLE to allow the event to the application
                        -IFX_EVENT_DISABLE to discard the event

   \return
   TAPI_statusOk - success
   TAPI_statusEvtNoHandle - event not known
*/
/**\todo Map the signal detection events to the low level signal
enable */
/**\todo group FAXMODE and make use of a common union member:
         if (TAPI_COD_EXTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[pEvent->module].bits.dis_external = value;
         if (TAPI_COD_INTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[pEvent->module].bits.dis_internal = value;
*/
static IFX_int32_t IFX_TAPI_Event_SetMask (TAPI_CHANNEL *pChannel,
                                    IFX_TAPI_EVENT_t *pEvent,
                                    IFX_uint32_t value)
{
   IFX_int32_t ret = TAPI_statusOk,
               retLL = TAPI_statusOk;
   /* set mask for event filtering */
   IFX_TAPI_DRV_CTX_t* pDrvCtx = pChannel->pTapiDevice->pDevDrvCtx;
   IFX_TAPI_EVENT_MASK_t *pEventMask, eventMaskBak;
   IFX_TAPI_MODULE_TYPE_t nModule;

   TAPI_ASSERT (pChannel);

   if (IFX_NULL == pChannel->pEventHandler)
      RETURN_STATUS (TAPI_statusEvtNoHandle, 0);

   /* set mask for event filtering */
   pEventMask = &(pChannel->pEventHandler->eventMask);

   /* make the restore point on failure */
   memcpy (&eventMaskBak, pEventMask, sizeof (*pEventMask));

#ifdef TAPI_VERSION3
   nModule = IFX_TAPI_MODULE_TYPE_COD;
#else
   nModule = pEvent->module;
#endif

   switch(pEvent->id)
   {
      case IFX_TAPI_EVENT_FXS_RING:
         pEventMask->fxs.bits.ring = value;
         break;
      case IFX_TAPI_EVENT_FXS_RINGBURST_END:
         pEventMask->fxs.bits.ringburst_end = value;
         break;
      case IFX_TAPI_EVENT_FXS_RINGING_END:
         pEventMask->fxs.bits.ringing_end = value;
         break;
      case IFX_TAPI_EVENT_FXS_ONHOOK:
         pEventMask->fxs.bits.onhook = value;
         break;
      case IFX_TAPI_EVENT_FXS_OFFHOOK:
         pEventMask->fxs.bits.offhook = value;
         break;
      case IFX_TAPI_EVENT_FXS_FLASH:
         pEventMask->fxs.bits.flash = value;
         break;
      case IFX_TAPI_EVENT_FXS_RAW_ONHOOK:
         pEventMask->fxs.bits.rawonhook = value;
         break;
      case IFX_TAPI_EVENT_FXS_RAW_OFFHOOK:
         pEventMask->fxs.bits.rawoffhook = value;
         break;
      case IFX_TAPI_EVENT_CONTMEASUREMENT:
         pEventMask->fxs.bits.contMeas = value;
         break;
      case IFX_TAPI_EVENT_FXS_LINE_MODE:
         pEventMask->fxs.bits.linemode = value;
         break;
      case IFX_TAPI_EVENT_FXS_COMTEL_END:
         pEventMask->fxs.bits.comtel_end = value;
         break;
      case IFX_TAPI_EVENT_FXO_RING_START:
         pEventMask->fxo.bits.ring_start = value;
         break;
      case IFX_TAPI_EVENT_FXO_RING_STOP:
         pEventMask->fxo.bits.ring_stop = value;
         break;
      case IFX_TAPI_EVENT_FXO_POLARITY:
         pEventMask->fxo.bits.polarity = value;
         break;
      case IFX_TAPI_EVENT_FXO_BAT_FEEDED:
         pEventMask->fxo.bits.bat_feeded = value;
         break;
      case IFX_TAPI_EVENT_FXO_BAT_DROPPED:
         pEventMask->fxo.bits.bat_dropped = value;
         break;
      case IFX_TAPI_EVENT_FXO_OSI:
         pEventMask->fxo.bits.osi = value;
         break;
      case IFX_TAPI_EVENT_FXO_APOH:
         pEventMask->fxo.bits.apoh = value;
         break;
      case IFX_TAPI_EVENT_FXO_NOPOH:
         pEventMask->fxo.bits.nopoh = value;
         break;
      case IFX_TAPI_EVENT_LT_GR909_RDY:
         pEventMask->lt.bits.gr909_rdy = value;
         break;
      case IFX_TAPI_EVENT_PULSE_DIGIT:
         pEventMask->pulse.bits.digit = value;
         break;
      case IFX_TAPI_EVENT_PULSE_START:
         pEventMask->pulse.bits.pulseStart = value;
         break;
      case IFX_TAPI_EVENT_DTMF_DIGIT:
         if (TAPI_COD_EXTERNAL (pEvent->data.dtmf))
            pEventMask->dtmf[nModule].bits.digit_external = value;

         if (TAPI_COD_INTERNAL (pEvent->data.dtmf))
            pEventMask->dtmf[nModule].bits.digit_internal = value;
         break;
      case IFX_TAPI_EVENT_DTMF_END:
         if (TAPI_COD_EXTERNAL (pEvent->data.dtmf))
            pEventMask->dtmf[nModule].bits.end_external = value;

         if (TAPI_COD_INTERNAL (pEvent->data.dtmf))
            pEventMask->dtmf[nModule].bits.end_internal = value;
         break;
      case IFX_TAPI_EVENT_CID_TX_SEQ_START:
         pEventMask->cid.bits.tx_seq_start = value;
         break;
      case IFX_TAPI_EVENT_CID_TX_SEQ_END:
         pEventMask->cid.bits.tx_seq_end = value;
         break;
      case IFX_TAPI_EVENT_CID_TX_INFO_START:
         pEventMask->cid.bits.tx_info_start = value;
         break;
      case IFX_TAPI_EVENT_CID_TX_INFO_END:
         pEventMask->cid.bits.tx_info_end = value;
         break;
      case IFX_TAPI_EVENT_CIDSM_END:
         pEventMask->cid.bits.sm_end = value;
         break;
      case IFX_TAPI_EVENT_CID_RX_END:
         pEventMask->cid.bits.rx_end = value;
         break;
      case IFX_TAPI_EVENT_CID_RX_CD:
         pEventMask->cid.bits.rx_cd = value;
         break;
      case IFX_TAPI_EVENT_CID_RX_ERROR_READ:
         pEventMask->cid.bits.rx_err_read = value;
         break;
      case IFX_TAPI_EVENT_CID_TX_RINGCAD_ERR:
         pEventMask->cid.bits.tx_ringcad_err = value;
         break;
      case IFX_TAPI_EVENT_CID_TX_NOACK_ERR:
      case IFX_TAPI_EVENT_CID_TX_NOACK2_ERR:
         pEventMask->cid.bits.tx_noack_err = value;
         break;
      case IFX_TAPI_EVENT_TONE_GEN_BUSY:
         pEventMask->tone_gen.bits.busy = value;
         break;
      case IFX_TAPI_EVENT_TONE_GEN_END:
         if ( TAPI_COD_EXTERNAL (pEvent->data.tone_gen))
            pEventMask->tone_gen.bits.end_external = value;
         if ( TAPI_COD_INTERNAL (pEvent->data.tone_gen))
            pEventMask->tone_gen.bits.end_internal = value;
         break;
      case IFX_TAPI_EVENT_TONE_DET_RECEIVE:
         pEventMask->tone_det[nModule].bits.receive = value;
         break;
      case IFX_TAPI_EVENT_TONE_DET_TRANSMIT:
         pEventMask->tone_det[nModule].bits.transmit = value;
         break;
      case IFX_TAPI_EVENT_TONE_DET_CPT:
#ifdef TAPI_VERSION3
         /* \note: always use all directions for CPE */
         pEvent->data.tone_det.external = IFX_TRUE;
         pEvent->data.tone_det.internal = IFX_TRUE;
#endif /* TAPI_VERSION3 */
         if (pEvent->data.tone_det.external)
            pEventMask->tone_det[nModule].bits.cpt_external = value;
         if (pEvent->data.tone_det.internal)
            pEventMask->tone_det[nModule].bits.cpt_internal = value;
         break;
      case IFX_TAPI_EVENT_TONE_DET_CPT_END:
#ifdef TAPI_VERSION3
         /* \note: always use all directions for CPE */
         pEvent->data.tone_det.external = IFX_TRUE;
         pEvent->data.tone_det.internal = IFX_TRUE;
#endif /* TAPI_VERSION3 */
         if (pEvent->data.tone_det.external)
            pEventMask->tone_det[nModule].bits.cpt_end_external = value;
         if (pEvent->data.tone_det.internal)
            pEventMask->tone_det[nModule].bits.cpt_end_internal = value;
         break;
      case IFX_TAPI_EVENT_TONE_DET_MF_R2_START:
         pEventMask->mf_r2[nModule].bits.start = value;
         break;
      case IFX_TAPI_EVENT_TONE_DET_MF_R2_END:
         pEventMask->mf_r2[nModule].bits.end = value;
         break;
      case IFX_TAPI_EVENT_FAXMODEM_DIS:
         if (TAPI_COD_EXTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[nModule].bits.dis_external = value;
         if (TAPI_COD_INTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[nModule].bits.dis_internal = value;
         break;
      case IFX_TAPI_EVENT_FAXMODEM_CED:
         if (TAPI_COD_EXTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[nModule].bits.ced_external = value;
         if (TAPI_COD_INTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[nModule].bits.ced_internal = value;
         break;
      case IFX_TAPI_EVENT_FAXMODEM_PR:
         if (TAPI_COD_EXTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[nModule].bits.pr_external = value;
         if (TAPI_COD_INTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[nModule].bits.pr_internal = value;
         break;
      case IFX_TAPI_EVENT_FAXMODEM_AM:
         if (TAPI_COD_EXTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[nModule].bits.am_external = value;
         if (TAPI_COD_INTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[nModule].bits.am_internal = value;
         break;
      case IFX_TAPI_EVENT_FAXMODEM_CNGFAX:
         if (TAPI_COD_EXTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[nModule].bits.cngfax_external = value;
         if (TAPI_COD_INTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[nModule].bits.cngfax_internal = value;
         break;
      case IFX_TAPI_EVENT_FAXMODEM_CNGMOD:
         if (TAPI_COD_EXTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[nModule].bits.cngmod_external = value;
         if (TAPI_COD_INTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[nModule].bits.cngmod_internal = value;
         break;
      case IFX_TAPI_EVENT_FAXMODEM_V21L:
         if (TAPI_COD_EXTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[nModule].bits.v21l_external = value;
         if (TAPI_COD_INTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[nModule].bits.v21l_internal = value;
         break;
      case IFX_TAPI_EVENT_FAXMODEM_V18A:
         if (TAPI_COD_EXTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[nModule].bits.v18a_external = value;
         if (TAPI_COD_INTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[nModule].bits.v18a_internal = value;
         break;
      case IFX_TAPI_EVENT_FAXMODEM_V27:
         if (TAPI_COD_EXTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[nModule].bits.v27_external = value;
         if (TAPI_COD_INTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[nModule].bits.v27_internal = value;
         break;
      case IFX_TAPI_EVENT_FAXMODEM_BELL:
         if (TAPI_COD_EXTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[nModule].bits.bell_external = value;
         if (TAPI_COD_INTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[nModule].bits.bell_internal = value;
         break;
      case IFX_TAPI_EVENT_FAXMODEM_V22:
         if (TAPI_COD_EXTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[nModule].bits.v22_external = value;
         if (TAPI_COD_INTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[nModule].bits.v22_internal = value;
         break;
      case IFX_TAPI_EVENT_FAXMODEM_V22ORBELL:
         if (TAPI_COD_EXTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[nModule].bits.v22orbell_external = value;
         if (TAPI_COD_INTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[nModule].bits.v22orbell_internal = value;
         break;
      case IFX_TAPI_EVENT_FAXMODEM_V32AC:
         if (TAPI_COD_EXTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[nModule].bits.v32ac_external = value;
         if (TAPI_COD_INTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[nModule].bits.v32ac_internal = value;
         break;
      case IFX_TAPI_EVENT_FAXMODEM_CAS_BELL:
         if (TAPI_COD_EXTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[nModule].bits.cas_external = value;
         if (TAPI_COD_INTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[nModule].bits.cas_internal = value;
         break;
      case IFX_TAPI_EVENT_FAXMODEM_V21H:
         if (TAPI_COD_EXTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[nModule].bits.v21h_external = value;
         if (TAPI_COD_INTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[nModule].bits.v21h_internal = value;
         break;
      case IFX_TAPI_EVENT_FAXMODEM_V8BIS:
         if (TAPI_COD_EXTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[nModule].bits.v8bis_external = value;
         if (TAPI_COD_INTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[nModule].bits.v8bis_internal = value;
         break;
      case IFX_TAPI_EVENT_FAXMODEM_HOLDEND:
         if (TAPI_COD_EXTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[nModule].bits.hold_external = value;
         if (TAPI_COD_INTERNAL (pEvent->data.fax_sig))
            pEventMask->fax_sig[nModule].bits.hold_internal = value;
         break;
      case IFX_TAPI_EVENT_RFC2833_EVENT:
         pEventMask->rfc2833.bits.event = value;
         break;
      case IFX_TAPI_EVENT_KPI_INGRESS_FIFO_FULL:
         pEventMask->kpi.bits.ingress_fifo_full = value;
         break;
      case IFX_TAPI_EVENT_LL_DRIVER_WD_FAIL:
         pEventMask->ll_driver.bits.alive = value;
         break;
      case IFX_TAPI_EVENT_FAULT_LINE_GK_POS:
         pEventMask->fault_line.bits.gk_pos = value;
         break;
      case IFX_TAPI_EVENT_FAULT_LINE_GK_NEG:
         pEventMask->fault_line.bits.gk_neg = value;
         break;
      case IFX_TAPI_EVENT_FAULT_LINE_GK_LOW:
         pEventMask->fault_line.bits.gk_low = value;
         break;
      case IFX_TAPI_EVENT_FAULT_LINE_GK_LOW_END:
         pEventMask->fault_line.bits.gk_low_end = value;
         break;
      case IFX_TAPI_EVENT_FAULT_LINE_GK_HIGH:
         pEventMask->fault_line.bits.gk_high = value;
         break;
      case IFX_TAPI_EVENT_FAULT_LINE_GK_HIGH_END:
         pEventMask->fault_line.bits.gk_high_end = value;
         break;
      case IFX_TAPI_EVENT_FAULT_LINE_OVERTEMP:
         pEventMask->fault_line.bits.overtemp = value;
         break;
      case IFX_TAPI_EVENT_FAULT_LINE_OVERTEMP_END:
         pEventMask->fault_line.bits.overtemp_end = value;
         break;
      case IFX_TAPI_EVENT_FAULT_HW_CLOCK_FAIL:
         pEventMask->fault_hw.bits.clock_fail = value;
         break;
      case IFX_TAPI_EVENT_FAULT_HW_CLOCK_FAIL_END:
         pEventMask->fault_hw.bits.clock_fail_end = value;
         break;
      case IFX_TAPI_EVENT_FAULT_HW_FAULT:
         pEventMask->fault_hw.bits.hw_fault = value;
         break;
      case IFX_TAPI_EVENT_FAULT_HW_SYNC:
         pEventMask->fault_hw.bits.sync = value;
         break;
      case IFX_TAPI_EVENT_FAULT_HW_RESET:
         pEventMask->fault_hw.bits.hw_reset = value;
         break;
      case IFX_TAPI_EVENT_FAULT_HW_SSI_ERR:
         pEventMask->fault_hw.bits.ssi_err = value;
         break;
      case IFX_TAPI_EVENT_COD_DEC_CHG:
         pEventMask->coder.bits.dec_chg = value;
         break;
      case IFX_TAPI_EVENT_COD_ROOM_NOISE:
         pEventMask->coder.bits.room_noise = value;
         break;
      case IFX_TAPI_EVENT_COD_ROOM_SILENCE:
         pEventMask->coder.bits.room_silence = value;
         break;
      case IFX_TAPI_EVENT_COD_ANNOUNCE_END:
         pEventMask->coder.bits.announce = value;
         break;
      case IFX_TAPI_EVENT_COD_MOS:
         pEventMask->coder.bits.mos = value;
         break;
      case IFX_TAPI_EVENT_INFO_MBX_CONGESTION:
         pEventMask->information.bits.mbx_congestion = value;
         break;
      case IFX_TAPI_EVENT_LIN_UNDERFLOW:
         pEventMask->lin.bits.buf_underrun = value;
         break;
      case IFX_TAPI_EVENT_METERING_END:
         pEventMask->metering.bits.metering_end = value;
         break;
      case IFX_TAPI_EVENT_CALIBRATION_END:
         pEventMask->calibration.bits.calibration_end = value;
         break;
      case IFX_TAPI_EVENT_LINE_MEASURE_CAPACITANCE_RDY:
         pEventMask->lcap.bits.lcap_disable = value;
         break;
      case IFX_TAPI_EVENT_NLT_END:
         pEventMask->lt.bits.nlt_end = value;
         break;
      case IFX_TAPI_EVENT_RTP_FIRST:
         pEventMask->rtp.bits.first = value;
         break;
      case IFX_TAPI_EVENT_RTP_EXT_BROKEN:
         pEventMask->rtp.bits.broken = value;
         break;
      case IFX_TAPI_EVENT_RTP_EXT_SSRC_CHANGED:
         pEventMask->rtp.bits.changed = value;
         break;
#if (TAPI_CFG_FEATURES & TAPI_FEAT_FAX_T38)
      case IFX_TAPI_EVENT_T38_ERROR_SETUP:
         pEventMask->t38.bits.error_setup = value;
         break;
      case IFX_TAPI_EVENT_T38_ERROR_DATA:
         pEventMask->t38.bits.error_data = value;
         break;
      case IFX_TAPI_EVENT_T38_ERROR_WRITE:
         pEventMask->t38.bits.error_write = value;
         break;
      case IFX_TAPI_EVENT_T38_ERROR_READ:
         pEventMask->t38.bits.error_read = value;
         break;
      case IFX_TAPI_EVENT_T38_ERROR_OVLD:
         pEventMask->t38.bits.error_ovld = value;
         break;
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_FAX_T38) */
      case IFX_TAPI_EVENT_DEBUG_CERR:
         pEventMask->debug.bits.cerr = value;
         break;
      case IFX_TAPI_EVENT_GPIO_HL:
         pEventMask->debug.bits.gpiohl = value;
         break;
      case IFX_TAPI_EVENT_GPIO_LH:
         pEventMask->debug.bits.gpiolh = value;
         break;
      default:
         RETURN_STATUS (TAPI_statusEvtNoHandle, 0);
   } /* switch(pEvent->id) */

   switch (((IFX_uint32_t)pEvent->id) & IFX_TAPI_EVENT_TYPE_MASK)
   {
      case IFX_TAPI_EVENT_TYPE_FAXMODEM_SIGNAL:
      {
         IFX_TAPI_SIG_DETECTION_t SigDetEvent;

         memset (&SigDetEvent, 0, sizeof(IFX_TAPI_SIG_DETECTION_t));

         ret = tapi_FaxModem_EventConvert (pEvent, &SigDetEvent);
         if (TAPI_SUCCESS (ret))
         {
            if (value == IFX_EVENT_ENABLE)
            {
               if (pDrvCtx->SIG.MFTD_Enable)
                  ret = pDrvCtx->SIG.MFTD_Enable (pChannel->pLLChannel,
                                             &SigDetEvent);
            }
            else
            {
               if (pDrvCtx->SIG.MFTD_Disable)
                  ret = pDrvCtx->SIG.MFTD_Disable (pChannel->pLLChannel,
                                                &SigDetEvent);
            }
         }
         break;
      }
      default:
         /* skip other types */
         break;
   }

   /* call LL-driver to enable/disable event generation */
   switch(pEvent->id)
   {
#if (TAPI_CFG_FEATURES & TAPI_FEAT_DTMF)
      case IFX_TAPI_EVENT_DTMF_DIGIT:
      case IFX_TAPI_EVENT_DTMF_END:
         if (IFX_TAPI_PtrChk (pDrvCtx->SIG.DTMFD_Enable) &&
             IFX_TAPI_PtrChk (pDrvCtx->SIG.DTMFD_Disable) &&
             (pChannel->nChannel < pChannel->pTapiDevice->nResource.DTMFRCount))
         {
            IFX_TAPI_EVENT_DTMF_BITS_t *pBits = &pEventMask->dtmf[nModule].bits;
            IFX_TAPI_LL_DTMFD_CFG_t cfgDtmf;

            /* Detection of DTMF end requires that the DTMF digit detection
               is either disabled or enabled in the same direction. */
            if (((pBits->digit_external == IFX_EVENT_ENABLE) ||
                 (pBits->digit_internal == IFX_EVENT_ENABLE)) &&
                ( ((pBits->end_external == IFX_EVENT_ENABLE) &&
                   (pBits->digit_external != IFX_EVENT_ENABLE)) ||
                  ((pBits->end_internal == IFX_EVENT_ENABLE) &&
                   (pBits->digit_internal != IFX_EVENT_ENABLE)) ) )
            {
               /*errmsg: Combination of directions for DTMF digit and DTMF end
                         detection is not possible. */
               ret = TAPI_statusDtmfRxDirectionCombination;
               break;
            }

            memset (&cfgDtmf, 0, sizeof (cfgDtmf));

            cfgDtmf.nMod = nModule;
            cfgDtmf.direction = IFX_TAPI_LL_DTMFD_DIR_NONE;
            cfgDtmf.bEndEvent = IFX_FALSE;

            if ((pBits->end_external == IFX_EVENT_ENABLE) ||
                (pBits->end_internal == IFX_EVENT_ENABLE))
            {
               cfgDtmf.bEndEvent = IFX_TRUE;
            }

            /* direction of the DTMF receiver */
            if ((pBits->digit_external == IFX_EVENT_ENABLE) &&
                (pBits->digit_internal == IFX_EVENT_ENABLE))
               cfgDtmf.direction = IFX_TAPI_LL_DTMFD_DIR_BOTH;
            else
            if ((pBits->digit_external == IFX_EVENT_ENABLE))
               cfgDtmf.direction = IFX_TAPI_LL_DTMFD_DIR_EXTERNAL;
            else
            if ((pBits->digit_internal == IFX_EVENT_ENABLE))
               cfgDtmf.direction = IFX_TAPI_LL_DTMFD_DIR_INTERNAL;

            /* still no direction? - end event will determine the direction */
            if (cfgDtmf.direction == IFX_TAPI_LL_DTMFD_DIR_NONE)
            {
               if ((pBits->end_external == IFX_EVENT_ENABLE) &&
                   (pBits->end_internal == IFX_EVENT_ENABLE))
                  cfgDtmf.direction = IFX_TAPI_LL_DTMFD_DIR_BOTH;
               else
               if ((pBits->end_external == IFX_EVENT_ENABLE))
                  cfgDtmf.direction = IFX_TAPI_LL_DTMFD_DIR_EXTERNAL;
               else
               if ((pBits->end_internal == IFX_EVENT_ENABLE))
                  cfgDtmf.direction = IFX_TAPI_LL_DTMFD_DIR_INTERNAL;
            }

            /* still no direction? - disable the DTMF receiver */
            if (cfgDtmf.direction != IFX_TAPI_LL_DTMFD_DIR_NONE)
            {
               retLL = pDrvCtx->SIG.DTMFD_Enable (pChannel->pLLChannel,
                                                  &cfgDtmf);
            }
            else
            {
               retLL = pDrvCtx->SIG.DTMFD_Disable (pChannel->pLLChannel,
                                                   &cfgDtmf);
            }
         }
         break;
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_DTMF) */
      case IFX_TAPI_EVENT_FXS_RAW_ONHOOK:
      case IFX_TAPI_EVENT_FXS_RAW_OFFHOOK:
         if (IFX_TAPI_PtrChk (pDrvCtx->ALM.RawHookCtrl))
         {
            if ((pEventMask->fxs.bits.rawonhook == IFX_EVENT_ENABLE) ||
                (pEventMask->fxs.bits.rawoffhook == IFX_EVENT_ENABLE))
            {
               retLL = pDrvCtx->ALM.RawHookCtrl (pChannel->pLLChannel, IFX_TRUE);
            }
            else
            if ((pEventMask->fxs.bits.rawonhook == IFX_EVENT_DISABLE) &&
                (pEventMask->fxs.bits.rawoffhook == IFX_EVENT_DISABLE))
            {
               retLL = pDrvCtx->ALM.RawHookCtrl (pChannel->pLLChannel, IFX_FALSE);
            }
         }
         else
         {
            RETURN_STATUS (TAPI_statusEvtNoHandle, 0);
         }
         break;
      default :
         /* nothing to do */
         break;
   }

   if (!TAPI_SUCCESS(retLL))
   {
      /* The LL reported an error so add an generic HL errorcode. */
      /* errmsg: LL driver returned an error */
      ret = TAPI_statusLLFailed;
   }

   if (!TAPI_SUCCESS (ret))
   {
      /* restore previous configuration on failure */
      memcpy (pEventMask, &eventMaskBak, sizeof (*pEventMask));
   }

   RETURN_STATUS (ret, retLL);
}


/**
   Reports the event information for a channel

   \param  pTapiDev     Pointer to TAPI device structure.
   \param  pEvent       Pointer to a event structure. Must not be NULL.

   \return
   TAPI_statusOk - success
   TAPI_statusParam - channel out of range
*/
static IFX_int32_t tapi_PhoneEventGet (
   TAPI_DEV *pTapiDev,
   IFX_TAPI_EVENT_t *pEvent)
{
   static IFX_uint8_t last_ch_nr = 0;  /* remember channel between calls */
   IFX_int16_t found_ch_nr;
   IFX_uint8_t i;
   IFX_int32_t moreEvents = 0;
#ifdef TAPI_SINGLE_THREAD
   IFX_TAPI_DRV_CTX_t       *pDrvCtx;

   pDrvCtx = pTapiDev->pDevDrvCtx;
   if (pDrvCtx != IFX_NULL && IFX_TAPI_PtrChk (pDrvCtx->POLL.pollEvents))
   {
      pDrvCtx->POLL.pollEvents((IFX_TAPI_LL_DEV_t *) pTapiDev->pLLDev);
   }
#endif /* TAPI_LIBRARY */
   if (pEvent->ch == IFX_TAPI_EVENT_ALL_CHANNELS)
   {
      /* Scan every channel to see if any of them has an event waiting.
         If an event is found return it to the application. So each call
         returns one event at one time. */

      /* If this device has less channels than the device in the last call
         where an event was found on a channel not present on this device
         reset the starting value to zero. */
      if (last_ch_nr >= pTapiDev->nMaxChannel)
      {
         last_ch_nr = 0;
      }

      /* We start scanning with the next channel after the last channel
         processed during the last call. We are called at least one time.
         Even if there is only one channel the loop will be aborted because
         of the wrap-around correction. The exit from the loop is done only
         with the break at the end of the loop. */
      for (i = last_ch_nr+1, found_ch_nr = -1 ; /* no condition */;i++)
      {
         i = ((i >= pTapiDev->nMaxChannel) ? 0 : i);

         /* As long as we have not found any event we check all the channels
            for events. As soon as we got an event we still check if there
            are any more events in any of the remaining channels. */
         if (found_ch_nr < 0)
         {
            /* check this channel for events */
            moreEvents = tapi_EventFifoGet (pTapiDev->pTapiChanelArray + i,
                                               pEvent);
            if (pEvent->id != IFX_TAPI_EVENT_NONE)
            {
               found_ch_nr = i;
            }
         }
         else
         {
            /* check only if events present but do not get them */
            moreEvents += (IFX_TAPI_EventFifoEmpty(
               pTapiDev->pTapiChanelArray + i) == 0) ? 1 : 0;
         }

         /* abort loop if we either wrapped around or found that there are
            more than just one event which implies that we already got one */
         if ((i == last_ch_nr) || (moreEvents != 0))
            break;
      }

      /* remember the last channel number that was serviced */
      last_ch_nr = found_ch_nr < 0 ? i : found_ch_nr;
   }
   else /* Scanning event by channel index. */
   {
      if (pEvent->ch >= pTapiDev->nMaxChannel)
         RETURN_DEVSTATUS (TAPI_statusParam, 0);
      /* Still get one event at one time. */
      moreEvents = tapi_EventFifoGet (pTapiDev->pTapiChanelArray + pEvent->ch,
                                         pEvent);
   }

   /* Set the more events flag inside the event message. The flag indicates
      if there are any more events in the channel that was given as a parameter.
      So for specific channels it returns if there are more events in the
      specific channel. For IFX_TAPI_EVENT_ALL_CHANNELS it returns if there
      are any more events in any of the channels. */
   pEvent->more = moreEvents;

   return TAPI_statusOk;
}


/**
   Returns the enable status of the fax modem event.

   \param pChannel - handle to tapi channel structure
   \param pEvent   - handle to event

   \return
      IFX_TRUE if the event is enabled, otherwise IFX_FALSE

   \remarks
*/
static IFX_boolean_t tapi_FaxModem_Status_Check (
   TAPI_CHANNEL *pChannel,
   IFX_TAPI_EVENT_t *pEvent)
{
   IFX_TAPI_EVENT_MASK_t *pEventMask;
   IFX_TAPI_MODULE_TYPE_t nModule;
   IFX_boolean_t bEventDisabled = IFX_FALSE,
                 bEventExternal,
                 bEventInternal;

   TAPI_ASSERT (pChannel);

   if (IFX_NULL == pChannel->pEventHandler)
      return IFX_FALSE;

   pEventMask = &(pChannel->pEventHandler->eventMask);

#ifdef TAPI_VERSION3
   nModule = IFX_TAPI_MODULE_TYPE_COD;
#else
   nModule = pEvent->module;
#endif
   if (nModule > IFX_TAPI_MODULE_TYPE_DECT)
   {
      TRACE (TAPI_DRV,DBG_LEVEL_HIGH,
         ("Invalid module type: %d.\n", nModule));
      return IFX_TRUE;
   }

   bEventExternal = TAPI_COD_EXTERNAL (pEvent->data.fax_sig) ? IFX_TRUE : IFX_FALSE;
   bEventInternal = TAPI_COD_INTERNAL (pEvent->data.fax_sig) ? IFX_TRUE : IFX_FALSE;
   if (pEvent->id == IFX_TAPI_EVENT_FAXMODEM_AM)
   {
      if ((bEventExternal && pEventMask->fax_sig[nModule].bits.am_external) ||
          (bEventInternal && pEventMask->fax_sig[nModule].bits.am_internal))
      {
         bEventDisabled = IFX_TRUE;
      }
   }
   if (pEvent->id == IFX_TAPI_EVENT_FAXMODEM_CED)
   {
      if ((bEventExternal && pEventMask->fax_sig[nModule].bits.ced_external) ||
          (bEventInternal && pEventMask->fax_sig[nModule].bits.ced_internal))
      {
         bEventDisabled = IFX_TRUE;
      }
   }
   if (pEvent->id == IFX_TAPI_EVENT_FAXMODEM_PR)
   {
      if ((bEventExternal && pEventMask->fax_sig[nModule].bits.pr_external) ||
          (bEventInternal && pEventMask->fax_sig[nModule].bits.pr_internal))
      {
         bEventDisabled = IFX_TRUE;
      }
   }
   if (pEvent->id == IFX_TAPI_EVENT_FAXMODEM_DIS)
   {
      if ((bEventExternal && pEventMask->fax_sig[nModule].bits.dis_external) ||
          (bEventInternal && pEventMask->fax_sig[nModule].bits.dis_internal))
      {
         bEventDisabled = IFX_TRUE;
      }
   }
   if (pEvent->id == IFX_TAPI_EVENT_FAXMODEM_CNGFAX)
   {
      if ((bEventExternal && pEventMask->fax_sig[nModule].bits.cngfax_external) ||
          (bEventInternal && pEventMask->fax_sig[nModule].bits.cngfax_internal))
      {
         bEventDisabled = IFX_TRUE;
      }
   }
   if (pEvent->id == IFX_TAPI_EVENT_FAXMODEM_CNGMOD)
   {
      if ((bEventExternal && pEventMask->fax_sig[nModule].bits.cngmod_external) ||
          (bEventInternal && pEventMask->fax_sig[nModule].bits.cngmod_internal))
      {
         bEventDisabled = IFX_TRUE;
      }
   }
   if (pEvent->id == IFX_TAPI_EVENT_FAXMODEM_V21L)
   {
      if ((bEventExternal && pEventMask->fax_sig[nModule].bits.v21l_external) ||
          (bEventInternal && pEventMask->fax_sig[nModule].bits.v21l_internal))
      {
         bEventDisabled = IFX_TRUE;
      }
   }
   if (pEvent->id == IFX_TAPI_EVENT_FAXMODEM_V18A)
   {
      if ((bEventExternal && pEventMask->fax_sig[nModule].bits.v18a_external) ||
          (bEventInternal && pEventMask->fax_sig[nModule].bits.v18a_internal))
      {
         bEventDisabled = IFX_TRUE;
      }
   }
   if (pEvent->id == IFX_TAPI_EVENT_FAXMODEM_V27)
   {
      if ((bEventExternal && pEventMask->fax_sig[nModule].bits.v27_external) ||
          (bEventInternal && pEventMask->fax_sig[nModule].bits.v27_internal))
      {
         bEventDisabled = IFX_TRUE;
      }
   }
   if (pEvent->id == IFX_TAPI_EVENT_FAXMODEM_BELL)
   {
      if ((bEventExternal && pEventMask->fax_sig[nModule].bits.bell_external) ||
          (bEventInternal && pEventMask->fax_sig[nModule].bits.bell_internal))
      {
         bEventDisabled = IFX_TRUE;
      }
   }
   if (pEvent->id == IFX_TAPI_EVENT_FAXMODEM_V22)
   {
      if ((bEventExternal && pEventMask->fax_sig[nModule].bits.v22_external) ||
          (bEventInternal && pEventMask->fax_sig[nModule].bits.v22_internal))
      {
         bEventDisabled = IFX_TRUE;
      }
   }
   if (pEvent->id == IFX_TAPI_EVENT_FAXMODEM_V22ORBELL)
   {
      if ((bEventExternal && pEventMask->fax_sig[nModule].bits.v22orbell_external) ||
          (bEventInternal && pEventMask->fax_sig[nModule].bits.v22orbell_internal))
      {
         bEventDisabled = IFX_TRUE;
      }
   }
   if (pEvent->id == IFX_TAPI_EVENT_FAXMODEM_V32AC)
   {
      if ((bEventExternal && pEventMask->fax_sig[nModule].bits.v32ac_external) ||
          (bEventInternal && pEventMask->fax_sig[nModule].bits.v32ac_internal))
      {
         bEventDisabled = IFX_TRUE;
      }
   }
   if (pEvent->id == IFX_TAPI_EVENT_FAXMODEM_CAS_BELL)
   {
      if ((bEventExternal && pEventMask->fax_sig[nModule].bits.cas_external) ||
          (bEventInternal && pEventMask->fax_sig[nModule].bits.cas_internal))
      {
         bEventDisabled = IFX_TRUE;
      }
   }
   if (pEvent->id == IFX_TAPI_EVENT_FAXMODEM_V21H)
   {
      if ((bEventExternal && pEventMask->fax_sig[nModule].bits.v21h_external) ||
          (bEventInternal && pEventMask->fax_sig[nModule].bits.v21h_internal))
      {
         bEventDisabled = IFX_TRUE;
      }
   }
   if (pEvent->id == IFX_TAPI_EVENT_FAXMODEM_V8BIS)
   {
      if ((bEventExternal && pEventMask->fax_sig[nModule].bits.v8bis_external) ||
          (bEventInternal && pEventMask->fax_sig[nModule].bits.v8bis_internal))
      {
         bEventDisabled = IFX_TRUE;
      }
   }
   if (pEvent->id == IFX_TAPI_EVENT_FAXMODEM_HOLDEND)
   {
      /* holding will usually send from internal and external as a sum */
      if ((bEventExternal && pEventMask->fax_sig[nModule].bits.hold_external) ||
          (bEventInternal && pEventMask->fax_sig[nModule].bits.hold_internal))
      {
         bEventDisabled = IFX_TRUE;
      }
   }

   return bEventDisabled;
}

/**
   Convert the event to a sig detection structure (old style)

   \param pEvent - Tapi event
   \param pResult - Result MFTD event

   \return
   TAPI_statusOk - success
   TAPI_statusEvtNoHandle - not supported event
*/
static IFX_int32_t tapi_FaxModem_EventConvert (
   IFX_TAPI_EVENT_t const *pEvent,
   IFX_TAPI_SIG_DETECTION_t *pResult)
{
   IFX_int32_t ret = TAPI_statusOk;
   IFX_uint32_t external, internal;

   external = TAPI_COD_EXTERNAL (pEvent->data.fax_sig);
   internal = TAPI_COD_INTERNAL (pEvent->data.fax_sig);

#ifdef TAPI_VERSION4
   pResult->module = pEvent->module;

   if (pResult->module == IFX_TAPI_MODULE_TYPE_NONE)
      /* set default */
      pResult->module = IFX_TAPI_MODULE_TYPE_COD;
#endif /* TAPI_VERSION4 */

   switch (pEvent->id)
   {
      case  IFX_TAPI_EVENT_FAXMODEM_AM:
         pResult->sig = IFX_TAPI_SIG_AM;
         pResult->sig_ext = IFX_TAPI_SIG_EXT_NONE;
         if (external)
         {
            pResult->sig |= IFX_TAPI_SIG_AMTX;
         }
         if (internal)
         {
            pResult->sig |= IFX_TAPI_SIG_AMRX;
         }
      break;
      case  IFX_TAPI_EVENT_FAXMODEM_CED:
         pResult->sig = IFX_TAPI_SIG_CED;
         pResult->sig_ext = IFX_TAPI_SIG_EXT_NONE;
         if (external)
         {
            pResult->sig |= IFX_TAPI_SIG_CEDTX;
         }
         if (internal)
         {
            pResult->sig |= IFX_TAPI_SIG_CEDRX;
         }
      break;
      case  IFX_TAPI_EVENT_FAXMODEM_CEDEND:
         pResult->sig = IFX_TAPI_SIG_CEDEND;
         pResult->sig_ext = IFX_TAPI_SIG_EXT_NONE;
         if (external)
         {
            pResult->sig |= IFX_TAPI_SIG_CEDENDTX;
         }
         if (internal)
         {
            pResult->sig |= IFX_TAPI_SIG_CEDENDRX;
         }
      break;
      case  IFX_TAPI_EVENT_FAXMODEM_DIS:
         pResult->sig = IFX_TAPI_SIG_DIS;
         pResult->sig_ext = IFX_TAPI_SIG_EXT_NONE;
         if (external)
         {
            pResult->sig |= IFX_TAPI_SIG_DISTX;
         }
         if (internal)
         {
            pResult->sig |= IFX_TAPI_SIG_DISRX;
         }
      break;
      case  IFX_TAPI_EVENT_FAXMODEM_PR:
         pResult->sig = IFX_TAPI_SIG_PHASEREV;
         pResult->sig_ext = IFX_TAPI_SIG_EXT_NONE;
         if (external)
         {
            pResult->sig |= IFX_TAPI_SIG_PHASEREVTX;
         }
         if (internal)
         {
            pResult->sig |= IFX_TAPI_SIG_PHASEREVRX;
         }
      break;
      case  IFX_TAPI_EVENT_FAXMODEM_CNGFAX:
         pResult->sig = IFX_TAPI_SIG_CNGFAX;
         pResult->sig_ext = IFX_TAPI_SIG_EXT_NONE;
         if (external)
         {
            pResult->sig |= IFX_TAPI_SIG_CNGFAXTX;
         }
         if (internal)
         {
            pResult->sig |= IFX_TAPI_SIG_CNGFAXRX;
         }
      break;
      case  IFX_TAPI_EVENT_FAXMODEM_CNGMOD:
         pResult->sig = IFX_TAPI_SIG_CNGMOD;
         pResult->sig_ext = IFX_TAPI_SIG_EXT_NONE;
         if (external)
         {
            pResult->sig |= IFX_TAPI_SIG_CNGMODTX;
         }
         if (internal)
         {
            pResult->sig |= IFX_TAPI_SIG_CNGMODRX;
         }
      break;
      case  IFX_TAPI_EVENT_FAXMODEM_V21L:
         pResult->sig = IFX_TAPI_SIG_NONE;
         pResult->sig_ext = IFX_TAPI_SIG_EXT_V21L;
         if (external)
         {
            pResult->sig_ext |= IFX_TAPI_SIG_EXT_V21LTX;
         }
         if (internal)
         {
            pResult->sig_ext |= IFX_TAPI_SIG_EXT_V21LRX;
         }
      break;
      case  IFX_TAPI_EVENT_FAXMODEM_V18A:
         pResult->sig = IFX_TAPI_SIG_NONE;
         pResult->sig_ext = IFX_TAPI_SIG_EXT_V18A;
         if (external)
         {
            pResult->sig_ext |= IFX_TAPI_SIG_EXT_V18ATX;
         }
         if (internal)
         {
            pResult->sig_ext |= IFX_TAPI_SIG_EXT_V18ARX;
         }
      break;
      case  IFX_TAPI_EVENT_FAXMODEM_V27:
         pResult->sig = IFX_TAPI_SIG_NONE;
         pResult->sig_ext = IFX_TAPI_SIG_EXT_V27;
         if (external)
         {
            pResult->sig_ext |= IFX_TAPI_SIG_EXT_V27TX;
         }
         if (internal)
         {
            pResult->sig_ext |= IFX_TAPI_SIG_EXT_V27RX;
         }
      break;
      case  IFX_TAPI_EVENT_FAXMODEM_BELL:
         pResult->sig = IFX_TAPI_SIG_NONE;
         pResult->sig_ext = IFX_TAPI_SIG_EXT_BELL;
         if (external)
         {
            pResult->sig_ext |= IFX_TAPI_SIG_EXT_BELLTX;
         }
         if (internal)
         {
            pResult->sig_ext |= IFX_TAPI_SIG_EXT_BELLRX;
         }
      break;
      case  IFX_TAPI_EVENT_FAXMODEM_V22:
         pResult->sig = IFX_TAPI_SIG_NONE;
         pResult->sig_ext = IFX_TAPI_SIG_EXT_V22;
         if (external)
         {
            pResult->sig_ext |= IFX_TAPI_SIG_EXT_V22TX;
         }
         if (internal)
         {
            pResult->sig_ext |= IFX_TAPI_SIG_EXT_V22RX;
         }
      break;
      case  IFX_TAPI_EVENT_FAXMODEM_V22ORBELL:
         pResult->sig = IFX_TAPI_SIG_NONE;
         pResult->sig_ext = IFX_TAPI_SIG_EXT_V22ORBELL;
         if (external)
         {
            pResult->sig_ext |= IFX_TAPI_SIG_EXT_V22ORBELLTX;
         }
         if (internal)
         {
            pResult->sig_ext |= IFX_TAPI_SIG_EXT_V22ORBELLRX;
         }
      break;
      case  IFX_TAPI_EVENT_FAXMODEM_V32AC:
         pResult->sig = IFX_TAPI_SIG_NONE;
         pResult->sig_ext = IFX_TAPI_SIG_EXT_V32AC;
         if (external)
         {
            pResult->sig_ext |= IFX_TAPI_SIG_EXT_V32ACTX;
         }
         if (internal)
         {
            pResult->sig_ext |= IFX_TAPI_SIG_EXT_V32ACRX;
         }
      break;
      case  IFX_TAPI_EVENT_FAXMODEM_V8BIS:
         pResult->sig = IFX_TAPI_SIG_NONE;
         pResult->sig_ext = IFX_TAPI_SIG_EXT_NONE;
         if (external)
         {
            pResult->sig |= IFX_TAPI_SIG_V8BISTX;
         }
         if (internal)
         {
            pResult->sig |= IFX_TAPI_SIG_V8BISRX;
         }
      break;
      case IFX_TAPI_EVENT_FAXMODEM_CAS_BELL:
         pResult->sig = IFX_TAPI_SIG_NONE;
         pResult->sig_ext = IFX_TAPI_SIG_EXT_CASBELL;
         if (external)
         {
            pResult->sig_ext |= IFX_TAPI_SIG_EXT_CASBELLTX;
         }
         if (internal)
         {
            pResult->sig_ext |= IFX_TAPI_SIG_EXT_CASBELLRX;
         }
         break;
      case IFX_TAPI_EVENT_FAXMODEM_V21H:
         pResult->sig = IFX_TAPI_SIG_NONE;
         pResult->sig_ext = IFX_TAPI_SIG_EXT_NONE;
         if (external)
         {
            pResult->sig_ext |= IFX_TAPI_SIG_EXT_V21HTX;
         }
         if (internal)
         {
            pResult->sig_ext |= IFX_TAPI_SIG_EXT_V21HRX;
         }
         break;
      case  IFX_TAPI_EVENT_FAXMODEM_HOLDEND:
         pResult->sig = IFX_TAPI_SIG_TONEHOLDING_END;
         pResult->sig_ext = IFX_TAPI_SIG_EXT_NONE;
         if (external)
         {
            pResult->sig |= IFX_TAPI_SIG_TONEHOLDING_ENDTX;
         }
         if (internal)
         {
            pResult->sig |= IFX_TAPI_SIG_TONEHOLDING_ENDRX;
         }
      break;
      case  IFX_TAPI_EVENT_FAXMODEM_VMD:
         pResult->sig = IFX_TAPI_SIG_NONE;
         pResult->sig_ext = IFX_TAPI_SIG_EXT_VMD;
         break;
      default:
         ret = TAPI_statusEvtNoHandle;
         break;
   }
   return ret;
}


#if (TAPI_CFG_FEATURES & TAPI_FEAT_FAX_T38)
/**

   Fax Status update Event handling function. Is called from interrupt routine
   or elsewhere a fax related error occurs.

   \param pChannel    - handle to TAPI_CHANNEL structure
   \param status      - Fax status
   \param error       - Fax error

   \return
      None
*/
IFX_void_t TAPI_FaxT38_Event_Update (TAPI_CHANNEL * pChannel,
                                     IFX_uint8_t status, IFX_uint8_t error)
{
   IFX_TAPI_DRV_CTX_t *pDrvCtx = pChannel->pTapiDevice->pDevDrvCtx;

   if (IFX_TAPI_PtrChk (pDrvCtx->COD.T38_Status_Set))
      (IFX_void_t)pDrvCtx->COD.T38_Status_Set (pChannel->pLLChannel, status);

   if (IFX_TAPI_PtrChk (pDrvCtx->COD.T38_Error_Set))
      (IFX_void_t)pDrvCtx->COD.T38_Error_Set (pChannel->pLLChannel, error);
}
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_FAX_T38) */


/**
   WakeUp the TAPI channel task.

   \param  pTapiDev     Pointer to TAPI device structure.
*/
static IFX_void_t tapi_WakeUp (TAPI_DEV *pTapiDev)
{
#if 0 /* Polling Not supported */
   /* In polled mode, there are no sleeping tasks (blocked on select), so no
      wakeup needed */
   if ((pTapiDev->bNeedWakeup == IFX_TRUE) &&
       (pDev->IrqPollMode & VIN_TAPI_WAKEUP))
#else
   if (pTapiDev->bNeedWakeup)
#endif /* 0 */
   {
      TAPI_OS_DrvSelectQueueWakeUp (&pTapiDev->wqEvent,
                                    TAPI_OS_DRV_SEL_WAKEUP_TYPE_RD);
   }
}


/**
  Check whether event fifo is empty or not (protected).
  \param pChannel - High level TAPI Channel

  \return
    If empty, return 1,else 0.
*/
IFX_uint8_t IFX_TAPI_EventFifoEmpty (TAPI_CHANNEL * pChannel)
{
   IFX_uint8_t ret = 0;

   if (pChannel->pEventHandler == IFX_NULL)
      return 1;
   /* Lock the fifo access protection. */
   TAPI_OS_MutexGet (&pChannel->pEventHandler->fifoAcc);

   if (fifoEmpty (pChannel->pEventHandler->pTapiEventFifoHi) &&
       fifoEmpty (pChannel->pEventHandler->pTapiEventFifoLo))
   {
      ret = 1;
   }
   else
   {
      ret = 0;
   }

   /* Unlock fifo access protection. */
   TAPI_OS_MutexRelease (&pChannel->pEventHandler->fifoAcc);

   return ret;
}


/**
  Return counters from the event fifos of the given channel.

  This function cares about protecting the fifo and returns counters from the
  fifos of the channel. Currently only the number of events waiting in the
  fifos are reported. Possible extensions are counters on the size, the full
  state or the number of transactions.

  \param  pChannel     Pointer to tapi channel structure.
  \param  pCounters    Pointer to struct in which the counters are returned.
                       Content is valid only if return is TAPI_statusOk.

  \return
  - TAPI_statusOk on success.
  - TAPI_statusErr if the channel is uninitialised or parameter is IFX_NULL.
*/
IFX_uint32_t IFX_TAPI_EventFifoCounters (TAPI_CHANNEL *pChannel,
                  IFX_TAPI_EVENT_FIFO_COUNTERS_t *pCounters)
{
   if ((pChannel->pEventHandler == IFX_NULL) || (pCounters == IFX_NULL))
      return TAPI_statusErr;

   /* Lock the fifo access protection. */
   TAPI_OS_MutexGet (&pChannel->pEventHandler->fifoAcc);

   pCounters->nHighWaiting =
                     fifoElements (pChannel->pEventHandler->pTapiEventFifoHi);
   pCounters->nLowWaiting  =
                     fifoElements (pChannel->pEventHandler->pTapiEventFifoLo);

   /* Unlock fifo access protection. */
   TAPI_OS_MutexRelease (&pChannel->pEventHandler->fifoAcc);

   return TAPI_statusOk;
}


/**
   Get event stored in the event fifo.

   \param pChannel -Channel context for HL TAPI.
   \param pTapiEvent - Event.

   \return
   If there are more events stored in hi/lo fifo, return 1, otherwise return 0.

   \remarks
   High Level (ioctl) event fifo access function.
   Because this function was called by ioctl, it needs semphore protection
   and ceases interrupt routine.
   It will free the buffer after it returns a event.
*/
static IFX_int32_t tapi_EventFifoGet (
   TAPI_CHANNEL *pChannel,
   IFX_TAPI_EVENT_t *pTapiEvent)
{
   IFX_TAPI_EXT_EVENT_PARAM_t *pTempEvt = IFX_NULL;
   IFX_int32_t moreEvent = 0;
   IFX_int32_t ret;
#ifndef TAPI_LIBRARY
   TAPI_OS_INTSTAT lock;
#endif /* TAPI_LIBRARY */

   TAPI_ASSERT (pChannel);

   if (IFX_NULL == pChannel->pEventHandler)
      return 0;

   /* Lock the fifo access protection. */
   TAPI_OS_MutexGet (&pChannel->pEventHandler->fifoAcc);

   /* Get event stored in high prio fifo */
   pTempEvt = fifoGet (pChannel->pEventHandler->pTapiEventFifoHi, NULL);

   if (pTempEvt)
   {
      /* copy event data into struct provided by the caller */
      memcpy (pTapiEvent, &pTempEvt->tapiEvent, sizeof (IFX_TAPI_EVENT_t));
      /* release the buffer back to the pool */
      TAPI_OS_MutexGet (&semBufferPoolAcc);
#ifndef TAPI_LIBRARY
      TAPI_OS_LOCKINT (lock);
#endif /* TAPI_LIBRARY */
      ret = bufferPoolPut ((IFX_void_t *) pTempEvt);
#ifndef TAPI_LIBRARY
      TAPI_OS_UNLOCKINT (lock);
#endif /* TAPI_LIBRARY */
      TAPI_OS_MutexRelease (&semBufferPoolAcc);
      if (ret == BUFFERPOOL_ERROR)
      {
         /* \todo if bufferpool handling fail? */
      }

      /* Check whether there are any more events stored in hi or lo fifo. */
      if (fifoEmpty (pChannel->pEventHandler->pTapiEventFifoHi) &&
          fifoEmpty (pChannel->pEventHandler->pTapiEventFifoLo))
      {
         /* no events stored in hi or lo fifo */
         moreEvent = 0;
      }
      else
      {
         moreEvent = 1;
      }

      /* Unlock fifo access protection. */
      TAPI_OS_MutexRelease (&pChannel->pEventHandler->fifoAcc);

      return moreEvent;
   }

   /* If hi prio fifo is empty, get event stored in low prio fifo. */
   pTempEvt = fifoGet (pChannel->pEventHandler->pTapiEventFifoLo, NULL);
   if (pTempEvt)
   {
      /* copy event data into struct provided by the caller */
      memcpy (pTapiEvent, &pTempEvt->tapiEvent, sizeof (IFX_TAPI_EVENT_t));
      /* release the buffer back to the pool */
      TAPI_OS_MutexGet (&semBufferPoolAcc);
#ifndef TAPI_LIBRARY
      TAPI_OS_LOCKINT (lock);
#endif /* TAPI_LIBRARY */
      ret = bufferPoolPut ((IFX_void_t *) pTempEvt);
#ifndef TAPI_LIBRARY
      TAPI_OS_UNLOCKINT (lock);
#endif /* TAPI_LIBRARY */
      TAPI_OS_MutexRelease (&semBufferPoolAcc);

      if (ret == BUFFERPOOL_ERROR)
      {
         /* \todo if bufferpool handling fail? */
      }

      /* Check whether there are any more events stored in lo fifo. */
      if (fifoEmpty (pChannel->pEventHandler->pTapiEventFifoLo))
      {
         /* no events stored in lo fifo */
         moreEvent = 0;
      }
      else
      {
         moreEvent = 1;
      }

      /* Unlock fifo access protection. */
      TAPI_OS_MutexRelease (&pChannel->pEventHandler->fifoAcc);

      return moreEvent;
   }

   /* Unlock fifo access protection. */
   TAPI_OS_MutexRelease (&pChannel->pEventHandler->fifoAcc);

   /* If high and low prio fifo is empty, set moreEvent to 0 and event id
      to IFX_TAPI_EVENT_NONE. Do not change the channel parameter. */
   pTapiEvent->id = IFX_TAPI_EVENT_NONE;
   moreEvent = 0;

   return moreEvent;
}


/********************NOTE *****************************************************

 1.IFX_TAPI_Event_Dispatch : Call this function to dispatch an event. The
 event is stored in an allocated buffer together with additional information
 such as the channel. This is then passed on to task context with the
 TAPI_DeferWork() function.

 2.IFX_TAPI_Event_Dispatch_ProcessCtx : This function is working in task
 context. It receives the events dispatched by IFX_TAPI_Event_Dispatch
 processes and filters them and finally put them into the queue towards the
 application.

*******************************************************************************/

/**
   High Level Event Dispatcher function
   (May be called from task or interrupt context.)

   \param pChannel     - handle to tapi channel structure
   \param pTapiEvent   - handle to event

   \return TAPI_statusOk on ok, otherwise TAPI_statusErr
 */
IFX_int32_t IFX_TAPI_Event_Dispatch (TAPI_CHANNEL *pChannel,
   IFX_TAPI_EVENT_t *pTapiEvent)
{
   return IFX_TAPI_Event_DispatchExt (pChannel, pTapiEvent,
      TAPI_OS_IN_INTERRUPT());
}


/**
   High Level Event Dispatcher function
   (May be called from task or interrupt context.)

   \param pChannel     - handle to TAPI channel structure
   \param pTapiEvent   - handle to TAPI event
   \param bDefer If set to IFX_TRUE the event will be deferred in
      a new task/thread. This is equal to the TAPI_OS_IN_INTERRUPT statement,
      which also leads to deffering the event.
      Typically most events can be handled directely, but some require follow-up
      handling and LL interaction.

   \return TAPI_statusOk on ok, otherwise TAPI_statusErr
 */
IFX_int32_t IFX_TAPI_Event_DispatchExt (TAPI_CHANNEL *pChannel,
   IFX_TAPI_EVENT_t *pTapiEvent, IFX_boolean_t bDefer)
{
   IFX_int32_t ret = TAPI_statusOk;
   IFX_TAPI_EXT_EVENT_PARAM_t *pDeferredEvent;
#ifndef TAPI_LIBRARY
   TAPI_OS_INTSTAT lock;
#endif /* TAPI_LIBRARY */

   TAPI_ASSERT (pChannel);
   TAPI_ASSERT (pTapiEvent);

   /* filter error message and just copy the error stack */
   if (pTapiEvent->id == IFX_TAPI_EVENT_FAULT_GENERAL_CHINFO ||
       pTapiEvent->id == IFX_TAPI_EVENT_FAULT_GENERAL_DEVINFO)
   {
      TAPI_DEV *pTapiDev = pChannel->pTapiDevice;
      if (pTapiDev->error.nCnt < IFX_TAPI_MAX_ERROR_ENTRIES - 1)
      {
         memcpy (&pTapiDev->error.stack[pTapiDev->error.nCnt++],
                 (IFX_TAPI_ErrorLine_t*)pTapiEvent->data.error,
                 sizeof (IFX_TAPI_ErrorLine_t));
         pTapiDev->error.nCh   = pChannel->nChannel;
         pTapiDev->error.nCode =
            (IFX_uint32_t)pTapiEvent->data.error->nLlCode;
      }
      return TAPI_statusOk;
   }

   /* in force set the responsible device index */
   pTapiEvent->dev = pChannel->pTapiDevice->nDev;

   /* global irq lock - multiple drivers may be loaded and all share this
      event dispatch function. Lock access to the shared buffer pool. */
#ifndef TAPI_LIBRARY
   if (!TAPI_OS_IN_INTERRUPT())
#endif /* TAPI_LIBRARY */
      TAPI_OS_MutexGet (&semBufferPoolAcc);
#ifndef TAPI_LIBRARY
   TAPI_OS_LOCKINT (lock);
#endif /* TAPI_LIBRARY */

   pDeferredEvent = (IFX_TAPI_EXT_EVENT_PARAM_t *)
                               bufferPoolGet (pIFX_TAPI_BP_Deferred_Event);

#ifndef TAPI_LIBRARY
   TAPI_OS_UNLOCKINT (lock);
   if (!TAPI_OS_IN_INTERRUPT())
#endif /* TAPI_LIBRARY */
      TAPI_OS_MutexRelease (&semBufferPoolAcc);

   if (pDeferredEvent == IFX_NULL)
   {
      TRACE(TAPI_DRV, DBG_LEVEL_HIGH,
            ("TAPI dev%d,ch%d: failure dispatching event 0x%08x "
            "(no free pIFX_TAPI_BP_Deferred_Event buffers)\n\r",
            pTapiEvent->dev, pTapiEvent->ch, pTapiEvent->id));

      /* if no deferred_event buffer is free - discard the event as well :-/ */
      RETURN_STATUS (TAPI_statusErr, 0);
   }

   memcpy (&pDeferredEvent->tapiEvent, pTapiEvent, sizeof (IFX_TAPI_EVENT_t));
   pDeferredEvent->pChannel = pChannel;

   /* decide if the event can be handled immediately or needs to be scheduled
      into the process context first (if called from interrupt context) */
   if (!bDefer)
   {
#ifndef TAPI_LIBRARY
      TAPI_ASSERT (!TAPI_OS_IN_INTERRUPT());
#endif /* TAPI_LIBRARY */
      /* immediately handle this event, no scheduling required */
      ret = IFX_TAPI_Event_Dispatch_ProcessCtx (pDeferredEvent);
   }
   else
   {
      /*lint -save -e611 \todo: FIXME
         (ANSI X3.159-1989)
         It is invalid to convert a function pointer to an object pointer
         or a pointer to void, or vice-versa.
      */
      ret = (IFX_int32_t)TAPI_DeferWork(
                           (IFX_void_t *) IFX_TAPI_Event_Dispatch_ProcessCtx,
                           (IFX_void_t *) pDeferredEvent);
      /*lint -restore */
      /* error handling */
      if (ret != TAPI_statusOk)
      {
         /* deferring failed - return the deferred_event data structure
            to the bufferpool */
#ifndef TAPI_LIBRARY
         if (!TAPI_OS_IN_INTERRUPT())
#endif /* TAPI_LIBRARY */
            TAPI_OS_MutexGet (&semBufferPoolAcc);
#ifndef TAPI_LIBRARY
         TAPI_OS_LOCKINT (lock);
#endif /* TAPI_LIBRARY */
         bufferPoolPut(pDeferredEvent);
#ifndef TAPI_LIBRARY
         TAPI_OS_UNLOCKINT (lock);
         if (!TAPI_OS_IN_INTERRUPT())
#endif /* TAPI_LIBRARY */
            TAPI_OS_MutexRelease (&semBufferPoolAcc);
         RETURN_STATUS (ret, 0);
      }
   }

   return ret;
}


/**
   Discard a TAPI deferred event buffer

   Helper routine used to return an allocated deferred event buffer back
   to its buffer pool. The routine takes care about interrupt and concurrent
   task protection.

   \param  pData        Pointer to event wrapper buffer.

   \remarks The function always succeeds.
   \remarks This function must not be called from IRQ context.
*/
static IFX_void_t tapi_DiscardEvent (IFX_void_t *pEvent)
{
#ifndef TAPI_LIBRARY
   TAPI_OS_INTSTAT lock;
#endif /* TAPI_LIBRARY */

   TAPI_OS_MutexGet (&semBufferPoolAcc);
#ifndef TAPI_LIBRARY
   TAPI_OS_LOCKINT (lock);
#endif /* TAPI_LIBRARY */
   bufferPoolPut (pEvent);
#ifndef TAPI_LIBRARY
   TAPI_OS_UNLOCKINT (lock);
#endif /* TAPI_LIBRARY */
   TAPI_OS_MutexRelease (&semBufferPoolAcc);
}


/**
   High Level Event Dispatcher task.

   \param pParam - parameters with task structure and detected event.

   \return TAPI_statusOk on ok, otherwise TAPI_statusErr

   \remarks
   This function is spawned as a task to dispatch all events from process
   context. It is the single resource of event processing. The buffers it
   uses are allocated in IFX_TAPI_Event_Dispatch above which is called
   from interrupt or task context. So locking needs to be done during
   all buffer operations. Multiple drivers may be loaded which all can
   generate events. Instead of locking each driver separately we lock all
   interrupts globally. Additionally semaphores are needed to lock the
   access to the buffers and fifos.
 */
IFX_int32_t IFX_TAPI_Event_Dispatch_ProcessCtx(IFX_TAPI_EXT_EVENT_PARAM_t *pParam)
{
   IFX_int32_t ret = TAPI_statusOk;
   IFX_boolean_t bEventDisabled = IFX_FALSE;
   IFX_boolean_t bEventDiscard = IFX_TRUE;
   TAPI_CHANNEL* pChannel;
   IFX_TAPI_EVENT_t *pEvent, TapiEvent;
   IFX_TAPI_DRV_CTX_t *pDrvCtx = IFX_NULL;
   IFX_TAPI_EVENT_MASK_t *pEventMask;
   IFX_TAPI_MODULE_TYPE_t nModule;
   FIFO_ID *pDispatchFifo;

   TAPI_ASSERT(pParam);
   pChannel = pParam->pChannel;
   if ((IFX_NULL == pChannel) || (IFX_NULL == pChannel->pEventHandler))
   {
      tapi_DiscardEvent (pParam);
      /* no error for not yet initialized event handler */
      return TAPI_statusOk;
   }
   /* at this point pChannel is valid */

   pDrvCtx = pChannel->pTapiDevice->pDevDrvCtx;

   pEvent = &pParam->tapiEvent;
   pEvent->ch = (IFX_uint16_t) pChannel->nChannel;
   /**\todo log event with new log type:
    LOG_STR (pEvent->ch, "Evt ID 0x%X DATA 0x%X", pEvent->id, pEvent->data.value);
    */

#ifdef TAPI_VERSION3
   nModule = IFX_TAPI_MODULE_TYPE_COD;
#else
   nModule = pEvent->module;
#endif
   if (nModule > IFX_TAPI_MODULE_TYPE_DECT)
   {
      tapi_DiscardEvent (pParam);
      RETURN_STATUS (TAPI_statusParam, 0);
   }

   pEventMask = &(pChannel->pEventHandler->eventMask);

   /* Errors are handled first and put into the high-priority fifo.
      Normal events are handled afterwards and go to the low-priority fifo. */
   if ((((IFX_uint32_t)pEvent->id) & IFX_TAPI_EVENT_TYPE_FAULT_MASK) ==
         IFX_TAPI_EVENT_TYPE_FAULT_MASK)
   {
      /*lint -save -e650 */
      /*
       * If any fault event occured, invoke the error state machine.
       *  Then, put this event into the high-priority FIFO.
       */
      pDispatchFifo = pChannel->pEventHandler->pTapiEventFifoHi;

      switch (((IFX_uint32_t)pEvent->id) & IFX_TAPI_EVENT_TYPE_MASK)
      {
      case IFX_TAPI_EVENT_TYPE_FAULT_GENERAL:
         {
            int src = pEvent->data.value & IFX_TAPI_ERRSRC_MASK;
            /* clear error clasification */
            pEvent->data.value &= ~(IFX_TAPI_ERRSRC_MASK);
            switch (src)
            {
               case  IFX_TAPI_ERRSRC_LL_DEV:
                  /**\todo put in device fifo */
                  pEvent->ch = IFX_TAPI_DEVICE_CH_NUMBER;
                  /*lint -fallthrough */
               case  IFX_TAPI_ERRSRC_LL_CH:
                  pEvent->data.value |= IFX_TAPI_ERRSRC_LL;
                  break;
               case  IFX_TAPI_ERRSRC_TAPI_CH:
               case  IFX_TAPI_ERRSRC_TAPI_DEV:
               default:
               /* do nothing */
                  break;
            }
            bEventDisabled = IFX_FALSE;
         }
         break;
      case IFX_TAPI_EVENT_TYPE_FAULT_LINE:
         switch (pEvent->id)
         {
         case IFX_TAPI_EVENT_FAULT_LINE_OVERTEMP:
            pChannel->TapiOpControlData.nLineMode = IFX_TAPI_LINE_FEED_DISABLED;
            bEventDisabled = pEventMask->fault_line.bits.overtemp ?
                            IFX_TRUE : IFX_FALSE;
            break;
         case IFX_TAPI_EVENT_FAULT_LINE_OVERTEMP_END:
            bEventDisabled = pEventMask->fault_line.bits.overtemp_end ?
                            IFX_TRUE : IFX_FALSE;
            break;
         case IFX_TAPI_EVENT_FAULT_LINE_GK_HIGH:
            bEventDisabled = pEventMask->fault_line.bits.gk_high ?
                            IFX_TRUE : IFX_FALSE;
            break;
         case IFX_TAPI_EVENT_FAULT_LINE_GK_HIGH_INT:
            /* Internal GK high event */
            IFX_TAPI_Dial_GNKH_Event (pChannel);
            pEvent->id = IFX_TAPI_EVENT_FAULT_LINE_GK_HIGH;
            bEventDisabled = IFX_FALSE;
            break;
         case IFX_TAPI_EVENT_FAULT_LINE_GK_HIGH_END:
            bEventDisabled = pEventMask->fault_line.bits.gk_high_end ?
                            IFX_TRUE : IFX_FALSE;
            break;
         case IFX_TAPI_EVENT_FAULT_LINE_GK_LOW_INT:
            /* Internal GK low event */
            /* set the TAPI line mode to power down. The TAPI will not signal
               hook changes to the client application anymore till the mode is
               changed again */
            pChannel->TapiOpControlData.bFaulCond = IFX_TRUE;
            pEvent->id = IFX_TAPI_EVENT_FAULT_LINE_GK_LOW;
            bEventDisabled = IFX_FALSE;
            break;
         case IFX_TAPI_EVENT_FAULT_LINE_GK_LOW:
            bEventDisabled = pEventMask->fault_line.bits.gk_low ?
                            IFX_TRUE : IFX_FALSE;
            break;
         case IFX_TAPI_EVENT_FAULT_LINE_GK_LOW_END:
            bEventDisabled = pEventMask->fault_line.bits.gk_low_end ?
                            IFX_TRUE : IFX_FALSE;
            break;
         case IFX_TAPI_EVENT_FAULT_LINE_GK_POS:
            bEventDisabled = pEventMask->fault_line.bits.gk_pos ?
                            IFX_TRUE : IFX_FALSE;
            break;
         case IFX_TAPI_EVENT_FAULT_LINE_GK_NEG:
            bEventDisabled = pEventMask->fault_line.bits.gk_neg ?
                            IFX_TRUE : IFX_FALSE;
            break;
         default:
            /* do nothing */
            break;
         }
#if (TAPI_CFG_FEATURES & TAPI_FEAT_PHONE_DETECTION)
         IFX_TAPI_PPD_HandleEvent(pChannel,pEvent);
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_PHONE_DETECTION) */
         break;
      case IFX_TAPI_EVENT_TYPE_FAULT_HW:
         switch (pEvent->id)
         {
         case IFX_TAPI_EVENT_FAULT_HW_CLOCK_FAIL:
            bEventDisabled = pEventMask->fault_hw.bits.clock_fail ?
                            IFX_TRUE : IFX_FALSE;
            break;
         case IFX_TAPI_EVENT_FAULT_HW_CLOCK_FAIL_END:
            bEventDisabled = pEventMask->fault_hw.bits.clock_fail_end ?
                            IFX_TRUE : IFX_FALSE;
            break;
         case IFX_TAPI_EVENT_FAULT_HW_FAULT:
            bEventDisabled = pEventMask->fault_hw.bits.hw_fault ?
                            IFX_TRUE : IFX_FALSE;
            break;
         case IFX_TAPI_EVENT_FAULT_HW_SYNC:
            bEventDisabled = pEventMask->fault_hw.bits.sync ?
                           IFX_TRUE : IFX_FALSE;
            break;
         case IFX_TAPI_EVENT_FAULT_HW_RESET:
            bEventDisabled = pEventMask->fault_hw.bits.hw_reset ?
                           IFX_TRUE : IFX_FALSE;
            break;
         case IFX_TAPI_EVENT_FAULT_HW_SSI_ERR:
            bEventDisabled = IFX_FALSE;
            break;
         default:
         /* do nothing */
            break;
         }
         break;
      case IFX_TAPI_EVENT_TYPE_FAULT_FW:
         switch (pEvent->id)
         {
            case IFX_TAPI_EVENT_FAULT_FW_WATCHDOG:
               /* FW watchdog event cannot be masked */
               bEventDisabled = IFX_FALSE;
               break;
            default:
            break;
         }
         break;
      case IFX_TAPI_EVENT_TYPE_FAULT_SW:
         break;
      default:
         TRACE (TAPI_DRV, DBG_LEVEL_HIGH,
                ("TAPI dev%d,ch%d: Unknown event ID: 0x%08x.\n",
                pEvent->dev, pEvent->ch, pEvent->id));
         break;
      }
      /*lint -restore */
   }
   else
   {
      /*
       * Normal events, put these into low-priority FIFO
       */
      pDispatchFifo = pChannel->pEventHandler->pTapiEventFifoLo;

      switch (pEvent->id)
      {
      /* EVENT TYPE: -- IFX_TAPI_EVENT_TYPE_FXS -- */

      case IFX_TAPI_EVENT_FXS_ONHOOK_INT:
      case IFX_TAPI_EVENT_FXS_OFFHOOK_INT:
         {
            IFX_boolean_t bSendHookEvent = IFX_TRUE;
            IFX_boolean_t bDoRingStop = IFX_TRUE;
            IFX_boolean_t bOffhook =
                  (pEvent->id == IFX_TAPI_EVENT_FXS_OFFHOOK_INT) ?
                  IFX_TRUE : IFX_FALSE;

#if (TAPI_CFG_FEATURES & TAPI_FEAT_CID)
            IFX_TAPI_CID_OnHookEvent(pChannel, bOffhook,
                                     &bSendHookEvent, &bDoRingStop);
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_CID) */

            if ((pEvent->id == IFX_TAPI_EVENT_FXS_OFFHOOK_INT) &&
                (bDoRingStop == IFX_TRUE))
            {
               /* stop ringing (including an optional CID transmission) */
               IFX_TAPI_Ring_Stop(pChannel);
            }

            if (bSendHookEvent == IFX_TRUE)
            {
               /* trigger event in the hook state machine */
               IFX_TAPI_Dial_HookEvent (pChannel, bOffhook);
            }

            /* We drop the internal hook events. The hook state machine will
               generate new events from this. Depending on the timing we get
               onhook, offhook, flashhook or pulsedialing digits */
            bEventDisabled = IFX_TRUE;
         }
         break;

      case IFX_TAPI_EVENT_FXS_ONHOOK:
         /* IFX_FALSE => nature of hook event is ONhook */
         pChannel->TapiOpControlData.bHookState = IFX_FALSE;
         bEventDisabled = pEventMask->fxs.bits.onhook ? IFX_TRUE : IFX_FALSE;
         break;

      case IFX_TAPI_EVENT_FXS_OFFHOOK:
         /* IFX_TRUE => nature of hook event is OFFhook */
         pChannel->TapiOpControlData.bHookState = IFX_TRUE;
         bEventDisabled = pEventMask->fxs.bits.offhook ? IFX_TRUE : IFX_FALSE;
         break;

      case IFX_TAPI_EVENT_FXS_FLASH:
         bEventDisabled = pEventMask->fxs.bits.flash ? IFX_TRUE : IFX_FALSE;
         break;

      case IFX_TAPI_EVENT_FXS_RAW_ONHOOK:
         /* IFX_FALSE => nature of hook event is ONhook */
         bEventDisabled = pEventMask->fxs.bits.rawonhook ? IFX_TRUE : IFX_FALSE;
         break;

      case IFX_TAPI_EVENT_FXS_RAW_OFFHOOK:
         /* IFX_TRUE => nature of hook event is OFFhook */
         bEventDisabled = pEventMask->fxs.bits.rawoffhook ? IFX_TRUE : IFX_FALSE;
         break;

      case IFX_TAPI_EVENT_FXS_RINGING_END:
         bEventDisabled = pEventMask->fxs.bits.ringing_end ?
                         IFX_TRUE : IFX_FALSE;
         break;

#if (TAPI_CFG_FEATURES & TAPI_FEAT_CONT_MEAS)
      case IFX_TAPI_EVENT_CONTMEASUREMENT:
         bEventDisabled = pEventMask->fxs.bits.contMeas ?
                         IFX_TRUE : IFX_FALSE;
         break;
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_CONT_MEAS) */

      /* line feed mode has changed */
      case IFX_TAPI_EVENT_FXS_LINE_MODE:
         TAPI_Phone_Change_Linefeed(pChannel, pEvent->data.linemode);
         bEventDisabled = pEventMask->fxs.bits.linemode ? IFX_TRUE : IFX_FALSE;
         break;

      case IFX_TAPI_EVENT_FXS_COMTEL_END:
         bEventDisabled = pEventMask->fxs.bits.comtel_end ? IFX_TRUE : IFX_FALSE;
         break;

      /* EVENT TYPE: -- IFX_TAPI_EVENT_TYPE_FXO -- */

      case IFX_TAPI_EVENT_FXO_RING_START:
         bEventDisabled = pEventMask->fxo.bits.ring_start ? IFX_TRUE :
            IFX_FALSE;
         break;

      case IFX_TAPI_EVENT_FXO_RING_STOP:
         bEventDisabled = pEventMask->fxo.bits.ring_stop ? IFX_TRUE : IFX_FALSE;
         break;

      case IFX_TAPI_EVENT_FXO_BAT_FEEDED:
         bEventDisabled = pEventMask->fxo.bits.bat_feeded ? IFX_TRUE :
            IFX_FALSE;
         break;

      case IFX_TAPI_EVENT_FXO_BAT_DROPPED:
         bEventDisabled = pEventMask->fxo.bits.bat_dropped ? IFX_TRUE :
            IFX_FALSE;
         break;

      case IFX_TAPI_EVENT_FXO_OSI:
         bEventDisabled = pEventMask->fxo.bits.osi ? IFX_TRUE : IFX_FALSE;
         break;

      case IFX_TAPI_EVENT_FXO_POLARITY:
         bEventDisabled = pEventMask->fxo.bits.polarity ? IFX_TRUE :
            IFX_FALSE;
         break;

      case IFX_TAPI_EVENT_FXO_APOH:
         bEventDisabled = pEventMask->fxo.bits.apoh ? IFX_TRUE : IFX_FALSE;
         break;

      case IFX_TAPI_EVENT_FXO_NOPOH:
         bEventDisabled = pEventMask->fxo.bits.nopoh ? IFX_TRUE : IFX_FALSE;
         break;

      /* EVENT TYPE: -- IFX_TAPI_EVENT_TYPE_LT -- */

#if (TAPI_CFG_FEATURES & TAPI_FEAT_GR909)
      case IFX_TAPI_EVENT_LT_GR909_RDY:
         pChannel->TapiOpControlData.nLineMode = IFX_TAPI_LINE_FEED_DISABLED;
         bEventDisabled = pEventMask->lt.bits.gr909_rdy ? IFX_TRUE : IFX_FALSE;
         break;
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_GR909) */

      case IFX_TAPI_EVENT_NLT_END:
         bEventDisabled = pEventMask->lt.bits.nlt_end ? IFX_TRUE : IFX_FALSE;
         break;

#if (TAPI_CFG_FEATURES & TAPI_FEAT_PHONE_DETECTION)
      case IFX_TAPI_EVENT_LINE_MEASURE_CAPACITANCE_RDY:
         if (IFX_TAPI_PPD_DisableEvent(pChannel) == IFX_TRUE)
         {
            bEventDisabled = IFX_TRUE;
         }
         else
         {
            bEventDisabled = pEventMask->lcap.bits.lcap_disable ? IFX_TRUE : IFX_FALSE;
         }
         break;

      case IFX_TAPI_EVENT_LINE_MEASURE_CAPACITANCE_RDY_INT:
         if (IFX_NULL != pDrvCtx->ALM.CapMeasResult)
         {
            (IFX_void_t)pDrvCtx->ALM.CapMeasResult(pChannel->pLLChannel);
         }
         bEventDisabled = IFX_TRUE;
         break;

      case IFX_TAPI_EVENT_LINE_MEASURE_CAPACITANCE_START_INT:
         if (IFX_NULL != pDrvCtx->ALM.CapMeasStartInt)
         {
            (IFX_void_t)pDrvCtx->ALM.CapMeasStartInt(pChannel->pLLChannel);
         }
         bEventDisabled = IFX_TRUE;
         break;
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_PHONE_DETECTION) */

      /* EVENT TYPE: -- IFX_TAPI_EVENT_TYPE_PULSE -- */

      case IFX_TAPI_EVENT_PULSE_DIGIT:
         bEventDisabled = pEventMask->pulse.bits.digit ? IFX_TRUE : IFX_FALSE;
         break;

      case IFX_TAPI_EVENT_PULSE_START:
         bEventDisabled = pEventMask->pulse.bits.pulseStart ?
                         IFX_TRUE : IFX_FALSE;
         break;

      /* EVENT TYPE: -- IFX_TAPI_EVENT_TYPE_DTMF -- */

      case IFX_TAPI_EVENT_DTMF_DIGIT:
         {
#if (TAPI_CFG_FEATURES & TAPI_FEAT_CID)
            if (IFX_TAPI_CID_OnDtmfEvent(pChannel, pEvent->data.dtmf.ascii))
            {
               bEventDisabled = IFX_TRUE;
            }
            else
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_CID) */
            {
#if (TAPI_CFG_FEATURES & TAPI_FEAT_DTMF)
               if (TAPI_COD_EXTERNAL (pEvent->data.dtmf))
               {
                  bEventDisabled = pEventMask->dtmf[nModule].bits.digit_external ?
                                  IFX_TRUE : IFX_FALSE;
               }
               else if (TAPI_COD_INTERNAL (pEvent->data.dtmf))
               {
                  bEventDisabled = pEventMask->dtmf[nModule].bits.digit_internal ?
                                  IFX_TRUE : IFX_FALSE;
               }
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_DTMF) */
            }
         }
         break;

#if (TAPI_CFG_FEATURES & TAPI_FEAT_DTMF)
      case IFX_TAPI_EVENT_DTMF_END:
         if (TAPI_COD_INTERNAL (pEvent->data.dtmf))
         {
            bEventDisabled = pEventMask->dtmf[nModule].bits.end_internal ?
                            IFX_TRUE : IFX_FALSE;
         }
         else if (TAPI_COD_EXTERNAL (pEvent->data.dtmf))
         {
            bEventDisabled = pEventMask->dtmf[nModule].bits.end_external ?
                            IFX_TRUE : IFX_FALSE;
         }
         break;
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_DTMF) */

      /* EVENT TYPE: -- IFX_TAPI_EVENT_TYPE_CID -- */

#if (TAPI_CFG_FEATURES & TAPI_FEAT_CID)
      case IFX_TAPI_EVENT_CID_RX_END:
         IFX_TAPI_CID_OnCidRxEndEvent(pChannel);
         bEventDisabled = pEventMask->cid.bits.rx_end ? IFX_TRUE : IFX_FALSE;
         break;

      case IFX_TAPI_EVENT_CID_RX_ERROR_READ:
         bEventDisabled = pEventMask->cid.bits.rx_err_read ?
                         IFX_TRUE : IFX_FALSE;
         break;

      case IFX_TAPI_EVENT_CID_TX_END:
         /* This event is sent by the lower-layer after a data transmission
            has ended. This is here translated into the ...TX_INFO_END or
            ...TX_SEQ_END depending on the mode of the CID transmission. */
         if (TAPI_Cid_UseSequence (pChannel) == IFX_TRUE)
         {
            pEvent->id = IFX_TAPI_EVENT_CID_TX_SEQ_END;
            bEventDisabled = pEventMask->cid.bits.tx_seq_end ?
                            IFX_TRUE : IFX_FALSE;
         }
         else
         {
            pEvent->id = IFX_TAPI_EVENT_CID_TX_INFO_END;
            bEventDisabled = pEventMask->cid.bits.tx_info_end ?
                            IFX_TRUE : IFX_FALSE;
         }
         break;

      case IFX_TAPI_EVENT_CIDSM_END:
         /* stop ringing (including an optional CID transmission) */
         IFX_TAPI_Ring_Stop_Ext(pChannel, IFX_TRUE);

         bEventDisabled = pEventMask->cid.bits.sm_end ?
                           IFX_TRUE : IFX_FALSE;
         break;

      case IFX_TAPI_EVENT_CID_TX_RINGCAD_ERR:
         bEventDisabled = pEventMask->cid.bits.tx_ringcad_err ?
                         IFX_TRUE : IFX_FALSE;
         break;

      case IFX_TAPI_EVENT_CID_TX_NOACK_ERR:
      case IFX_TAPI_EVENT_CID_TX_NOACK2_ERR:
         bEventDisabled = pEventMask->cid.bits.tx_noack_err ?
                         IFX_TRUE : IFX_FALSE;
         break;

      case IFX_TAPI_EVENT_CID_RX_CD:
         bEventDisabled = pEventMask->cid.bits.rx_cd ?
                         IFX_TRUE : IFX_FALSE;
         break;

      case IFX_TAPI_EVENT_CID_TX_INFO_START:
         bEventDisabled = pEventMask->cid.bits.tx_info_start ?
                         IFX_TRUE : IFX_FALSE;
         break;

      case IFX_TAPI_EVENT_CID_TX_SEQ_START:
         bEventDisabled = pEventMask->cid.bits.tx_seq_start ?
                         IFX_TRUE : IFX_FALSE;
         break;
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_CID) */

      /* EVENT TYPE: -- IFX_TAPI_EVENT_TYPE_TONE_GEN -- */

      case IFX_TAPI_EVENT_TONE_GEN_END:
         {
            bEventDisabled = IFX_TRUE;
            /* check event masks */
            if (TAPI_COD_EXTERNAL (pEvent->data.tone_gen) &&
                (!pEventMask->tone_gen.bits.end_external))
               bEventDisabled = IFX_FALSE;

            if (TAPI_COD_INTERNAL (pEvent->data.tone_gen) &&
                (!pEventMask->tone_gen.bits.end_internal))
               bEventDisabled = IFX_FALSE;
         }
         break;

      /* EVENT TYPE: -- IFX_TAPI_EVENT_TYPE_TONE_GEN_RAW -- */
      /*                not available to the application    */
      case IFX_TAPI_EVENT_TONE_GEN_END_RAW:
         {
            TAPI_Tone_Step_Completed (pChannel, (IFX_uint8_t)pEvent->data.value);
            bEventDisabled = IFX_TRUE;
         }
         break;

      /* EVENT TYPE: -- IFX_TAPI_EVENT_TYPE_TONE_DET -- */

      case IFX_TAPI_EVENT_TONE_DET_CPT:
         {
#ifdef TAPI_VERSION3
            /* \note: always use all directions for CPE */
            pEvent->data.tone_det.external = IFX_TRUE;
            pEvent->data.tone_det.internal = IFX_TRUE;
#endif /* TAPI_VERSION3 */
            bEventDisabled = IFX_TRUE;
            /* check event masks */
            if ((pEvent->data.tone_det.external) &&
               (!pEventMask->tone_det[nModule].bits.cpt_external))
               bEventDisabled = IFX_FALSE;
            if ((pEvent->data.tone_det.internal) &&
               (!pEventMask->tone_det[nModule].bits.cpt_internal))
               bEventDisabled = IFX_FALSE;
         }
         break;

      case IFX_TAPI_EVENT_TONE_DET_CPT_END:
         {
#ifdef TAPI_VERSION3
            /* \note: always use all directions for CPE */
            pEvent->data.tone_det.external = IFX_TRUE;
            pEvent->data.tone_det.internal = IFX_TRUE;
#endif /* TAPI_VERSION3 */
            bEventDisabled = IFX_TRUE;
            /* check event masks */
            if ((pEvent->data.tone_det.external) &&
               (!pEventMask->tone_det[nModule].bits.cpt_end_external))
               bEventDisabled = IFX_FALSE;
            if ((pEvent->data.tone_det.internal) &&
               (!pEventMask->tone_det[nModule].bits.cpt_end_internal))
               bEventDisabled = IFX_FALSE;
         }
         break;

      case IFX_TAPI_EVENT_TONE_DET_RECEIVE:
         bEventDisabled = pEventMask->tone_det[nModule].bits.receive ?
                         IFX_TRUE : IFX_FALSE;
         break;
      case IFX_TAPI_EVENT_TONE_DET_TRANSMIT:
         bEventDisabled = pEventMask->tone_det[nModule].bits.transmit ?
                         IFX_TRUE : IFX_FALSE;
         break;
      case IFX_TAPI_EVENT_TONE_DET_MF_R2_START:
         bEventDisabled = pEventMask->mf_r2[nModule].bits.start ?
            IFX_TRUE : IFX_FALSE;
         break;
      case IFX_TAPI_EVENT_TONE_DET_MF_R2_END:
         bEventDisabled = pEventMask->mf_r2[nModule].bits.end ?
            IFX_TRUE : IFX_FALSE;
         break;

      /* EVENT TYPE: -- IFX_TAPI_EVENT_TYPE_FAXMODEM_SIGNAL -- */

      case  IFX_TAPI_EVENT_FAXMODEM_AM:
      case  IFX_TAPI_EVENT_FAXMODEM_CED:
      case  IFX_TAPI_EVENT_FAXMODEM_CEDEND:
      case  IFX_TAPI_EVENT_FAXMODEM_DIS:
      case  IFX_TAPI_EVENT_FAXMODEM_PR:
      case  IFX_TAPI_EVENT_FAXMODEM_CNGFAX:
      case  IFX_TAPI_EVENT_FAXMODEM_CNGMOD:
      case  IFX_TAPI_EVENT_FAXMODEM_V21L:
      case  IFX_TAPI_EVENT_FAXMODEM_V18A:
      case  IFX_TAPI_EVENT_FAXMODEM_V27:
      case  IFX_TAPI_EVENT_FAXMODEM_BELL:
      case  IFX_TAPI_EVENT_FAXMODEM_V22:
      case  IFX_TAPI_EVENT_FAXMODEM_V22ORBELL:
      case  IFX_TAPI_EVENT_FAXMODEM_V32AC:
      case  IFX_TAPI_EVENT_FAXMODEM_V8BIS:
      case  IFX_TAPI_EVENT_FAXMODEM_HOLDEND:
      case IFX_TAPI_EVENT_FAXMODEM_CAS_BELL:
      case IFX_TAPI_EVENT_FAXMODEM_V21H:
      case IFX_TAPI_EVENT_FAXMODEM_VMD:
         bEventDisabled = tapi_FaxModem_Status_Check (pChannel, pEvent);
         break;

      /* EVENT TYPE: -- IFX_TAPI_EVENT_TYPE_CODER -- */

      case IFX_TAPI_EVENT_COD_DEC_CHG:
         bEventDisabled = pEventMask->coder.bits.dec_chg ? IFX_TRUE : IFX_FALSE;
         if ( !bEventDisabled )
         {
            IFX_TAPI_DEC_DETAILS_t dec_details;
            memset (&dec_details, 0, sizeof(IFX_TAPI_DEC_DETAILS_t));
            if (IFX_TAPI_PtrChk (pDrvCtx->COD.DEC_Chg_Evt_Detail_Req))
            {
               ret = pDrvCtx->COD.DEC_Chg_Evt_Detail_Req(pChannel->pLLChannel,
                                                         &dec_details);
               if (ret == TAPI_statusOk)
               {
                  pEvent->data.dec_chg.dec_type        = dec_details.dec_type;
                  pEvent->data.dec_chg.dec_framelength = dec_details.dec_framelength;
               }
            }
         }
         break;

      case IFX_TAPI_EVENT_COD_ROOM_NOISE:
         bEventDisabled = pEventMask->coder.bits.room_noise ?
                         IFX_TRUE : IFX_FALSE;
         break;

      case IFX_TAPI_EVENT_COD_ROOM_SILENCE:
         bEventDisabled = pEventMask->coder.bits.room_silence ?
                         IFX_TRUE : IFX_FALSE;
         break;

#if (TAPI_CFG_FEATURES & TAPI_FEAT_ANNOUNCEMENTS)
      case IFX_TAPI_EVENT_COD_ANNOUNCE_END:
         bEventDisabled = pEventMask->coder.bits.announce ?
                         IFX_TRUE : IFX_FALSE;
         break;
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_ANNOUNCEMENTS) */

      case IFX_TAPI_EVENT_COD_MOS:
         bEventDisabled = pEventMask->coder.bits.mos ?
                         IFX_TRUE : IFX_FALSE;
         break;

      /* EVENT TYPE: -- IFX_TAPI_EVENT_TYPE_RTP -- */

      case IFX_TAPI_EVENT_RTP_FIRST:
         bEventDisabled =
         pEventMask->rtp.bits.first ? IFX_TRUE : IFX_FALSE;
         break;

      case IFX_TAPI_EVENT_RTP_EXT_BROKEN:
         bEventDisabled =
         pEventMask->rtp.bits.broken ? IFX_TRUE : IFX_FALSE;
         break;

      case IFX_TAPI_EVENT_RTP_EXT_SSRC_CHANGED:
         bEventDisabled =
         pEventMask->rtp.bits.changed ? IFX_TRUE : IFX_FALSE;
         break;

      /* EVENT TYPE: -- IFX_TAPI_EVENT_TYPE_AAL -- */

      /* EVENT TYPE: -- IFX_TAPI_EVENT_TYPE_RFC2833 -- */

      case IFX_TAPI_EVENT_RFC2833_EVENT:
         bEventDisabled = pEventMask->rfc2833.bits.event ? IFX_TRUE : IFX_FALSE;
         if ( !bEventDisabled )
         {
            IFX_uint16_t nEvent;
            if (IFX_TAPI_PtrChk (pDrvCtx->SIG.EPOU_Trig_Evt_Detail_Req))
            {
               ret = pDrvCtx->SIG.EPOU_Trig_Evt_Detail_Req(pChannel->pLLChannel,
                                                           &nEvent);
               if (ret == TAPI_statusOk)
               {
                  pEvent->data.rfc2833.event = nEvent;
               }
            }
         }
         break;

      /* EVENT TYPE: -- IFX_TAPI_EVENT_TYPE_KPI -- */

      case IFX_TAPI_EVENT_KPI_INGRESS_FIFO_FULL:
         bEventDisabled = pEventMask->kpi.bits.ingress_fifo_full ?
                         IFX_TRUE : IFX_FALSE;
         break;

      /* EVENT TYPE: -- IFX_TAPI_EVENT_TYPE_T38 -- */

#if (TAPI_CFG_FEATURES & TAPI_FEAT_FAX_T38)
      case IFX_TAPI_EVENT_T38_ERROR_OVLD:
         bEventDisabled = pEventMask->t38.bits.error_ovld ? IFX_TRUE : IFX_FALSE;
         if ( !bEventDisabled)
            TAPI_FaxT38_Event_Update (pChannel, IFX_TAPI_FAX_T38_TX_OFF,
                                      IFX_TAPI_FAX_T38_ERROR_MIPS_OVLD);
         break;

      case IFX_TAPI_EVENT_T38_ERROR_DATA:
         bEventDisabled = pEventMask->t38.bits.error_data ? IFX_TRUE : IFX_FALSE;
         if ( !bEventDisabled )
         {
            if (pEvent->data.value)
               TAPI_FaxT38_Event_Update (pChannel, IFX_TAPI_FAX_T38_TX_OFF,
                                         IFX_TAPI_FAX_T38_ERROR_DATAPUMP);
            else
               TAPI_FaxT38_Event_Update (pChannel, 0,
                                         IFX_TAPI_FAX_T38_ERROR_DATAPUMP);
         }
         break;

      case IFX_TAPI_EVENT_T38_ERROR_WRITE:
         bEventDisabled = pEventMask->t38.bits.error_write ?
                         IFX_TRUE : IFX_FALSE;
         if ( !bEventDisabled )
            TAPI_FaxT38_Event_Update (pChannel, 0, IFX_TAPI_FAX_T38_ERROR_WRITE);
         break;

      case IFX_TAPI_EVENT_T38_ERROR_READ:
         bEventDisabled = pEventMask->t38.bits.error_read ? IFX_TRUE : IFX_FALSE;
         if ( !bEventDisabled )
            TAPI_FaxT38_Event_Update (pChannel, 0, IFX_TAPI_FAX_T38_ERROR_READ);
         break;

      case IFX_TAPI_EVENT_T38_ERROR_SETUP:
         bEventDisabled = pEventMask->t38.bits.error_setup ? IFX_TRUE : IFX_FALSE;
         if ( !bEventDisabled )
            TAPI_FaxT38_Event_Update (pChannel, 0, IFX_TAPI_FAX_T38_ERROR_SETUP);
         break;

      case IFX_TAPI_EVENT_T38_NONE:
         TAPI_FaxT38_Event_Update (pChannel, IFX_TAPI_FAX_T38_TX_OFF,
                                   IFX_TAPI_FAX_T38_ERROR_NONE);
         bEventDisabled = IFX_TRUE;
         break;

      case IFX_TAPI_EVENT_T38_FDP_REQ:
         /* FAX T38 FDP REQ; used for polling mode access only -
            currently not maskable */
         bEventDisabled = IFX_FALSE;
         break;
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_FAX_T38) */

#if (TAPI_CFG_FEATURES & TAPI_FEAT_FAX_STACK)
      case IFX_TAPI_EVENT_T38_STATE_CHANGE:
         bEventDisabled = IFX_FALSE;
         break;
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_FAX_STACK) */

      /* EVENT TYPE: -- IFX_TAPI_EVENT_TYPE_JB -- */

      /* EVENT TYPE: -- IFX_TAPI_EVENT_TYPE_DOWNLOAD -- */

      /* EVENT TYPE: -- IFX_TAPI_EVENT_TYPE_INFORMATION -- */

      case IFX_TAPI_EVENT_INFO_MBX_CONGESTION:
         bEventDisabled = pEventMask->information.bits.mbx_congestion ?
                         IFX_TRUE : IFX_FALSE;
         break;

      /* EVENT TYPE: -- IFX_TAPI_EVENT_TYPE_DEBUG -- */

      case IFX_TAPI_EVENT_DEBUG_CERR:
         if (IFX_TAPI_PtrChk (pDrvCtx->Dbg_CErr_Handler))
         {
            IFX_TAPI_DBG_CERR_t data;
            memset(&data, 0, sizeof(IFX_TAPI_DBG_CERR_t));
            if (TAPI_statusOk ==
                pDrvCtx->Dbg_CErr_Handler(pChannel->pTapiDevice->pLLDev, &data))
            {
               pEvent->data.cerr.fw_id = 0x0001;
               pEvent->data.cerr.reason = data.cause;
               pEvent->data.cerr.command = data.cmd;
            }
         }
         /* print data if any data is available */
         if (pEvent->data.cerr.fw_id != 0x0000)
         {
            TRACE(TAPI_DRV, DBG_LEVEL_HIGH,
                  ("\n!!! cErrHdlr cause 0x%04X, cHdr 0x%08X\n",
                   pEvent->data.cerr.reason, pEvent->data.cerr.command));
         }
         bEventDisabled = pEventMask->debug.bits.cerr ? IFX_TRUE : IFX_FALSE;
         break;

      case IFX_TAPI_EVENT_GPIO_HL:
         bEventDisabled = pEventMask->debug.bits.gpiohl ?
                         IFX_TRUE : IFX_FALSE;
         break;

      case IFX_TAPI_EVENT_GPIO_LH:
         bEventDisabled = pEventMask->debug.bits.gpiolh ?
                         IFX_TRUE : IFX_FALSE;

      /* EVENT TYPE: -- IFX_TAPI_EVENT_TYPE_LL_DRIVER -- */

      case IFX_TAPI_EVENT_LL_DRIVER_WD_FAIL:
         bEventDisabled =
         pEventMask->ll_driver.bits.alive ? IFX_TRUE : IFX_FALSE;
         break;

      /* EVENT TYPE: -- IFX_TAPI_EVENT_TYPE_LIN -- */

      case IFX_TAPI_EVENT_LIN_UNDERFLOW:
         bEventDisabled = pEventMask->lin.bits.buf_underrun ? IFX_TRUE : IFX_FALSE;
         break;

      /* EVENT TYPE: -- IFX_TAPI_EVENT_METERING -- */

#if (TAPI_CFG_FEATURES & TAPI_FEAT_METERING)
      case IFX_TAPI_EVENT_METERING_END:
         IFX_TAPI_Meter_EventServe (pChannel);
         bEventDisabled =
         pEventMask->metering.bits.metering_end ? IFX_TRUE : IFX_FALSE;
         break;
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_METERING) */

      /* EVENT TYPE: -- IFX_TAPI_EVENT_TYPE_CALIBRATION -- */

      case IFX_TAPI_EVENT_CALIBRATION_END:
         pChannel->TapiOpControlData.nLineMode = IFX_TAPI_LINE_FEED_DISABLED;
         bEventDisabled =
            pEventMask->calibration.bits.calibration_end ? IFX_TRUE : IFX_FALSE;
         break;

      case IFX_TAPI_EVENT_CALIBRATION_END_INT:
         if (IFX_TAPI_PtrChk (pDrvCtx->ALM.Calibration_Finish))
         {
            (IFX_void_t)pDrvCtx->ALM.Calibration_Finish(pChannel->pLLChannel);
         }
         bEventDisabled = IFX_TRUE;
         break;

      case IFX_TAPI_EVENT_CALIBRATION_END_SINT:
         if (IFX_TAPI_PtrChk (pDrvCtx->ALM.Calibration_Finish))
         {
            (IFX_void_t)pDrvCtx->ALM.Calibration_Finish(pChannel->pLLChannel);
         }
         bEventDisabled = IFX_TRUE;
         break;

      default:
         TRACE (TAPI_DRV, DBG_LEVEL_HIGH,
                ("TAPI dev%d,ch%d: Unknown event ID: 0x%08x.\n",
                pEvent->dev, pEvent->ch, pEvent->id));
         break;
      }

#if (TAPI_CFG_FEATURES & TAPI_FEAT_PHONE_DETECTION)
      IFX_TAPI_PPD_HandleEvent(pChannel,pEvent);
#endif /* (TAPI_CFG_FEATURES & TAPI_FEAT_PHONE_DETECTION) */
   }

   /* Make a copy of the event for use after the next block in which the
      ownership of the event is transferred. */
   memcpy (&TapiEvent, pEvent, sizeof (TapiEvent));

   if (bEventDisabled == IFX_FALSE)
   {
      /* Event is not disabled so we put it into its respective fifo
         and wake up the application sleeping on read file descriptor */

#ifdef EVENT_COUNTER
      if (pEvent->ch == IFX_TAPI_EVENT_ALL_CHANNELS)
         pEvent->nEventSeqNo = pChannel->pTapiDevice->nEventSeqNo++;
      else
         pEvent->nEventSeqNo = pChannel->nEventSeqNo++;
#endif

      /* Lock the fifo access protection */
      TAPI_OS_MutexGet (&pChannel->pEventHandler->fifoAcc);

      ret = fifoPut (pDispatchFifo, (IFX_void_t *)pParam,
                     sizeof(IFX_TAPI_EXT_EVENT_PARAM_t));

      if (ret == IFX_SUCCESS)
      {
         tapi_WakeUp (pChannel->pTapiDevice);
         /* the event is now owned by the fifo - do not discard it here */
         bEventDiscard = IFX_FALSE;
      }
      else
      {
         IFX_TAPI_EXT_EVENT_PARAM_t *pLostDeferredEvent;

         /* Event fifo queue full. This is not a driver error. The fifo
            contains a limited depth and needs to be read out by the
            application to be emptied. When the fifo gets full every
            consequtive event gets discarded. */

         /* Retrieve the event that was written last into the fifo and
            overwrite it with an fifo overflow event. */
         pLostDeferredEvent = fifoPeekTail (pDispatchFifo, IFX_NULL);
         if (pLostDeferredEvent != IFX_NULL)
         {
            IFX_TAPI_EVENT_t *pLostEvent = &pLostDeferredEvent->tapiEvent;

            if (pLostEvent->id !=
                IFX_TAPI_EVENT_FAULT_GENERAL_EVT_FIFO_OVERFLOW)
            {
               TRACE (TAPI_DRV, DBG_LEVEL_HIGH,
                      ("TAPI dev%d,ch%d: Event 0x%08x lost (first).\n",
                      pLostEvent->dev, pLostEvent->ch, pLostEvent->id));

               /* Overwrite the Event-ID. */
               pLostEvent->id =
                  IFX_TAPI_EVENT_FAULT_GENERAL_EVT_FIFO_OVERFLOW;
               /* We just lost the event that we have overwritten. */
               pLostEvent->data.event_fifo.lost = 1;
            }
            pLostEvent->data.event_fifo.lost++;
         }

         TRACE (TAPI_DRV, DBG_LEVEL_HIGH,
                ("TAPI dev%d,ch%d: Event 0x%08x lost.\n",
                pEvent->dev, pEvent->ch, pEvent->id));

         /* errmsg: Event fifo queue full */
         ret = TAPI_statusEvtFifoFull;
      }

      TAPI_OS_MutexRelease (&pChannel->pEventHandler->fifoAcc);
   }

   if (bEventDiscard == IFX_TRUE)
   {
      tapi_DiscardEvent (pParam);
   }

   /* reset TAPI device on serious failure */
   /* This works on a copy of the event because the ownership of the event
      has been changed just above. */
   switch (TapiEvent.id)
   {
      case IFX_TAPI_EVENT_FAULT_HW_SYNC:
         if (IFX_TAPI_EVENT_FAULT_HW_SYNC_SPI_ABORT != TapiEvent.data.hw_sync)
            break;
         /*lint -fallthrough */
      case IFX_TAPI_EVENT_FAULT_HW_RESET:
      case IFX_TAPI_EVENT_FAULT_FW_EBO_OF:
         IFX_TAPI_DeviceReset (pChannel->pTapiDevice);
         /*lint -fallthrough */
      default:
         break;
   }

   RETURN_STATUS (ret, 0);
}


/**
   Resource allocation and initialisation upon loading the driver.

   This function allocates memory for the buffer pool of events that the
   event dispatcher transports. The buffers contain the event and additional
   fields needed to route and defer the event.
   The pool is shared between all devices of this driver.

   \return
   Returns IFX_ERROR in case of an error, otherwise returns IFX_SUCCESS.
*/
IFX_int32_t IFX_TAPI_Event_On_Driver_Start(IFX_void_t)
{
   /* allocate memory pool for the deferred event structures */
   if (pIFX_TAPI_BP_Deferred_Event == IFX_NULL)
   {
      pIFX_TAPI_BP_Deferred_Event =
         bufferPoolInit (sizeof (IFX_TAPI_EXT_EVENT_PARAM_t),
                         IFX_TAPI_EVENT_POOL_INITIAL_SIZE,
                         IFX_TAPI_EVENT_POOL_GROW_SIZE);
      bufferPoolGrowthLimitSet(pIFX_TAPI_BP_Deferred_Event,
                         IFX_TAPI_EVENT_POOL_GROW_LIMIT);
      bufferPoolIDSet (pIFX_TAPI_BP_Deferred_Event, 10);
   }

   /* initialize buffer pool access protection semaphore */
   if (pIFX_TAPI_BP_Deferred_Event != IFX_NULL)
   {
      TAPI_OS_MutexInit (&semBufferPoolAcc);
   }

   return (pIFX_TAPI_BP_Deferred_Event != IFX_NULL) ? IFX_SUCCESS : IFX_ERROR;
}


/**
   Free resources upon unloading the driver.

   This function frees the memory for the buffer pool of events that the
   event dispatcher transports.
*/
IFX_void_t IFX_TAPI_Event_On_Driver_Stop(IFX_void_t)
{
#ifndef TAPI_LIBRARY
   TAPI_OS_INTSTAT lock;

   TAPI_OS_LOCKINT (lock);
#endif /* TAPI_LIBRARY */

   /* free memory pool for the deferred event structures */
   if (pIFX_TAPI_BP_Deferred_Event != IFX_NULL)
   {
      if (bufferPoolFree (pIFX_TAPI_BP_Deferred_Event) != IFX_SUCCESS)
      {
         TRACE (TAPI_DRV, DBG_LEVEL_HIGH,
                ("WARN: Free deferred-event buffer error.\n"));
      }
      pIFX_TAPI_BP_Deferred_Event = IFX_NULL;
   }

   /* delete the buffer pool access protection semaphore */
   TAPI_OS_MutexDelete (&semBufferPoolAcc);
#ifndef TAPI_LIBRARY
   TAPI_OS_UNLOCKINT(lock);
#endif /* TAPI_LIBRARY */
}


/**
   High Level Event Dispatcher function init.

   \param  pChannel     Pointer to tapi channel structure.

   \return
   - TAPI_statusNoMem No memory for data storage
   - TAPI_statusErr   Initialisiation of FIFOs failed
   - TAPI_statusOk    Initialisation successful
*/
IFX_int32_t IFX_TAPI_EventDispatcher_Init (TAPI_CHANNEL * pChannel)
{
   IFX_TAPI_EVENT_HANDLER_DATA_t *pEventHandlerData;
   IFX_int32_t ret = TAPI_statusOk;

   TAPI_ASSERT(pChannel);

   TAPI_OS_MutexGet (&pChannel->semTapiChDataLock);

   /* allocate data storage on the channel if not already existing */
   if (pChannel->pEventHandler == IFX_NULL)
   {
      /* allocate data storage */
      if ((pChannel->pEventHandler =
           TAPI_OS_Malloc (sizeof(*pChannel->pEventHandler))) == IFX_NULL)
      {
         TAPI_OS_MutexRelease (&pChannel->semTapiChDataLock);
         /* errmsg: Memory not available */
         RETURN_STATUS (TAPI_statusNoMem, 0);
      }
      /* zero the just allocated data storage */
      memset(pChannel->pEventHandler, 0x00, sizeof(*pChannel->pEventHandler));
   }
   /* from here on pChannel->pEventHandler is valid */

   /* Get pointer to data in the channel. */
   pEventHandlerData = pChannel->pEventHandler;

   /* Initialize high and low priority FIFO struct */
   if (pEventHandlerData->pTapiEventFifoHi == IFX_NULL)
   {
      pEventHandlerData->pTapiEventFifoHi = fifoInit(IFX_TAPI_EVENT_FIFO_SIZE);
   }
   if (pEventHandlerData->pTapiEventFifoLo == IFX_NULL)
   {
      pEventHandlerData->pTapiEventFifoLo = fifoInit(IFX_TAPI_EVENT_FIFO_SIZE);
      /* initialize fifo access protection semaphore */
      TAPI_OS_MutexInit (&pEventHandlerData->fifoAcc);
   }

   if ((pEventHandlerData->pTapiEventFifoHi == NULL) ||
       (pEventHandlerData->pTapiEventFifoLo == IFX_NULL))
   {
      /* cleanup previously allocated memory */
      if (pEventHandlerData->pTapiEventFifoHi != NULL)
      {
         fifoFree (pEventHandlerData->pTapiEventFifoHi);
      }
      TAPI_OS_MutexDelete (&pEventHandlerData->fifoAcc);

      TAPI_OS_Free (pChannel->pEventHandler);
      pChannel->pEventHandler = IFX_NULL;

      TAPI_OS_MutexRelease (&pChannel->semTapiChDataLock);
      /* errmsg: Memory not available */
      RETURN_STATUS (TAPI_statusNoMem, 0);
   }

#ifdef TAPI_VERSION4
   /* Set the mask to disable all events. */
   memset(&pEventHandlerData->eventMask, IFX_EVENT_DISABLE * 0xFF,
          sizeof (pEventHandlerData->eventMask));
#endif /* TAPI_VERSION4 */

#ifdef TAPI_VERSION3
   /* Clear the mask to enable all events. */
   memset(&pEventHandlerData->eventMask, IFX_EVENT_ENABLE * 0xFF,
          sizeof (pEventHandlerData->eventMask));

   /* disable events */
   {
      IFX_TAPI_MODULE_TYPE_t nModule = IFX_TAPI_MODULE_TYPE_COD;

      /* disable DTMF end events */
      pEventHandlerData->eventMask.dtmf[nModule].bits.end_external =
         IFX_EVENT_DISABLE;
      pEventHandlerData->eventMask.dtmf[nModule].bits.end_internal =
         IFX_EVENT_DISABLE;

      /* disable Pulse-start event */
      pEventHandlerData->eventMask.pulse.bits.pulseStart = IFX_EVENT_DISABLE;
   }
#endif /* TAPI_VERSION3 */

   TAPI_OS_MutexRelease (&pChannel->semTapiChDataLock);

   /* from here on pChannel->pEventHandler is valid */
   return ret;
}

/**
   High Level Event Dispatcher function exit.

   \param  pChannel     Pointer to tapi channel structure.

   \return IFX_SUCCESS on ok, otherwise IFX_ERROR
*/
IFX_int32_t IFX_TAPI_EventDispatcher_Exit (TAPI_CHANNEL * pChannel)
{
   IFX_TAPI_EVENT_HANDLER_DATA_t *pEventHandlerData;
   IFX_int32_t ret = TAPI_statusOk;
   IFX_void_t *pEvt;

   TAPI_ASSERT(pChannel);

   TAPI_OS_MutexGet (&pChannel->semTapiChDataLock);

   /* Get pointer to data in the channel. */
   pEventHandlerData = pChannel->pEventHandler;

   if (pEventHandlerData != IFX_NULL)
   {
      /* Lock the fifo access protection. */
      TAPI_OS_MutexGet (&pEventHandlerData->fifoAcc);

      /* Silently drop all events stored in high-prio fifo */
      while ((pEvt = fifoGet(pEventHandlerData->pTapiEventFifoHi, NULL)) != NULL)
      {
         bufferPoolPut(pEvt);
      }
      /* Silently drop all events stored in low-prio fifo */
      while ((pEvt = fifoGet(pEventHandlerData->pTapiEventFifoLo, NULL)) != NULL)
      {
         bufferPoolPut(pEvt);
      }

      /* Free the low prio fifo itself */
      if (fifoFree (pEventHandlerData->pTapiEventFifoHi) != IFX_SUCCESS)
      {
         TRACE (TAPI_DRV, DBG_LEVEL_HIGH,
                ("INFO: Free event hi fifo error. ch(%d)\n", pChannel->nChannel));
         ret = IFX_ERROR;
      }
      /* Free the high prio fifo itself */
      if (fifoFree (pEventHandlerData->pTapiEventFifoLo) != IFX_SUCCESS)
      {
         TRACE (TAPI_DRV, DBG_LEVEL_HIGH,
                ("INFO: Free event lo fifo error. ch(%d)\n", pChannel->nChannel));
         ret = IFX_ERROR;
      }

      /* Unlock the protection and delete the semaphore */
      TAPI_OS_MutexRelease (&pEventHandlerData->fifoAcc);
      TAPI_OS_MutexDelete (&pEventHandlerData->fifoAcc);

      /* Free the data storage on the channel */
      TAPI_OS_Free (pChannel->pEventHandler);
      pChannel->pEventHandler = IFX_NULL;
   }

   TAPI_OS_MutexRelease (&pChannel->semTapiChDataLock);

   return ret;
}

/**
   Change the DTMF sending mode of out-of-band (OOB)

   \param   pChannel    Handle to TAPI channel structure.
   \param   nOobMode    Mode of DTMFD (Inband or Out of band transmission
                        of RFC2833 event packets)

   \return
      TAPI_statusOk - success
      TAPI_statusInvalidCh - OOB is unavailable on that channel
      TAPI_statusLLNotSupp - sefice not supported by low-level driver
      TAPI_statusLLFailed - failed low-level call
*/
IFX_int32_t TAPI_EVENT_PKT_EV_Oob_Dtmf_Set (TAPI_CHANNEL *pChannel,
   IFX_uint32_t nOobMode)
{
   IFX_TAPI_DRV_CTX_t *pDrvCtx = pChannel->pTapiDevice->pDevDrvCtx;
   IFX_int32_t retLL;

   /* do not touch anything if LL does not support OOB*/
   if (!IFX_TAPI_PtrChk (pDrvCtx->SIG.DTMFD_OOB))
   {
      /* LEC not supported by LL driver */
      RETURN_STATUS (TAPI_statusLLNotSupp, 0);
   }

   retLL = pDrvCtx->SIG.DTMFD_OOB (pChannel->pLLChannel,
      (IFX_TAPI_PKT_EV_OOB_t)nOobMode);
   if (!TAPI_SUCCESS (retLL))
   {
      RETURN_STATUS (TAPI_statusLLFailed, retLL);
   }

   return TAPI_statusOk;
}

/**
   Configures the transmission of tone signals detected by MFTD.
   Tones can be transmitted inband or out-of-band
   with RFC2833 RTP event packets.

   \param   pChannel    Handle to TAPI channel structure.
   \param   nOobMode    Mode of signal transmission (Inband or Out of band
                        transmission of RFC2833 event packets)

   \return
      TAPI_statusOk - success
      TAPI_statusLLNotSupp - sefice not supported by low-level driver
      TAPI_statusLLFailed - failed low-level call
*/
IFX_int32_t TAPI_EVENT_PKT_EV_Oob_Mftd_Set (TAPI_CHANNEL *pChannel,
   IFX_TAPI_PKT_EV_OOB_MFTD_t const *pOob)
{
   IFX_TAPI_DRV_CTX_t *pDrvCtx = pChannel->pTapiDevice->pDevDrvCtx;
   IFX_int32_t retLL;

   /* do not touch anything if LL does not support OOB*/
   if (!IFX_TAPI_PtrChk (pDrvCtx->SIG.MFTD_OOB))
   {
      /* LEC not supported by LL driver */
      RETURN_STATUS (TAPI_statusLLNotSupp, 0);
   }

   retLL = pDrvCtx->SIG.MFTD_OOB (pChannel->pLLChannel, pOob->nTransmitMode);
   if (!TAPI_SUCCESS (retLL))
   {
      RETURN_STATUS (TAPI_statusLLFailed, retLL);
   }

   return TAPI_statusOk;
}

/**
   Generate events either as tone events played locally or as RFC2833 events

   \param pChannel -Channel context for HL TAPI.
   \param pPacketEvent - Event description.

   \return
   On success TAPI_statusOk or else TAPI_statusErr

   \remarks
   This functions is generic. This is called when user calls through ioctl to
   handle DTMF and also when other module like hapi reports the events to TAPI
*/
IFX_int32_t TAPI_EVENT_PKT_EV_Generate (TAPI_CHANNEL * pChannel,
                                        IFX_TAPI_PKT_EV_GENERATE_t const *pPacketEvent)
{
   IFX_TAPI_DRV_CTX_t *pDrvCtx;
   IFX_int8_t nVolume;

   if ((pChannel->bInitialized == IFX_FALSE) || (pPacketEvent == IFX_NULL))
   {
      return TAPI_statusErr;
   }

   pDrvCtx = pChannel->pTapiDevice->pDevDrvCtx;

   /* Check if Event packet generation function is available */
   if (!IFX_TAPI_PtrChk (pDrvCtx->COD.RTP_EV_Generate))
   {
      TRACE(TAPI_DRV, DBG_LEVEL_HIGH,
         ("%s - COD.RTP_EV_Generate not valid\n", __FUNCTION__));
      return TAPI_statusErr;
   }


   /* Transform enum value to value expected by lower layer. */
   nVolume = (IFX_int8_t)pPacketEvent->volume -
             (IFX_int8_t)IFX_TAPI_PKT_EV_GEN_VOLUME_0_dB;

   /* start/stop generation of rfc 2833 pkt */
   (IFX_void_t)pDrvCtx->COD.RTP_EV_Generate(pChannel->pLLChannel,
                                (IFX_uint8_t)pPacketEvent->event,
                                pPacketEvent->action,
                                pPacketEvent->duration,
                                nVolume);

   return TAPI_statusOk;
}

/**
   Retrieve the overall number of elements of the event wrapper bufferpool.

   \return the overall number of elements
*/
IFX_int32_t IFX_TAPI_EventWrpBufferPool_ElementCountGet(IFX_void_t)
{
#ifndef TAPI_LIBRARY
   TAPI_OS_INTSTAT lock;
#endif /* TAPI_LIBRARY */
   IFX_int32_t  elements;

   if (!TAPI_OS_IN_INTERRUPT())
   {
      TAPI_OS_MutexGet (&semBufferPoolAcc);
   }
   TAPI_OS_LOCKINT (lock);

   elements = bufferPoolSize( pIFX_TAPI_BP_Deferred_Event );

   TAPI_OS_UNLOCKINT(lock);
   if (!TAPI_OS_IN_INTERRUPT())
   {
      TAPI_OS_MutexRelease (&semBufferPoolAcc);
   }

   return elements;
}


/**
   Retrieve the available (free) number of elements of the
   event bufferpool.

   \return the number of available elements
*/
IFX_int32_t IFX_TAPI_EventWrpBufferPool_ElementAvailCountGet(IFX_void_t)
{
#ifndef TAPI_LIBRARY
   TAPI_OS_INTSTAT lock;
#endif /* TAPI_LIBRARY */
   IFX_int32_t  elements;

   if (!TAPI_OS_IN_INTERRUPT())
   {
      TAPI_OS_MutexGet (&semBufferPoolAcc);
   }
   TAPI_OS_LOCKINT (lock);

   elements = bufferPoolAvail( pIFX_TAPI_BP_Deferred_Event );

   TAPI_OS_UNLOCKINT(lock);
   if (!TAPI_OS_IN_INTERRUPT())
   {
      TAPI_OS_MutexRelease (&semBufferPoolAcc);
   }

   return elements;
}

/**
   Generate hook event for testing purpose

   \param   pChannel    Handle to TAPI channel structure.
   \param   nHook       Hook state

   \return
      TAPI_statusOk - success
      TAPI_statusLLNotSupp - sefice not supported by low-level driver
      TAPI_statusLLFailed - failed low-level call
*/
IFX_int32_t TAPI_Test_Hook_Gen (TAPI_CHANNEL *pChannel, IFX_uint32_t nHook)
{
   IFX_TAPI_DRV_CTX_t *pDrvCtx = pChannel->pTapiDevice->pDevDrvCtx;
   IFX_int32_t retLL;

   /* Check for LL driver functionality */
   if (!IFX_TAPI_PtrChk (pDrvCtx->ALM.TestHookGen))
   {
      RETURN_STATUS (TAPI_statusLLNotSupp, 0);
   }

   retLL = pDrvCtx->ALM.TestHookGen (pChannel->pLLChannel,
      (IFX_boolean_t)nHook);
   if (!TAPI_SUCCESS(retLL))
   {
      RETURN_STATUS (TAPI_statusLLFailed, retLL);
   }

   RETURN_STATUS (TAPI_statusOk, 0);
}

/**
   Retrieve event from TAPI device(s)

   \param   pDrvCtx  Handle to the low-level driver context
   \param   pEvent   Handle to the memory for event

   \return
      TAPI_statusOk - success
      TAPI_statusParam - incorrect parameters are passed
*/
IFX_int32_t TAPI_Event_Get (IFX_TAPI_DRV_CTX_t *pDrvCtx, IFX_TAPI_EVENT_t *pEvent)
{
   TAPI_DEV *pTapiDev = IFX_NULL;
   IFX_uint16_t nDevMax;
   IFX_int32_t ret = TAPI_statusOk;

   if (pEvent->dev == IFX_TAPI_EVENT_ALL_DEVICES)
   {
      /*  select first device */
      pTapiDev = pDrvCtx->pTapiDev;
      nDevMax = pDrvCtx->maxDevs;
   }
   else if (pEvent->dev >= pDrvCtx->maxDevs)
   {
      return TAPI_statusParam;
   }
   else
   {
      pTapiDev = pDrvCtx->pTapiDev + pEvent->dev;
      nDevMax = pEvent->dev;
   }

   do /* for all devices */
   {
      /* touch only initialized devices */
      if (pTapiDev->bInitialized == IFX_TRUE)
      {
         ret = tapi_PhoneEventGet (pTapiDev, pEvent);
      }
   }
   while (TAPI_SUCCESS (ret) &&
      (pEvent->id == IFX_TAPI_EVENT_NONE) &&
      ((++pTapiDev) < (pDrvCtx->pTapiDev + nDevMax)));

   return ret;
}

/**
   Enables or disables a event on a given channel. The parameter
   specifies which channel and also the details of the event.

   \param   pDrvCtx     Handle to the low-level driver context
   \param   pEvent      Handle to the event descriptor
   \param   nEvtState   Event reporting state
                        -IFX_EVENT_ENABLE to allow the event to the application
                        -IFX_EVENT_DISABLE to discard the event

   \return TAPI_statusOk on success
*/
static IFX_int32_t tapi_EventStateSet (
   IFX_TAPI_DRV_CTX_t *pDrvCtx,
   IFX_TAPI_EVENT_t *pEvent,
   IFX_uint32_t nEvtState)
{
   TAPI_DEV *pTapiDev = IFX_NULL;
   IFX_uint32_t nDevMax, nCh, nChMin, nChMax;
   IFX_int32_t ret = TAPI_statusOk;

   if (pEvent->dev == IFX_TAPI_EVENT_ALL_DEVICES)
   {
      /*  select first device */
      pTapiDev = pDrvCtx->pTapiDev;
      nDevMax = pDrvCtx->maxDevs;
   }
   else if (pEvent->dev >= pDrvCtx->maxDevs)
   {
      return TAPI_statusParam;
   }
   else
   {
      pTapiDev = pDrvCtx->pTapiDev + pEvent->dev;
      nDevMax = pEvent->dev;
   }

   do /* for all devices */
   {
      /* touch only initialized devices */
      if (pTapiDev->bInitialized)
      {
         if (pEvent->ch == IFX_TAPI_EVENT_ALL_CHANNELS)
         {
            nChMin = 0;
            nChMax = pTapiDev->nMaxChannel;
         }
         else if (pEvent->ch >= pTapiDev->nMaxChannel)
         {
            TRACE(TAPI_DRV, DBG_LEVEL_HIGH,
              ("TAPI: IFX_TAPI_EVENT_EN-/DISABLE called with channel "
               "parameter '%d' out of range for device '%d'. Device parameter "
               "'%d'.\n",
               pEvent->ch, pTapiDev->nDev, pEvent->dev));
            continue;
         }
         else
         {
            nChMin = pEvent->ch;
            nChMax = nChMin + 1;
         }

         for (nCh = nChMin; nCh < nChMax; nCh++)
         {
            /* call function to set event reporting mask according to the
               ioctl on either one or all channels of a device */
            ret = IFX_TAPI_Event_SetMask (pTapiDev->pTapiChanelArray + nCh,
               pEvent, nEvtState);

            if (!TAPI_SUCCESS (ret))
               RETURN_DEVSTATUS (ret, 0);
         }
      }
   }
   while (TAPI_SUCCESS (ret) &&
      ((++pTapiDev) < (pDrvCtx->pTapiDev + nDevMax)));

   return ret;
}

/**
   Enables or disables a event on a given channel. The parameter
   specifies which channel and also the details of the event.

   \param   pDrvCtx     Handle to the low-level driver context
   \param   pEvent      Handle to the event descriptor
   \param   nEvtState   Event reporting state
                        -IFX_EVENT_ENABLE to allow the event to the application
                        -IFX_EVENT_DISABLE to discard the event

   \return TAPI_statusOk on success
*/
static IFX_int32_t TAPI_EventMultiState_Set (IFX_TAPI_DRV_CTX_t *pDrvCtx,
   IFX_TAPI_EVENT_MULTI_t *pMulEvent, IFX_uint32_t nEvtState)
{
   IFX_TAPI_EVENT_t event;
   IFX_uint16_t idx;
   IFX_int32_t ret = TAPI_statusOk;
   IFX_TAPI_EVENT_ENTRY_t *pEvent = IFX_NULL;

   if (pMulEvent->nCount <= 0)
   {
      /* nothing to do for empty list */
      return ret;
   }

#if defined (LINUX) && defined (__KERNEL__)
   pEvent = TAPI_OS_Malloc (pMulEvent->nCount * sizeof (IFX_TAPI_EVENT_ENTRY_t));
   if (!IFX_TAPI_PtrChk (pEvent))
      return TAPI_statusNoMem;

   if (!TAPI_OS_CpyUsr2Kern (pEvent, pMulEvent->pEvent,
         pMulEvent->nCount * sizeof (IFX_TAPI_EVENT_ENTRY_t)))
   {
      TAPI_OS_Free (pEvent);
      return TAPI_statusErrKernCpy;
   }
#else
   pEvent = pMulEvent->pEvent;
#endif /* LINUX && __KERNEL__ */

   memset (&event, 0, sizeof (event));

   event.dev = pMulEvent->dev;
   event.ch = pMulEvent->ch;
#ifdef TAPI_VERSION4
   event.module = pMulEvent->module;
#endif /* TAPI_VERSION4 */

   for (idx = 0;idx < pMulEvent->nCount; idx++)
   {
      event.id = pEvent[idx].id;
      event.data = pEvent[idx].data;

      /* apply single event */
      ret = tapi_EventStateSet (pDrvCtx, &event, nEvtState);

      if (!TAPI_SUCCESS (ret))
      {
         if (pEvent != pMulEvent->pEvent)
            TAPI_OS_Free (pEvent);

         return ret;
      }
   }

   if (pEvent != pMulEvent->pEvent)
      TAPI_OS_Free (pEvent);

   return ret;
}

/**
   Enable a event on a given channel. The parameter
   specifies which channel and also the details of the event.

   \param   pDrvCtx     Handle to the low-level driver context
   \param   pEvent      Handle to the event descriptor

   \return TAPI_statusOk on success
*/
IFX_int32_t IOCTL_TAPI_EventEnable (IFX_TAPI_DRV_CTX_t *pDrvCtx, IFX_TAPI_EVENT_t *pEvent)
{
   return tapi_EventStateSet (pDrvCtx, pEvent, IFX_EVENT_ENABLE);
}

/**
   Disable a event on a given channel. The parameter
   specifies which channel and also the details of the event.

   \param   pDrvCtx     Handle to the low-level driver context
   \param   pEvent      Handle to the event descriptor

   \return TAPI_statusOk on success
*/
IFX_int32_t IOCTL_TAPI_EventDisable (IFX_TAPI_DRV_CTX_t *pDrvCtx, IFX_TAPI_EVENT_t *pEvent)
{
   return tapi_EventStateSet (pDrvCtx, pEvent, IFX_EVENT_DISABLE);
}

/**
   Enable a event on a given channel. The parameter
   specifies which channel and also the details of the event.

   \param   pDrvCtx     Handle to the low-level driver context
   \param   pEvent      Handle to the event descriptor

   \return TAPI_statusOk on success
*/
IFX_int32_t TAPI_EventMultiEnable (IFX_TAPI_DRV_CTX_t *pDrvCtx, IFX_TAPI_EVENT_MULTI_t *pMulEvent)
{
   return TAPI_EventMultiState_Set (pDrvCtx, pMulEvent, IFX_EVENT_ENABLE);
}

/**
   Disable a event on a given channel. The parameter
   specifies which channel and also the details of the event.

   \param   pDrvCtx     Handle to the low-level driver context
   \param   pEvent      Handle to the event descriptor

   \return TAPI_statusOk on success
*/
IFX_int32_t TAPI_EventMultiDisable (IFX_TAPI_DRV_CTX_t *pDrvCtx, IFX_TAPI_EVENT_MULTI_t *pMulEvent)
{
   return TAPI_EventMultiState_Set (pDrvCtx, pMulEvent, IFX_EVENT_DISABLE);
}

/*lint -restore*/
