#ifndef _DRV_TAPI_LINUX_H
#define _DRV_TAPI_LINUX_H
/******************************************************************************

                              Copyright (c) 2009
                            Lantiq Deutschland GmbH
                     Am Campeon 3; 85579 Neubiberg, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

******************************************************************************/

/**
   \file drv_tapi_linux.h
   This file contains the declarations of the linux specific driver functions.
*/

/* ============================= */
/* Global Includes               */
/* ============================= */


#ifndef TAPI_LIBRARY
#ifdef CONFIG_DEVFS_FS
   #include <linux/devfs_fs_kernel.h>
#endif /* CONFIG_DEVFS_FS */
#endif /* !TAPI_LIBRARY */

#ifdef __KERNEL__
   #ifdef LINUX_2_6
      #include <linux/workqueue.h>           /* work_struct */
   #endif /* LINUX_2_6 */
#endif /* __KERNEL__ */

/* ============================= */
/* Global variables              */
/* ============================= */

/* ============================= */
/* Macros & Definitions          */
/* ============================= */

/* ============================= */
/* Global Structures             */
/* ============================= */
#ifdef TAPI_HAVE_TIMERS
   typedef struct Timer_ID_s *Timer_ID;
   typedef IFX_void_t (*TIMER_ENTRY)(Timer_ID timer_id, IFX_ulong_t arg);

   #ifdef __KERNEL__
      #define TIMER_ELEMENT   struct timer_list

      /* Functionpointer to the callbackfunction */
      typedef IFX_void_t (*linux_timer_callback)(IFX_ulong_t);

      /* Timer ID */
      struct Timer_ID_s
      {
      #ifdef LINUX_2_6
         /* !!! important, work struct must be the first element, because we
                need to cast it later on to Timer_ID */
         struct work_struct timerTask;
      #else
         struct tq_struct timerTask;
      #endif /* LINUX_2_6 */
         struct timer_list Timer_List;
         IFX_boolean_t bPeriodical;
         IFX_uint32_t Periodical_Time;
         TIMER_ENTRY pTimerEntry;
         IFX_ulong_t nArgument;
         IFX_boolean_t bStopped;
      };
   #else /* ! __KERNEL__ */
      struct Timer_ID_s
      {
         TIMER_ENTRY pTimerEntry;
         IFX_boolean_t bPeriodical;
         IFX_uint32_t Periodical_Time;
         IFX_ulong_t nArgument;
         /* Timer thread */
         IFXOS_ThreadCtrl_t TimerThread;
         /* Event to wait timer stop call */
         IFXOS_event_t TapiTimerStopEvent;
         /* locking the access to the timer event is needed
            because the event is deleted and initialised to clear the event */
         IFXOS_lock_t LockTimerEvent;
      };
   #endif /* __KERNEL__ */
#endif /* TAPI_HAVE_TIMERS */

/* ============================= */
/* Global function declaration   */
/* ============================= */
#ifndef TAPI_LIBRARY
   extern IFX_int32_t TAPI_OS_ThreadPriorityModify(IFX_uint32_t newPriority);
   extern IFX_void_t  TAPI_OS_ThreadKill(TAPI_OS_ThreadCtrl_t *pThrCntrl);
#endif /* !TAPI_LIBRARY */

#ifdef TAPI_HAVE_TIMERS
   extern Timer_ID          TAPI_Create_Timer     (TIMER_ENTRY pTimerEntry, IFX_ulong_t nArgument);
   extern IFX_boolean_t     TAPI_SetTime_Timer    (Timer_ID Timer, IFX_uint32_t nTime, IFX_boolean_t bPeriodically, IFX_boolean_t bRestart);
   extern IFX_boolean_t     TAPI_Delete_Timer     (Timer_ID Timer);
   extern IFX_boolean_t     TAPI_Stop_Timer       (Timer_ID Timer);
#endif /* TAPI_HAVE_TIMERS */

#endif /*_DRV_TAPI_LINUX_H */
