#ifndef _DRV_TAPI_OSMAP_H
#define _DRV_TAPI_OSMAP_H
/******************************************************************************

                              Copyright (c) 2009
                            Lantiq Deutschland GmbH
                     Am Campeon 3; 85579 Neubiberg, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

*******************************************************************************/

/**
   \file drv_tapi_osmap.h
   This file contains the includes and the defines specific to the OS.
*/

#include <ifx_types.h>     /* ifx type definitions */

#ifndef HAVE_IFX_ULONG_T
   #warning please update your ifx_types.h, using local definition of IFX_ulong_t
   /* unsigned long type - valid for 32bit systems only */
   typedef unsigned long               IFX_ulong_t;
   #define HAVE_IFX_ULONG_T
#endif /* HAVE_IFX_ULONG_T */

#ifndef HAVE_IFX_LONG_T
   #warning please update your ifx_types.h, using local definition of IFX_long_t
   /* long type - valid for 32bit systems only */
   typedef long                        IFX_long_t;
   #define HAVE_IFX_LONG_T
#endif /* HAVE_IFX_LONG_T */

#ifndef HAVE_IFX_INTPTR_T
   #warning please update your ifx_types.h, using local definition of IFX_intptr_t
   typedef IFX_long_t                  IFX_intptr_t;
   #define HAVE_IFX_INTPTR_T
#endif /* HAVE_IFX_INTPTR_T */

#ifndef HAVE_IFX_SIZE_T
   #warning please update your ifx_types.h, using local definition of IFX_size_t
   typedef IFX_ulong_t                 IFX_size_t;
   #define HAVE_IFX_SIZE_T
#endif /* HAVE_IFX_SIZE_T */

#ifndef HAVE_IFX_SSIZE_T
   #warning please update your ifx_types.h, using local definition of IFX_ssize_t
   typedef IFX_long_t                  IFX_ssize_t;
   #define HAVE_IFX_SSIZE_T
#endif /* HAVE_IFX_SSIZE_T */

#ifndef TAPI_LIBRARY
   #include <ifxos_interrupt.h>
   #ifdef VXWORKS
      /** Determine if the current state is in interrupt or task context. */
      #define TAPI_OS_IN_INTERRUPT() \
         ((intContext() == TRUE) ? IFX_TRUE : IFX_FALSE)
   #endif
   #if defined (LINUX) && defined (__KERNEL__)
      #include <linux/irq.h>
      #include <linux/threads.h>
      #include <linux/interrupt.h>           /* in_interrupt() */
      #include <asm/poll.h>                  /* POLLIN, POLLOUT */

      /** Definitions for select queues. */
      /** Returned by select() to indicate that the fd is ready to writing. */
      #define  TAPI_OS_SYSWRITE  POLLOUT
      /** Returned by select() to indicate that the fd is ready for reading. */
      #define  TAPI_OS_SYSREAD   POLLIN

      /** Determine if the current state is in interrupt or task context. */
      #define TAPI_OS_IN_INTERRUPT() \
         /*lint -save -e{155, 506, 774} */    \
         (in_interrupt() ? IFX_TRUE : IFX_FALSE) \
         /*lint -restore */
      /** lock/take the mutex but be interruptable by signals.
          Difference to TAPI_OS_MutexGet is that the sleep is also interrupted
          when a signal is received by the sleeping thread.
      \param mutex - mutex handle
      */
      #define TAPI_OS_LOCK_GET_INTERRUPTIBLE(lockId)     \
         /*lint -save -e{155, 506, 774} */    \
         (down_interruptible(&(lockId)->object)) \
         /*lint -restore */

   #endif /* LINUX */
   #ifdef WINDOWS
      /** Determine if the current state is in interrupt or task context. */
      #define TAPI_OS_IN_INTERRUPT() IFX_FALSE
      /*lint -restore */
   #endif
#else /* TAPI_LIBRARY */
   #include <ifxos_std_defs.h>

   /** Determine if the current state is in interrupt or task context. */
   #define TAPI_OS_IN_INTERRUPT() IFX_FALSE
   #define IFXOS_LOCKINT(var)
   #define IFXOS_UNLOCKINT(var)
   #define IFXOS_IRQ_ENABLE(irq)
   #define IFXOS_IRQ_DISABLE(irq)
   typedef unsigned long IFXOS_INTSTAT;
#endif /* TAPI_LIBRARY */

#include <ifxos_memory_alloc.h>
#include <ifxos_copy_user_space.h>
#include <ifxos_debug.h> /* debug features */
#include <ifxos_event.h>
#include <ifxos_select.h>
#include <ifxos_mutex.h>
#include <ifxos_lock.h>
#include <ifxos_time.h>
#include <ifx_fifo.h>  /* fifo (used for streaming) */
#include <ifxos_thread.h>

/* ============================= */
/* Global Defines                */
/* ============================= */
#define DBG_LEVEL_OFF      4
#define DBG_LEVEL_HIGH     3
#define DBG_LEVEL_NORMAL   2
#define DBG_LEVEL_LOW      1

/** Define the used CR/LF sequence */
#define TAPI_CRLF      IFXOS_CRLF

/*lint -save -esym(773,SetTraceLevel) -esym(773,CREATE_TRACE_GROUP) */
#define CREATE_TRACE_GROUP(name) \
   unsigned int G_nTraceGroup##name = DBG_LEVEL_HIGH

#define DECLARE_TRACE_GROUP(name) extern unsigned int G_nTraceGroup##name
#define PRINTF IFXOS_PRINT_USR_RAW /* defined in lib_ifxos */
#define TRACE(name,level,message) do {if(level >= G_nTraceGroup##name) \
      { PRINTF message ; } } while(0)
#define SetTraceLevel(name, level) G_nTraceGroup##name = (level)
/*lint -restore */

#ifdef DEBUG
/** assert in debug code
\param expr - expression to be evaluated. If expr != TRUE assert is printed
              out with line number */
#define TAPI_ASSERT(expr) \
   do { \
      /*lint -e{506, 774} */ \
      if(!(expr)) { \
         (void) IFXOS_DBG_PRINT_USR ("\n\r" __FILE__ ":%d: Assertion %s " \
                                     "failed!\n\r",__LINE__, #expr); \
      } \
   } while (0)
#else /* DEBUG */
/** assert in debug code
\param expr - expression to be evaluated. If expr != TRUE assert is printed
              out with line number */
#define TAPI_ASSERT(expr)
#endif /* DEBUG */


/* ============================= */
/* Name mapping tables           */
/* ============================= */

#ifdef VXWORKS
   #ifndef GetImmr
      #define GetImmr()                   vxImmrGet()
   #endif
   #define IMAP_ADDR                      vxImmrGet()

   /** Definitions for select queues. */
   /** Returned by select() to indicate that the fd is ready to writing. */
   #define TAPI_OS_SYSWRITE               0x00000002
   /** Returned by select() to indicate that the fd is ready for reading. */
   #define TAPI_OS_SYSREAD                0x00000001

   #ifndef TAPI_OS_ThreadPriorityModify
      #define TAPI_OS_ThreadPriorityModify(prio) \
            taskPrioritySet(taskIdSelf(), prio)
   #endif /* TAPI_OS_ThreadPriorityModify */

   #define TAPI_OS_ThreadKill(pThrCntrl)

   /** lock/take the mutex but be interruptable by signals.
       Difference to TAPI_OS_MutexGet is that the sleep is also interrupted when
       a signal is received by the sleeping thread.
       This concept is not available on VxWorks. So the normal get() is used.
   \param mutex - mutex handle
   */
   #define TAPI_OS_LOCK_GET_INTERRUPTIBLE    TAPI_OS_LockGet
#endif /* VXWORKS */

#ifdef WINDOWS
   #include <sys_tickedtimer.h>
   /** Definitions for select queues. */
   /** Returned by select() to indicate that the fd is ready to writing. */
   #define  TAPI_OS_SYSWRITE                 0x00000002
   /** Returned by select() to indicate that the fd is ready for reading. */
   #define  TAPI_OS_SYSREAD                  0x00000001

   /** lock/take the mutex but be interruptable by signals.
       Difference to TAPI_OS_MutexGet is that the sleep is also interrupted when
       a signal is received by the sleeping thread.
       This concept is not available on Windows. So normal get is used.
   \param mutex - mutex handle
   */
   #define TAPI_OS_LOCK_GET_INTERRUPTIBLE    TAPI_OS_LockGet

   #define TAPI_OS_ThreadKill(pThrCntrl)

   #define TAPI_OS_ThreadPriorityModify      IFXOS_ThreadPriorityModify

#endif /* WINDOWS */

/*
   Mapping table - Dynamic memory handling.
*/
#if defined(LINUX) && defined(__KERNEL__)
#define TAPI_OS_Malloc                    IFXOS_BlockAlloc
#define TAPI_OS_Free                      IFXOS_BlockFree
#else
#define TAPI_OS_Malloc                    IFXOS_MemAlloc
#define TAPI_OS_Free                      IFXOS_MemFree
#endif

/*
   Mapping table - Kernel-space / User-space data exchange.
*/
#if defined(LINUX) && !defined(__KERNEL__)
#define TAPI_OS_CpyKern2Usr(pTo,pFrom,size) \
      (memcpy(pTo,pFrom,size)!=IFX_NULL)
#define TAPI_OS_CpyUsr2Kern(pTo,pFrom,size) \
      (memcpy(pTo,pFrom,size)!=IFX_NULL)
#else
#define TAPI_OS_CpyKern2Usr               IFXOS_CpyToUser
#define TAPI_OS_CpyUsr2Kern               IFXOS_CpyFromUser
#endif /* defined(LINUX) && !defined(__KERNEL__) */

/*
   Mapping table - Mutex handling.
*/
#define TAPI_OS_mutex_t                   IFXOS_mutex_t
#define TAPI_OS_MutexInit                 IFXOS_MutexInit
#define TAPI_OS_MutexDelete               IFXOS_MutexDelete
#define TAPI_OS_MutexGet                  IFXOS_MutexGet
#define TAPI_OS_MutexRelease              IFXOS_MutexRelease

/*
   Mapping table - Lock handling.
*/
#define TAPI_OS_lock_t                    IFXOS_lock_t
#define TAPI_OS_LockInit                  IFXOS_LockInit
#define TAPI_OS_LockGet                   IFXOS_LockGet
#define TAPI_OS_LockRelease               IFXOS_LockRelease
#define TAPI_OS_LockDelete                IFXOS_LockDelete

/*
   Mapping table - Interrupt handling.
*/
#define TAPI_OS_INTSTAT                   IFXOS_INTSTAT
#define TAPI_OS_LOCKINT                   IFXOS_LOCKINT
#define TAPI_OS_UNLOCKINT                 IFXOS_UNLOCKINT

/*
   Mapping table - Event signalling.
*/
#define TAPI_OS_event_t                   IFXOS_event_t
#define TAPI_OS_EventInit                 IFXOS_EventInit
#define TAPI_OS_EventWakeUp               IFXOS_EventWakeUp
#define TAPI_OS_EventWait                 IFXOS_EventWait
#define TAPI_OS_EventDelete               IFXOS_EventDelete
#define TAPI_OS_WAIT_FOREVER              IFXOS_WAIT_FOREVER

/*
   Mapping table - Select handling.
*/
#define TAPI_OS_drvSelectQueue_t          IFXOS_drvSelectQueue_t
#define TAPI_OS_drvSelectTable_t          IFXOS_drvSelectTable_t
#define TAPI_OS_drvSelectOSArg_t          IFXOS_drvSelectOSArg_t
#ifdef TAPI_LIBRARY
#define TAPI_OS_DrvSelectQueueInit(pEventId) while(0)
#define TAPI_OS_DrvSelectQueueWakeUp(pDrvSelectQueue,drvSelType) while(0)
#else
#define TAPI_OS_DrvSelectQueueInit        IFXOS_DrvSelectQueueInit
#define TAPI_OS_DrvSelectQueueWakeUp      IFXOS_DrvSelectQueueWakeUp
#endif /* TAPI_LIBRARY */
#define TAPI_OS_DRV_SEL_WAKEUP_TYPE_RD    IFXOS_DRV_SEL_WAKEUP_TYPE_RD
#define TAPI_OS_DrvSelectQueueAddTask     IFXOS_DrvSelectQueueAddTask
#define TAPI_OS_DrvSelectQueueDelete(param) /* does not exist */

#ifdef LINUX
#ifndef IFXOS_THREAD_PRIO_HIGH
#define IFXOS_THREAD_PRIO_HIGH                     60
#endif
#ifndef IFXOS_THREAD_PRIO_HIGHEST
#define IFXOS_THREAD_PRIO_HIGHEST                  80
#endif
#endif
/*
   Mapping table - Thread handling.
*/
#define TAPI_OS_THREAD_PRIO_HIGH          IFXOS_THREAD_PRIO_HIGH
#define TAPI_OS_THREAD_PRIO_HIGHEST       IFXOS_THREAD_PRIO_HIGHEST
#define TAPI_OS_ThreadCtrl_t              IFXOS_ThreadCtrl_t
#define TAPI_OS_ThreadParams_t            IFXOS_ThreadParams_t
#define TAPI_OS_ThreadInit                IFXOS_ThreadInit
#define TAPI_OS_THREAD_PRIORITY_MODIFY    TAPI_OS_ThreadPriorityModify
#define TAPI_OS_THREAD_KILL               TAPI_OS_ThreadKill
#define TAPI_OS_ThreadDelete              IFXOS_ThreadDelete
#define TAPI_OS_THREAD_INIT_VALID         IFXOS_THREAD_INIT_VALID

/*
   Mapping table - Time and Waiting.
*/
#define TAPI_OS_MSecSleep                 IFXOS_MSecSleep

/*
   Function map - IOCTL Handling
*/
/** Returns the magic number of IOCTL command
\param iocmd - ioctl command of which magic number is decoded */
#define TAPI_IOC_MAGIC(cmd)   (_IOC_TYPE(cmd))

/** Returns the write request of IOCTL command
\param iocmd - ioctl command */
#define TAPI_IOC_WRITE(cmd)   ((_IOC_DIR(cmd) & _IOC_WRITE) ? IFX_TRUE : IFX_FALSE)

/** Returns the read request of IOCTL command
\param iocmd - ioctl command */
#define TAPI_IOC_READ(cmd)    ((_IOC_DIR(cmd) & _IOC_READ) ? IFX_TRUE : IFX_FALSE)

/** Returns the IOCTL command number
\param iocmd - ioctl command */
#define TAPI_IOC_IDX(cmd)     (_IOC_NR(cmd))

/** Returns the argument size of IOCTL command
\param iocmd - ioctl command */
#define TAPI_IOC_SIZE(cmd)    (_IOC_SIZE(cmd))

/* To make EventLogger happy */
#define IFXOS_GET_TICK()                  IFXOS_ElapsedTimeMSecGet(0)

/*
   Function map - Time and Wait Functions and Defines.
*/
#define TAPI_OS_USecSleep                 IFXOS_USecSleep

#ifdef LINUX
#include "drv_tapi_linux.h"
#endif
#ifdef VXWORKS
#include "drv_tapi_vxworks.h"
#endif
#ifdef WINDOWS
#include "drv_tapi_win.h"
#endif
#endif /* _DRV_TAPI_OSMAP_H */
